; This file is part of Magical Fairy Force.

; Magical Fairy Force is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Magical Fairy Force is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Magical Fairy Force.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2020 Average Software.

; 7ZC-BX character data and logic routines

; 7ZC-BX's attack flow:
; A targetting crosshair will start in the
; center of the opponents screen.  This
; crosshair will begin following the enemy
; around the playfield.

; After 60 frames, the crosshair will stop, turn
; red for 15 frames, and then two damaging lasers
; will intersect the cross hair for 30 frames.

; This will happen 3 times.

; Attack constants.
seven_attack_cost       equ -max_fairy_power

; Attack phase constants.
seven_chase_phase       equ 0
seven_ready_phase       equ 1
seven_fire_phase        equ 2

; Phase length constants.
seven_chase_length      equ 60
seven_ready_length      equ 15
seven_fire_length       equ 30
seven_strobe_length     equ 4
seven_total_cycles      equ 2

; Color constants.
seven_chase_color       equ $0f
seven_ready_color       equ $38

; Attack block data offsets.
seven_tick_count        equ 0
seven_attack_phase      equ 1
seven_crosshair_base    equ 2
seven_aim_row           equ 4
seven_aim_column        equ 6
seven_crosshair_x       equ 8
seven_crosshair_y       equ 9
seven_crosshair_x_sub   equ 10
seven_strobe_count      equ 11
seven_strobe_phase      equ 12
seven_cycle_count       equ 13
seven_sound_ticks       equ 14

seven_start_attack subroutine
    beq     .attack_player_0
    ldx     #seven_attack_cost
    jsr     general_player_0_attack_start
    mwa     $221b,p0_attack_data + seven_crosshair_base
    mva     #$3b,p0_attack_data + seven_crosshair_y
    mwa     $2013,p0_attack_data + seven_aim_column
    mwa     $2208,p0_attack_data + seven_aim_row
    ldy     #seven_chase_color
    sty     p1_playfield_0_color
    mwa     p0_attack_data,$fc
    jmp     .init_attack_data
.attack_player_0
    ldx     #seven_attack_cost
    jsr     general_player_1_attack_start
    mwa     $36cb,p1_attack_data + seven_crosshair_base
    mva     #$b0,p1_attack_data + seven_crosshair_y
    mwa     $3153,p1_attack_data + seven_aim_column
    mwa     $36b8,p1_attack_data + seven_aim_row
    ldy     #seven_chase_color
    sty     p0_playfield_0_color
    mwa     p1_attack_data,$fc
.init_attack_data
    jsr     seven_init_attack_data
    jmp     start_attack_return

seven_process_attack subroutine
    bne     seven_attack_player_1
    jmp     seven_attack_player_0

seven_attack_player_1 subroutine
    ldy     #2                      ; Suppress cloud spawns.
    sty     p1_cloud_timer
    lda     p0_attack_data + seven_attack_phase
    beq     .chase_phase
    cmp     #seven_ready_phase
    beq     .ready_phase
    dec     p0_attack_data + seven_tick_count
    bne     .process_fire
    lda     #seven_chase_color
    sta     p1_playfield_0_color
    mwa     p0_attack_data,$fc
    jsr     seven_clear_attack
    beq     .not_done
    jmp     .done
.not_done
    jmp     end_player_0_attack
.process_fire
    lda     pf_high                 ; Check for hits.
    beq     .no_hit
    jsr     damage_player_1
.no_hit
    dec     p0_attack_data + seven_strobe_count
    beq     .not_done_2
    jmp     .done
.not_done_2
    mva     #seven_strobe_length,p0_attack_data + seven_strobe_count
    lda     p0_attack_data + seven_strobe_phase
    eor     #1
    sta     p0_attack_data + seven_strobe_phase
    beq     .toggle_color_1
    lda     #seven_chase_color
    .byte   $2c                     ; bit instruction skip trick.
.toggle_color_1
    lda     #seven_ready_color
    sta     p1_playfield_0_color
    jmp     p0_process_return
.ready_phase
    dec     p0_attack_data + seven_tick_count
    bne     .done
    lda     #seven_fire_length
    sta     p0_attack_data + seven_tick_count
    inc     p0_attack_data + seven_attack_phase
    mwa     p0_attack_data,$fc
    jsr     seven_prepare_fire
    mwa     seven_laser_fire,$fe
    jsr     play_sound_3
    jmp     p0_process_return
.chase_phase
    dec     p0_attack_data + seven_tick_count
    bne     .process_chase
    lda     #seven_ready_length
    sta     p0_attack_data + seven_tick_count
    inc     p0_attack_data + seven_attack_phase
    lda     #seven_ready_color
    sta     p1_playfield_0_color
    mwa     seven_lock_beep,$fe
    jsr     play_sound_3
    jmp     p0_process_return
.process_chase
    mwa     p0_attack_data,$fc
    ldx     p1_real_hpos
    ldy     p1_vpos
    jsr     seven_chase_player
    dec     p0_attack_data + seven_sound_ticks
    bne     .done
    lda     #20
    sta     p0_attack_data + seven_sound_ticks
    mwa     seven_seek_beep,$fe
    jsr     play_sound_3
.done
    jmp     p0_process_return

seven_attack_player_0 subroutine
    lda     #2                      ; Suppress cloud spawns.
    sta     p0_cloud_timer
    lda     p1_attack_data + seven_attack_phase
    beq     .chase_phase
    cmp     #seven_ready_phase
    beq     .ready_phase
    dec     p1_attack_data + seven_tick_count
    bne     .process_fire
    lda     #seven_chase_color
    sta     p0_playfield_0_color
    mwa     p1_attack_data,$fc
    jsr     seven_clear_attack
    beq     .not_done
    jmp     .done
.not_done
    jmp     end_player_1_attack
.process_fire
    lda     pf_low                  ; Check for hits.
    beq     .no_hit
    jsr     damage_player_0
.no_hit
    dec     p1_attack_data + seven_strobe_count
    beq     .not_done_2
    jmp     .done
.not_done_2
    mva     #seven_strobe_length,p1_attack_data + seven_strobe_count
    lda     p1_attack_data + seven_strobe_phase
    eor     #1
    sta     p1_attack_data + seven_strobe_phase
    beq     .toggle_color_1
    lda     #seven_chase_color
    .byte   $2c                     ; bit instruction skip trick.
.toggle_color_1
    lda     #seven_ready_color
    sta     p0_playfield_0_color
    jmp     p1_process_return
.ready_phase
    dec     p1_attack_data + seven_tick_count
    bne     .done
    lda     #seven_fire_length
    sta     p1_attack_data + seven_tick_count
    inc     p1_attack_data + seven_attack_phase
    mwa     p1_attack_data,$fc
    jsr     seven_prepare_fire
    mwa     seven_laser_fire,$fe
    jsr     play_sound_4
    jmp     p1_process_return
.chase_phase
    dec     p1_attack_data + seven_tick_count
    bne     .process_chase
    lda     #seven_ready_length
    sta     p1_attack_data + seven_tick_count
    inc     p1_attack_data + seven_attack_phase
    lda     #seven_ready_color
    sta     p0_playfield_0_color
    mwa     seven_lock_beep,$fe
    jsr     play_sound_3
    jmp     p1_process_return
.process_chase
    mwa     p1_attack_data,$fc
    ldx     p0_real_hpos
    ldy     p0_vpos
    jsr     seven_chase_player
    dec     p1_attack_data + seven_sound_ticks
    bne     .done
    lda     #20
    sta     p1_attack_data + seven_sound_ticks
    mwa     seven_seek_beep,$fe
    jsr     play_sound_4
.done
    jmp     p1_process_return

; Initialize the attack block at $fc-$fd.
; Expects the crosshair address to be set
; previously so the crosshair can be drawn.
seven_init_attack_data subroutine
    ldy     #seven_tick_count
    lda     #seven_chase_length
    sta     ($fc),y
    iny
    lda     #seven_chase_phase
    sta     ($fc),y
    iny
    lda     ($fc),y                 ; Crosshair address.
    sta     $fe
    iny
    lda     ($fc),y
    sta     $ff
    ldy     #seven_crosshair_x
    lda     #124
    sta     ($fc),y
    ldy     #seven_crosshair_x_sub
    lda     #0
    sta     ($fc),y
    ldy     #seven_cycle_count
    lda     #seven_total_cycles
    sta     ($fc),y
    ldy     #seven_sound_ticks
    lda     #1
    sta     ($fc),y
    ; Fall through into seven_draw_crosshair.

; Draw the crosshair at the address in $fe-$ff.
seven_draw_crosshair subroutine
    ldx     #0                      ; Index into the image.
.loop
    ldy     #0                      ; Index into the target.
    lda     seven_crosshair,x
    sta     ($fe),y
    inx
    iny
    lda     seven_crosshair,x
    sta     ($fe),y
    jsr     next_playfield_row
    cpx     #31                     ; Copy 32 bytes.
    beq     .done
    inx
    jmp     .loop
.done
    rts

; Erase the crosshair at the address in $fe-$ff.
seven_erase_crosshair subroutine
    ldx     #16                     ; Erase 16 rows.
    ldy     #0
.loop
    tya                             ; Get 0 from y.
    sta     ($fe),y
    iny
    sta     ($fe),y
    dex
    beq     .done
    jsr     next_playfield_row
    dey
    jmp     .loop
.done
    rts

; Pursue the player with the crosshair.
; Expects the player's data block in
; $fc-$fd and the target's x, y position
; in x and y.
seven_chase_player subroutine
    lda     #0                      ; Track if movement is needed.
    sta     $f9
    sty     $fb                     ; Save the target y positions.
    ldy     #seven_crosshair_base   ; Get the old address for erasure.
    lda     ($fc),y
    sta     $fe
    iny
    lda     ($fc),y
    sta     $ff
    ldy     #seven_crosshair_x      ; Check the left-right first.
    txa
    cmp     ($fc),y
    beq     .x_move_done
    bcc     .move_left
    lda     ($fc),y                 ; Update the x position.
    adc     #0                      ; Carry is set.
    sta     ($fc),y
    ldy     #seven_crosshair_x_sub  ; Update the subpixel, move right
    lda     ($fc),y                 ; when the subpixel hits 4.
    cmp     #3
    beq     .move_right_full
    clc
    adc     #1                      ; Adjust the subpixel.
    sta     ($fc),y
    jmp     .x_move_done
.move_right_full
    lda     #0                      ; Reset the subpixel.
    sta     ($fc),y
    sty     $f9                     ; Request movement.
    clc                             ; Update the address.
    ldy     #seven_crosshair_base
    lda     ($fc),y
    adc     #1
    sta     ($fc),y
    iny
    lda     ($fc),y
    adc     #0
    sta     ($fc),y
    ldy     #seven_aim_column       ; Update the aim column.
    lda     ($fc),y
    clc
    adc     #1
    sta     ($fc),y
    iny
    lda     ($fc),y
    adc     #0
    sta     ($fc),y
    jmp     .x_move_done
.move_left
    lda     ($fc),y                 ; Update the x position.
    sbc     #0                      ; Carry is clear.
    sta     ($fc),y
    ldy     #seven_crosshair_x_sub  ; Update the subpixel, move left
    lda     ($fc),y                 ; when the subpixel hits -1.
    beq     .move_left_full
    sec
    sbc     #1                      ; Carry is clear here.
    sta     ($fc),y
    jmp     .x_move_done
.move_left_full
    lda     #3                      ; Reset the subpixel.
    sta     ($fc),y
    sta     $f9                     ; Request movement.
    sec                             ; Update the address.
    ldy     #seven_crosshair_base
    lda     ($fc),y
    sbc     #1
    sta     ($fc),y
    iny
    lda     ($fc),y
    sbc     #0
    sta     ($fc),y
    ldy     #seven_aim_column       ; Update the aim column.
    lda     ($fc),y
    sec
    sbc     #1
    sta     ($fc),y
    iny
    lda     ($fc),y
    sbc     #0
    sta     ($fc),y
.x_move_done
    ldy     #seven_crosshair_y      ; Check the up-down now.
    lda     ($fc),y
    cmp     $fb
    beq     .y_move_done
    bcs     .move_up
    adc     #1                      ; Update the y posiiton.
    sta     ($fc),y                 ; Carry is clear.
    sty     $f9                     ; Request movement.
    ldy     #seven_crosshair_base   ; Move down one row.
    lda     ($fc),y
    adc     #40                     ; Carry is clear.
    sta     ($fc),y
    iny
    lda     ($fc),y
    adc     #0
    sta     ($fc),y
    ldy     #seven_aim_row          ; Update the aim row.
    lda     ($fc),y
    clc
    adc     #40
    sta     ($fc),y
    iny
    lda     ($fc),y
    adc     #0
    sta     ($fc),y
    jmp     .y_move_done
.move_up
    sbc     #1                      ; Update the y position.
    sta     ($fc),y                 ; Carry is set.
    sty     $f9                     ; Request movement.
    ldy     #seven_crosshair_base   ; Move up one row.
    lda     ($fc),y
    sbc     #40                     ; Carry is set.
    sta     ($fc),y
    iny
    lda     ($fc),y
    sbc     #0
    sta     ($fc),y
    ldy     #seven_aim_row          ; Update the aim row.
    lda     ($fc),y
    sec
    sbc     #40
    sta     ($fc),y
    iny
    lda     ($fc),y
    sbc     #0
    sta     ($fc),y
.y_move_done
    lda     $f9                     ; See if a move is needed.
    beq     .no_move
    jsr     seven_erase_crosshair   ; Blank the previous crosshair.
    ldy     #seven_crosshair_base
    lda     ($fc),y                 ; Get the address of the new one.
    sta     $fe
    iny
    lda     ($fc),y
    sta     $ff
    jsr     seven_draw_crosshair
.no_move
    rts

; Prepare the laser effect.  Expects the player's
; attack block address in $fc-$fd.
seven_prepare_fire subroutine
    ldx     #16                     ; Draw 16 rows of laser.
    ldy     #seven_aim_row
    lda     ($fc),y
    sta     $fe
    iny
    lda     ($fc),y
    sta     $ff
.laser_h_loop
    lda     #$55
    jsr     set_playfield_row
    jsr     next_playfield_row
    dex
    bne     .laser_h_loop
    ldx     #69                     ; Draw 69 bytes in the column.
    ldy     #seven_aim_column
    lda     ($fc),y
    sta     $fe
    iny
    lda     ($fc),y
    sta     $ff
.laser_v_loop
    lda     #$55
    ldy     #0
    sta     ($fe),y
    iny
    sta     ($fe),y
    jsr     next_playfield_row
    dex
    bne     .laser_v_loop
    ldy     #seven_strobe_count
    lda     #seven_strobe_length
    sta     ($fc),y
    iny
    lda     #0
    sta     ($fc),y                 ; Strobe phase.
    rts

; End a laser attack and possibly return to chase mode.
; Expects the player's attack block in $fc-$fd.  Returns
; 0 in the accumulator if the attack if over.
seven_clear_attack subroutine
    ldx     #16                     ; Clear the 16 rows.
    ldy     #seven_aim_row
    lda     ($fc),y
    sta     $fe
    iny
    lda     ($fc),y
    sta     $ff
.laser_h_loop
    lda     #0
    jsr     set_playfield_row
    jsr     next_playfield_row
    dex
    bne     .laser_h_loop
    ldx     #69                     ; Clear 69 rows.
    ldy     #seven_aim_column
    lda     ($fc),y
    sta     $fe
    iny
    lda     ($fc),y
    sta     $ff
.laser_v_loop
    lda     #0
    tay
    sta     ($fe),y
    iny
    sta     ($fe),y
    jsr     next_playfield_row
    dex
    bne     .laser_v_loop
    ldy     #seven_cycle_count
    lda     ($fc),y
    beq     .end_attack
    sec
    sbc     #1
    sta     ($fc),y
    ldy     #seven_crosshair_base
    lda     ($fc),y
    sta     $fe
    iny
    lda     ($fc),y
    sta     $ff
    jsr     seven_draw_crosshair
    ldy     #seven_tick_count
    lda     #seven_chase_length
    sta     ($fc),y
    iny
    lda     #seven_chase_phase
    sta     ($fc),y
    tya                             ; Non-0 result.
.end_attack
    rts

seven_cpu subroutine
    lda     p0_attack_data + seven_attack_phase
    cmp     #seven_fire_phase
    bne     .handle_chase
    lda     #0
    sta     p1_h_input
    sta     p1_v_input
    rts                             ; Do nothing.
.handle_chase
    inc     cpu_state
    lda     p0_attack_data + seven_crosshair_x
    cmp     #hpos_center
    bcs     .crosshair_right
    ldy     cpu_right_speed
    iny
    sty     cpu_hspeed
    lda     p1_real_hpos
    clc
    adc     #30
    bne     .done_hspeed            ; Always true.
.crosshair_right
    ldy     cpu_left_speed
    dey
    sty     cpu_hspeed
    lda     p1_real_hpos
    sec
    sbc     #30
.done_hspeed
    sta     cpu_xdest
    lda     p0_attack_data + seven_crosshair_y
    cmp     #$4a
    bcs     .crosshair_bottom
    ldy     cpu_down_speed
    iny
    sty     cpu_vspeed
    lda     p1_vpos
    clc
    adc     #30
    bne     .done_vspeed            ; Always true.
.crosshair_bottom
    ldy     cpu_up_speed
    dey
    sty     cpu_vspeed
    lda     p1_vpos
    sec
    sbc     #30
.done_vspeed
    sta     cpu_ydest
    rts

; Sprite images.
seven_p1_sprite
    .byte       %00110000
    .byte       %01111000
    .byte       %00100000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00010000
    .byte       %00011000
    .byte       %00001000
    .byte       %00000000
    .byte       %00001000
    .byte       %00001100
    .byte       %00001100
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000

seven_p2_sprite
    .byte       %00000000
    .byte       %00000000
    .byte       %01011000
    .byte       %01111000
    .byte       %01111100
    .byte       %01111100
    .byte       %00000000
    .byte       %00100000
    .byte       %00110000
    .byte       %00011000
    .byte       %00010000
    .byte       %00110000
    .byte       %00110000
    .byte       %01111110
    .byte       %00010000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000

seven_p3_sprite
    .byte       %00000000
    .byte       %00000000
    .byte       %00000100
    .byte       %00000100
    .byte       %00000000
    .byte       %00000000
    .byte       %00111000
    .byte       %01000100
    .byte       %01000110
    .byte       %11000010
    .byte       %10000011
    .byte       %10000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00111100
    .byte       %00111100
    .byte       %00101000
    .byte       %01101000
    .byte       %01101000
    .byte       %00101100

; Letter data for the select screen.
; See pink_b.s for details.
seven_letter_data
    .word       big_letter_7,$3000,$29cd
    .byte       big_letter_7_width
    .word       big_letter_z,$3002,$29cf
    .byte       big_letter_z_width
    .word       big_letter_c,$3004,$29d1
    .byte       big_letter_c_width
    .word       big_letter_hyphen,$3006,$29d3
    .byte       big_letter_hyphen_width
    .word       big_letter_b,$3007,$29d4
    .byte       big_letter_b_width
    .word       big_letter_x,$3009,$29d6
    .byte       big_letter_x_width

seven_game_row_offset       equ 14
seven_game_portrait_offset  equ 59 - seven_game_row_offset

seven_matrix_data
    .byte 136,144,136,144   ; Player positions.
    .byte 0,0,0,0           ; Missile positions.
    .byte $0c,$0c,$84,$84   ; Colors.

seven_matrix_image equ seven_matrix_image_real_start - 16
seven_matrix_image_real_start
    .byte %00100000         ; Player 0.
    .byte %00100000
    .byte %00100000
    .byte %01000000
    .byte %01000000
    .byte %01110000
    .byte %01010000
    .byte %01010100
    .byte %01010100
    .byte %01010111
    .byte %00110111
    .byte %00110111
    .byte %00010111
    .byte %00000011
    .byte %00000001
    .byte %00000001

    .byte %00000000         ; Player 1.
    .byte %00000000
    .byte %10000000
    .byte %10000000
    .byte %10000000
    .byte %01000000
    .byte %01011000
    .byte %01111000
    .byte %01111000
    .byte %11111000
    .byte %10011000
    .byte %01011000
    .byte %11110000
    .byte %11110000
    .byte %11100000
    .byte %01000000

    .byte %00001010         ; Player 2.
    .byte %00011010
    .byte %00011011
    .byte %00000100
    .byte %00001100
    .byte %00001000
    .byte %00000000
    .byte %00000000
    .byte %00001000
    .byte %00001000
    .byte %00001000
    .byte %00001000
    .byte %00001000
    .byte %00001000
    .byte %00000000
    .byte %00001000

    .byte %00011100         ; Player 3.
    .byte %10000000
    .byte %00000010
    .byte %00000010
    .byte %00000010
    .byte %00000010
    .byte %00000000
    .byte %00000000
    .byte %00000010
    .byte %00000010
    .byte %00000010
    .byte %00000010
    .byte %00000010
    .byte %00000000
    .byte %00000000
    .byte %00000000

seven_matrix_background
    .byte %00101010,%10101010,%10101010,%10101000
    .byte %00101010,%10101010,%10101010,%10101000
    .byte %00101010,%10101010,%10101010,%10101000
    .byte %00101010,%10101010,%10101010,%10101000
    .byte %00101010,%10101010,%10101010,%10101000
    .byte %00101010,%10101010,%10101010,%10101000
    .byte %00101010,%10101010,%10101010,%10101000
    .byte %10101010,%10101010,%10101010,%10101000
    .byte %10101010,%10101010,%10101010,%10101000
    .byte %00101010,%10101010,%10101010,%10101000
    .byte %00001010,%10101010,%10101010,%10101000
    .byte %00001010,%10101010,%10101010,%10101000
    .byte %00001010,%10101010,%10101010,%10101000
    .byte %00000010,%10101010,%10101010,%00101000
    .byte %00000010,%10101010,%10101010,%00100000
    .byte %00000010,%10000010,%10101010,%00100000

seven_crosshair
    .byte %00010101,%01010100
    .byte %01000001,%01000001
    .byte %01000001,%01000001
    .byte %01000001,%01000001
    .byte %01000001,%01000001
    .byte %01000001,%01000001
    .byte %01000000,%00000001
    .byte %01010100,%00010101
    .byte %01010100,%00010101
    .byte %01000000,%00000001
    .byte %01000001,%01000001
    .byte %01000001,%01000001
    .byte %01000001,%01000001
    .byte %01000001,%01000001
    .byte %01000001,%01000001
    .byte %00010101,%01010100
   
seven_seek_beep
    .byte   $e3
    .byte   $10,$5
    .byte   $0
    
seven_lock_beep
    .byte   $e3
    .byte   $08,$e
    .byte   $0

seven_laser_fire
    .byte   $8f
    .byte   $04,$0a
    .byte   $0a,$0a
    .byte   $10,$04
    .byte   $0

seven_portrait
    include "7zcbx_portrait.s"

seven_portrait_color_1 equ $0 ; 0, 0, 0
seven_portrait_color_2 equ $a ; 165, 165, 165
seven_portrait_color_3 equ $90 ; 0, 0, 99

seven_portrait_table
    .byte 4 + 128
    .byte seven_portrait_color_1
    .byte 18 + 128
    .byte seven_portrait_color_2
    .byte 1
    .byte seven_portrait_color_1
    .byte 9 + 128
    .byte seven_portrait_color_2
    .byte 23 + 128
    .byte seven_portrait_color_1
    .byte 5 + 128
    .byte seven_portrait_color_3
    .byte 4 + 128
    .byte seven_portrait_color_2
    .byte 18 + 128
    .byte seven_portrait_color_1
    .byte 1
    .byte seven_portrait_color_2
    .byte 9 + 128
    .byte seven_portrait_color_1
    .byte 10 + 128
    .byte seven_portrait_color_2
    .byte 1
    .byte seven_portrait_color_3
    .byte 5 + 128
    .byte seven_portrait_color_2
    .byte 7 + 128
    .byte seven_portrait_color_3
    .byte 5 + 128
    .byte seven_portrait_color_1
    .byte 42 + 128
    .byte seven_portrait_color_3
    .byte 1
    .byte seven_portrait_color_2
    .byte 5 + 128
    .byte seven_portrait_color_3
    .byte 12 + 128
    .byte seven_portrait_color_2

