; This file is part of Magical Fairy Force.

; Magical Fairy Force is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Magical Fairy Force is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Magical Fairy Force.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2020 Average Software.

; Character selection screen

; Colors and other constants.
p0_cell_color           equ $30
p1_cell_color           equ $c0
move_delay              equ 30
neutral_color_pattern   equ $55
selection_color_pattern equ $ff
big_letter_color_1      equ $28
big_letter_color_2      equ $22
neutral_cell_color      equ $0f
background_color        equ $84
upper_matrix_base       equ $6f
lower_matrix_base       equ $81

; Data.
active_player_color     equ $20
p0_matrix_pos_low       equ $33
p1_matrix_pos_low       equ $34
p2_matrix_pos_low       equ $35
p3_matrix_pos_low       equ $36
m0_matrix_pos_low       equ $37
m1_matrix_pos_low       equ $38
m2_matrix_pos_low       equ $39
m3_matrix_pos_low       equ $3a
player_0_cell           equ $3b
player_1_cell           equ $3c
active_player           equ $3d
p0_delay                equ $3e
p1_delay                equ $3f
p0_selected             equ $40
p1_selected             equ $41

p0_portrait_table_0     equ $1900
p0_portrait_table_1     equ p0_portrait_table_0 + 60
p0_portrait_table_2     equ p0_portrait_table_1 + 60
p1_portrait_table_0     equ p0_portrait_table_2 + 60
p1_portrait_table_1     equ p1_portrait_table_0 + 60
p1_portrait_table_2     equ p1_portrait_table_1 + 60

start_character_select subroutine
    mwa     ENDINT,VVBLKD           ; Shut down the vblank interrupt.
    lda     #<null_display_list
    sta     sDLISTL
    sta     DLISTL
    lda     #>null_display_list
    sta     sDLISTH
    sta     DLISTH
    lda     #0
    sta     NMIEN
    sta     SIZEP0
    sta     SIZEP1
    sta     SIZEP2
    sta     SIZEP3
    sta     VDSLST
    lda     #$ae
    sta     VDSLST + 1
    jsr     hide_all_sprites
    lda     #background_color
    sta     sCOLBK
    lda     #1
    sta     AUDCTL                  ; Configure audio.
    sta     PRIOR                   ; Players over playfield.
    ldx     #8                      ; Blank video RAM.
.ram_clear_loop
    stx     $ff
    jsr     clear_page
    inx
    cpx     #$40
    bne     .ram_clear_loop
    jsr     build_byte_reverse_table
    lda     other_screen_active     ; Skip the matrix if some
    beq     .draw_matrix            ; other screen is using this code.
    ldy     p0_character
    jsr     select_player_0
    jmp     .players_configured
.draw_matrix
    lda     #7                      ; Draw the character matrix.
    sta     $f0                     ; Save the index.
    ldx     #neutral_color_pattern
.matrix_loop
    ldy     $f0
    jsr     draw_cell
    dec     $f0
    bpl     .matrix_loop
    mwa     pink_b_matrix_background,$fe
    mwa     $834,$fc
    jsr     draw_cell_background
    mwa     frooty_matrix_background,$fe
    mwa     $839,$fc
    jsr     draw_cell_background
    mwa     seven_matrix_background,$fe
    mwa     $83e,$fc
    jsr     draw_cell_background
    mwa     kitty_matrix_background,$fe
    mwa     $843,$fc
    jsr     draw_cell_background
    mwa     porkchop_matrix_background,$fe
    mwa     $b04,$fc
    jsr     draw_cell_background
    mwa     xorex_matrix_background,$fe
    mwa     $b09,$fc
    jsr     draw_cell_background
    mwa     q_sugar_matrix_background,$fe
    mwa     $b0e,$fc
    jsr     draw_cell_background
    mwa     zevasha_matrix_background,$fe
    mwa     $b13,$fc
    jsr     draw_cell_background
    ldy     #0
    sty     player_0_cell
    sty     active_player
    sty     p0_delay
    sty     p1_delay
    sty     p0_selected
    sty     p1_selected
    jsr     select_player_0
    lda     game_mode
    cmp     #1
    beq     .one_player
    ldy     #3
    sty     player_1_cell
    jsr     select_player_1
    jmp     .players_configured
.one_player
    lda     #-1
    sta     player_1_cell
.players_configured
    jsr     create_character_select_display_list
    mva     #$ff,last_key_pressed
    mwa     character_select_vblank,VVBLKD
    lda     #0
    sta     sDLISTL                 ; Set up the display list.
    sta     DLISTL
    lda     #4
    sta     sDLISTH
    sta     DLISTH
    mva     #$c0,NMIEN
    jsr     detect_controllers
    lda     other_screen_active     ; If another screen is using
    beq     character_select_loop   ; this code, return.
    jsr     wait_for_vb
    jmp     start_schedule
    
character_select_loop subroutine
    jsr     wait_for_vb
    lda     last_key_pressed
    bmi     .no_key
    cmp     #$0e                    ; Check for reset.
    bne     .no_key
    jsr     wait_for_vb
    jmp     start_title_screen
.no_key
    jsr     read_inputs
    lda     p0_selected
    beq     .p0_still_active
    jmp     .p0_input_done
.p0_still_active
    lda     p0_delay
    beq     .p0_delay_over
    dec     p0_delay
    jmp     .p0_move_done
.p0_delay_over
    lda     player_0_cell           ; Save the old value so we
    sta     $f0                     ; can check for changes.
    lda     p0_h_input
    beq     .p0_check_v_input
    cmp     #2
    bpl     .p0_move_right
    cmp     #-1
    bpl     .p0_check_v_input
    ldx     #0
    jmp     .p0_find_new_cell
.p0_move_right
    ldx     #1
.p0_find_new_cell
    lda     player_0_cell
    jsr     get_next_cell
    sta     player_0_cell
.p0_check_v_input
    lda     p0_v_input
    beq     .p0_check_cell_change
    cmp     #2
    bpl     .p0_vertical_change
    cmp     #-1
    bpl     .p0_check_cell_change
.p0_vertical_change
    lda     player_0_cell
    eor     #4                      ; Flip the 4 bit to change rows.
    sta     player_0_cell
.p0_check_cell_change
    ldy     $f0                     ; See if player 0's cell changed.
    cpy     player_0_cell
    beq     .p0_move_done           ; No change, ignore.
    lda     #move_delay             ; Reset move delay.
    sta     p0_delay
    ldx     #neutral_color_pattern  ; Erase the old cell.
    jsr     draw_cell
    ldy     player_0_cell           ; Set up the new selection.
    jsr     select_player_0
    mwa     select_move_sound,$fe   ; Play the sound effect.
    jsr     play_sound_1
.p0_move_done
    lda     p0_trig1                ; Check for selection.
    bne     .p0_input_done
    lda     #1
    sta     p0_selected
    mwa     select_confirm_sound,$fe
    jsr     play_sound_1
.p0_input_done
    lda     player_1_cell           ; See if player 1 is active.
    bpl     .player_1_active
    jmp     .p1_input_done
.player_1_active
    lda     p1_selected
    beq     .p1_still_active
    jmp     .p1_input_done
.p1_still_active
    lda     p1_delay
    beq     .p1_delay_over
    dec     p1_delay
    jmp     .p1_move_done
.p1_delay_over
    lda     player_1_cell           ; Save the old value so we
    sta     $f0                     ; can check for changes.
    lda     p1_h_input
    beq     .p1_check_v_input
    cmp     #2
    bpl     .p1_move_right
    cmp     #-1
    bpl     .p1_check_v_input
    ldx     #0
    jmp     .p1_find_new_cell
.p1_move_right
    ldx     #1
.p1_find_new_cell
    lda     player_1_cell
    jsr     get_next_cell
    sta     player_1_cell
.p1_check_v_input
    lda     p1_v_input
    beq     .p1_check_cell_change
    cmp     #2
    bpl     .p1_vertical_change
    cmp     #-1
    bpl     .p1_check_cell_change
.p1_vertical_change
    lda     player_1_cell
    eor     #4                      ; Flip the 4 bit to change rows.
    sta     player_1_cell
.p1_check_cell_change
    ldy     $f0                     ; See if player 1's cell changed.
    cpy     player_1_cell
    beq     .p1_move_done           ; No change, ignore.
    lda     #move_delay             ; Reset move delay.
    sta     p1_delay
    ldx     #neutral_color_pattern  ; Erase the old cell.
    jsr     draw_cell
    ldy     player_1_cell           ; Set up the new selection.
    jsr     select_player_1
    mwa     select_move_sound,$fe   ; Play the sound effect.
    jsr     play_sound_1
.p1_move_done
    lda     p1_trig1                ; Check for selection.
    bne     .p1_input_done
    lda     #1
    sta     p1_selected
    mwa     select_confirm_sound,$fe
    jsr     play_sound_1
.p1_input_done
    lda     game_mode               ; Check for all players ready.
    cmp     #1
    beq     .one_player
    lda     p0_selected
    beq     .not_done_yet
    lda     p1_selected
    beq     .not_done_yet
    lda     player_1_cell
    sta     p1_character
    lda     player_0_cell
    sta     p0_character
    ldy     #120
    jsr     wait_for_y_frames
    jmp     start_versus_screen
.one_player
    lda     p0_selected
    beq     .not_done_yet
    lda     player_0_cell
    sta     p0_character
    cmp     #7                      ; If Zevasha, make Queen Sugar
    bne     .not_zevasha            ; the final opponent.
    ldy     #0
.schedule_loop
    lda     fight_schedule,y
    cmp     #6                      ; Is this Queen Sugar's slot?
    beq     .adjust_schedule
    iny
    bne     .schedule_loop          ; Always true.
.adjust_schedule
    sta     fight_schedule + 7      ; Replace Zevasha with Queen Sugar.
    lda     #7                      ; Replace Queen Sugar with Zevasha.
    sta     fight_schedule,y
.not_zevasha
    ldy     #120
    jsr     wait_for_y_frames
    lda     #1
    sta     other_screen_active
    jsr     clear_pm_and_matrix
    jmp     start_schedule
.not_done_yet
    jmp     character_select_loop

; Create the display list for the character
; select screen.  This list is shared by the
; versus screen, and is thus factored out.
create_character_select_display_list subroutine
    lda     #$0e                    ; Set the entire $4 page to 0e.
    ldy     #0
.loop
    dey
    sta     $400,y
    bne     .loop
    lda     #$70                    ; Overscan skip.
    sta     $400
    sta     $401
    sta     $402
    lda     #$ce                    ; Fill in the interesting parts.
    sta     $403                    ; See data.s for details on this
    lda     #0                      ; display list.
    sta     $404
    sta     $454
    sta     $47a
    sta     $4ca
    lda     #$20
    sta     $405
    lda     #$8e
    sta     $442
    sta     $452
    sta     $465
    sta     $478
    sta     $48b
    lda     #$4e
    sta     $453
    sta     $479
    lda     #$08
    sta     $455
    lda     #$30
    sta     $47b
    lda     #$41
    sta     $4c9
    lda     #$40
    sta     $4cb
    rts

; Select a character for player 0.  Expects the
; cell number to be passed in y.
select_player_0 subroutine
    lda     portrait_color_low,y    ; Build the portrait color table.
    sta     $f6
    lda     portrait_color_high,y
    sta     $f7
    tya                             ; Save the cell number.
    pha
    mwa     $32d0,$fe               ; Erase the old portrait.
    jsr     erase_portrait
    mwa     p0_portrait_table_0,$fe
    jsr     decompress_portrait_table
    pla
    tay                             ; Retrieve the cell number.
    lda     player_character_counts,y
    sta     $f0                     ; Save the character count.
    lda     player_letter_data_low,y
    sta     $f2
    lda     player_letter_data_high,y
    sta     $f3                     ; Set up the data fields.
    lda     player_portrait_data_low,y
    sta     $f6
    lda     player_portrait_data_high,y
    sta     $f7
    lda     #0                      ; Index into the character data.
    sta     $f1                     ; Save it.
    mwa     $3000,$f4               ; Erase at $3000.
    jsr     erase_big_name
.name_loop
    ldy     $f1                     ; Get the index into the character data.
    lda     ($f2),y                 ; Get the letter address.
    sta     $fc
    iny
    lda     ($f2),y
    sta     $fd
    iny
    lda     ($f2),y                 ; Get the target address.
    sta     $fe
    iny
    lda     ($f2),y
    sta     $ff
    iny                             ; Skip the player 1 target.
    iny
    iny
    lda     ($f2),y                 ; Get the letter width.
    iny
    sty     $f1                     ; Save the index.
    tay
    jsr     copy_big_letter
    dec     $f0                     ; Check for completion.
    bne     .name_loop
    lda     #$d0                    ; Place at $32d0.
    sta     $f0
    lda     #$32
    sta     $f1
    lda     #0                      ; Full portrait.
    jmp     copy_portrait           ; Tail call.

; The same, but for player 1.
select_player_1 subroutine
    lda     portrait_color_low,y    ; Build the portrait color table.
    sta     $f6
    lda     portrait_color_high,y
    sta     $f7
    tya                             ; Save the cell number.
    pha
    mwa     $2019,$fe               ; Erase the old portrait.
    jsr     erase_portrait
    mwa     p1_portrait_table_0,$fe
    jsr     decompress_portrait_table
    pla                             ; Retrieve the cell number.
    tay
    lda     player_character_counts,y
    sta     $f0                     ; Save the character count.
    lda     player_letter_data_low,y
    sta     $f2
    lda     player_letter_data_high,y
    sta     $f3                     ; Set up the data fields.
    lda     player_portrait_data_low,y
    sta     $f6
    lda     player_portrait_data_high,y
    sta     $f7
    ldy     #59                     ; Set up the top row portrait colors.
    lda     p1_portrait_table_0,y
    sta     sCOLOR0
    lda     p1_portrait_table_1,y
    sta     sCOLOR1
    lda     p1_portrait_table_2,y
    sta     sCOLOR2
    lda     #0                      ; Index into the character data.
    sta     $f1                     ; Save it.
    mwa     $29c7,$f4               ; Erase at $29c7.
    jsr     erase_big_name
.name_loop
    ldy     $f1                     ; Get the index into the character data.
    lda     ($f2),y                 ; Get the letter address.
    sta     $fc
    iny
    lda     ($f2),y
    sta     $fd
    iny                             ; Skip the player 0 target.
    iny
    iny
    lda     ($f2),y                 ; Get the target address.
    sta     $fe
    iny
    lda     ($f2),y
    sta     $ff
    iny
    lda     ($f2),y                 ; Get the letter width.
    iny
    sty     $f1                     ; Save the index.
    tay
    jsr     copy_big_letter
    dec     $f0                     ; Check for completion.
    bne     .name_loop
    lda     #$19                    ; Place at $2019.
    sta     $f0
    lda     #$20
    sta     $f1
    lda     #0                      ; Full portrait.
    jmp     copy_portrait_mirrored  ; Tail call.

; Copy portrait table data to the address in $fe-$ff.
; The compressed table pointer should be in $f6-$f7.
decompress_portrait_table subroutine
    lda     #180                    ; 180 bytes expected.
    sta     $f0
    ldy     #0                      ; Write index.
.outer_loop
    sty     $f1                     ; Save the write index.
    jsr     decompress_portrait_chunk
    cpx     #0                      ; Check for compressed section.
    bpl     .uncompressed_chunk
    txa                             ; Mask off the sign bit.
    and     #$7f
    tax                             ; Repeat count.
    lda     $1000                   ; Byte to repeat.
    ldy     $f1                     ; Write index.
.compressed_loop
    sta     ($fe),y                 ; Write the byte.
    dec     $f0                     ; Drop the total count.
    beq     .done
    iny                             ; Next write index.
    dex                             ; Drop the block count.
    beq     .outer_loop
    bne     .compressed_loop        ; Always true.
.uncompressed_chunk
    stx     $f2                     ; Block size.
    ldx     #0                      ; Read index.
    ldy     $f1                     ; Write index.
.inner_loop
    lda     $1000,x                 ; Grab a byte.
    sta     ($fe),y                 ; Write a byte.
    dec     $f0                     ; Drop the total count.
    beq     .done
    iny                             ; Next write index.
    dec     $f2                     ; Drop the block count.
    beq     .outer_loop
    inx                             ; Next read byte.
    bne     .inner_loop             ; Always true.
.done
    rts

; Erase a character portrait.  This is done during selection
; changes to avoid any nastiness from palette/data mismatches.
; Expects the portrait base in $fe-$ff.
erase_portrait subroutine
    ldx     #60                     ; 60 rows.
.loop
    ldy     #14                     ; Row index.
    lda     #0
.row_loop
    sta     ($fe),y
    dey
    bpl     .row_loop
    dex
    beq     .done
    lda     $fe                     ; Update the pointer.
    clc
    adc     #40
    sta     $fe
    bcc     .loop
    inc     $ff
    bcs     .loop                   ; Always true.
.done
    rts

; Erase a big character name.  Expects the base address
; to be stored in $f4-$f5.
erase_big_name subroutine
    ldx     #16                     ; 16 rows need erasure.
.loop
    ldy     #16                     ; 17 bytes to erase on each row.
    lda     #0
.row_loop
    sta     ($f4),y
    dey
    bpl     .row_loop
    dex
    beq     .done
    lda     $f4                     ; Advance the address.
    clc
    adc     #40
    sta     $f4
    bcc     .loop
    inc     $f5
    bcs     .loop                   ; Always true.
.done
    rts

; Copy a big character name letter to the screen.
; Destination address should be in $fe-$ff, source
; address in $fc-$fd.  Byte-width of the letter
; should be passed in y.
copy_big_letter subroutine
    lda     #16                     ; Row count.  All letters are 16 high.
    sta     $fb                     ; Save it.
    sty     $fa                     ; Save the row width.
.loop
    jsr     copy_memory             ; Copy the row.
    dec     $fb
    beq     .done
    lda     $fe                     ; Advance to the next row.
    clc
    adc     #40
    sta     $fe
    lda     $ff
    adc     #0
    sta     $ff
    lda     $fc                     ; Update the source address.
    clc
    adc     $fa
    sta     $fc
    lda     $fd
    adc     #0
    sta     $fd
    ldy     $fa
    jmp     .loop
.done
    rts

; Determine the cell to move to in response to
; a horizontal movement.  Takes the current
; cell in the accumulator and the direction
; of movement in x.  Movement codes are:
; 0 - left
; 1 - right
; The new cell will be in the accumulator.
get_next_cell subroutine
    cpx     #0                      ; Check the movement direction.
    bne     .move_right
    cmp     #0                      ; Move left.  On 0 or 4, wrap
    beq     .left_wrap              ; around.  Subtract 1 otherwise.
    cmp     #4
    beq     .left_wrap
    sec
    sbc     #1
    rts
.left_wrap
    clc                             ; Add 3 to wrap.
    adc     #3
    rts
.move_right
    cmp     #3                      ; On 3 or 7, wrap around.
    beq     .right_wrap             ; Add 1 otherwise.
    cmp     #7
    beq     .right_wrap
    clc
    adc     #1
    rts
.right_wrap
    sec                             ; Subtract 3 to wrap.
    sbc     #3
    rts

; Draws a cell border using the color in x.
; The cell number should be passed in y.
draw_cell subroutine
    lda     matrix_upper_byte,y     ; Get the low bytes of the
    sta     $fe                     ; base addresses.
    lda     matrix_lower_byte,y
    sta     $fc
    cpy     #4                      ; Check the row.
    bmi     .upper_row
    lda     #$a                     ; The high byte will be $a.
    sta     $ff
    lda     #$d                     ; The high byte will be $d.
    sta     $fd
    jmp     .create_masks
.upper_row
    lda     #8                      ; The high byte will be $8.
    sta     $ff
    lda     #$a                     ; The high byte will be $a.
    sta     $fd
.create_masks
    txa                             ; Create left/right masks.
    and     #$0f
    sta     $fb                     ; Right-side mask.
    txa
    and     #$f0
    sta     $fa                     ; Left-side mask.
    ldy     #0
    and     ($fe),y
    ora     $fb
    sta     ($fe),y
    lda     $fa
    and     ($fc),y
    ora     $fb
    sta     ($fc),y
    ldy     #5
    lda     $fb
    and     ($fe),y
    ora     $fa
    sta     ($fe),y
    lda     $fb
    and     ($fc),y
    ora     $fa
    sta     ($fc),y
    ldy     #4
    txa                             ; Use the full color in the middle.
.outside_row_loop
    sta     ($fe),y
    sta     ($fc),y
    dey
    bne     .outside_row_loop
    ldy     #16                     ; 16 rows.
    sty     $f9
.inside_row_loop
    lda     $fe                     ; Update the address.
    clc
    adc     #40
    sta     $fe
    lda     $ff
    adc     #0
    sta     $ff
    ldy     #0
    lda     $fa
    and     ($fe),y
    ora     $fb
    sta     ($fe),y                 ; Left edge.
    ldy     #5
    lda     $fb
    and     ($fe),y
    ora     $fa
    sta     ($fe),y                 ; Right edge.
    dec     $f9
    bne     .inside_row_loop
    rts

; Draws the background image inside a cell.
; Expects the source data in $fe-$ff and the
; target location in $fc-$fd.
draw_cell_background subroutine
    ldy     #0                      ; Index into the source data.
    sty     $fa
    ldy     #16                     ; 16 rows to copy.
    sty     $f9
.loop
    ldy     #0                      ; Index into the target.
    sty     $fb
    ldx     #4                      ; 4 bytes per row.
.row_loop
    ldy     $fa                     ; Get the source index.
    lda     ($fe),y
    ldy     $fb                     ; Get the target index.
    sta     ($fc),y
    inc     $fa
    dex
    beq     .row_done
    inc     $fb
    jmp     .row_loop
.row_done
    dec     $f9
    beq     .done
    lda     $fc                     ; Update the target address.
    clc
    adc     #40
    sta     $fc
    lda     $fd
    adc     #0
    sta     $fd
    jmp     .loop
.done
    rts
    
character_select_vblank subroutine
    lda     $f9                     ; Save $f9 through $ff.
    pha                             ; draw_cell and process_sounds
    lda     $fa                     ; clobber these locations.
    pha
    lda     $fb
    pha
    lda     $fc
    pha
    lda     $fd
    pha
    lda     $fe
    pha
    lda     $ff
    pha
    jsr     process_sounds
    lda     #0
    sta     ATRACT
    ldy     key_debounce
    beq     .no_debounce
    dec     key_debounce
.no_debounce
    sta     VDSLST
    lda     #$ae
    sta     VDSLST + 1
    lda     other_screen_active
    bne     .other_skip
    ldx     #neutral_color_pattern
    lda     active_player
    beq     .show_player_1_cell
    ldy     player_1_cell
    bmi     .player_1_inactive
    jsr     draw_cell
.player_1_inactive
    ldy     player_0_cell
    jsr     .copy_matrix_data
    ldx     #selection_color_pattern
    lda     p0_selected
    bne     .cells_done
    ldy     player_0_cell
    jsr     draw_cell
    lda     #p0_cell_color
    jmp     .cells_done
.show_player_1_cell
    ldy     player_0_cell
    jsr     draw_cell
    ldy     player_1_cell
    bmi     .erase_p1_cell
    jsr     .copy_matrix_data
    ldx     #selection_color_pattern
    lda     p1_selected
    bne     .cells_done
    ldy     player_1_cell
    bpl     .player_1_active
    lda     #neutral_cell_color
    jmp     .cells_done
.erase_p1_cell
    ldy     #7
    lda     #0
.erase_loop
    sta     HPOSP0,y
    sta     p0_matrix_pos_low,y
    dey
    bpl     .erase_loop
    jmp     .cells_done
.player_1_active
    jsr     draw_cell
    lda     #p1_cell_color
.cells_done
    sta     active_player_color
    lda     active_player
    eor     #1
    sta     active_player
.other_skip
    pla
    sta     $ff
    pla
    sta     $fe
    pla
    sta     $fd
    pla
    sta     $fc
    pla
    sta     $fb
    pla
    sta     $fa
    pla
    sta     $f9
    jmp     ENDINT
.copy_matrix_data
    sty     $f9
    cpy     #4
    bmi     .erase_upper_cell
    ldx     #lower_matrix_base
    jmp     .copy_new_cell
.erase_upper_cell
    ldx     #upper_matrix_base
.copy_new_cell
    ldy     $f9
    lda     player_matrix_image_low,y
    sta     $fe
    lda     player_matrix_image_high,y
    sta     $ff
    ldy     #0
    sty     $fa
    stx     $fc
    lda     #$13
    sta     $fd
.copy_new_cell_outer_loop
    ldx     #0
    stx     $fb
.copy_new_cell_inner_loop
    ldy     $fa
    lda     ($fe),y
    ldy     $fb
    sta     ($fc),y
    cpy     #15
    beq     .copy_new_cell_inner_loop_done
    inc     $fa
    inc     $fb
    jmp     .copy_new_cell_inner_loop
.copy_new_cell_inner_loop_done
    ldy     $fa
    cpy     #79
    beq     .copy_new_cell_done
    inc     $fa
    inc     $fd
    jmp     .copy_new_cell_outer_loop
.copy_new_cell_done
    ldy     $f9
    lda     player_matrix_data_low,y
    sta     $fc
    lda     player_matrix_data_high,y
    sta     $fd
    ldx     #0
    cpy     #4
    bpl     .matrix_data_pos_low
    ldy     #7
.matrix_data_pos_loop_high
    lda     ($fc),y
    sta     HPOSP0,y
    stx     p0_matrix_pos_low,y
    dey
    bpl     .matrix_data_pos_loop_high
    jmp     .matrix_data_color
.matrix_data_pos_low
    ldy     #7
.matrix_data_pos_loop_low
    lda     ($fc),y
    sta     p0_matrix_pos_low,y
    txa
    sta     HPOSP0,y
    dey
    bpl     .matrix_data_pos_loop_low
.matrix_data_color
    ldy     #8
.matrix_data_color_loop
    lda     ($fc),y
    sta     COLPM0 - 8,y
    cpy     #11
    beq     .done_matrix_data
    iny
    jmp     .matrix_data_color_loop
.done_matrix_data
    rts

