; This file is part of Magical Fairy Force.

; Magical Fairy Force is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Magical Fairy Force is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Magical Fairy Force.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2020 Average Software.

; Frooty character data and logic routines

; Frooty's attack flow:

; The lollipop object enters from the center
; of the screen and begins bouncing about the
; playfield much like a Breakout ball.  Contact
; with the lollipop is damaging.

; The lollipop will gradually increase in speed
; and will disappear after X seconds, ending the
; attack.

; Attack constants.
frooty_attack_cost      equ -max_fairy_power / 2
frooty_max_xsubpixel    equ 64
frooty_max_ysubpixel    equ 10
frooty_left_bound       equ 0
frooty_right_bound      equ 38
frooty_top_bound        equ 0
frooty_bottom_bound     equ 53
frooty_color_1          equ $4a
frooty_color_2          equ $4e

; Attack block offsets.
frooty_tick_count       equ 0
frooty_phase            equ 1
frooty_address          equ 2
frooty_xpos             equ 4
frooty_ypos             equ 5
frooty_xsub             equ 6
frooty_ysub             equ 7
frooty_xspeed           equ 8
frooty_yspeed           equ 9

frooty_start_attack subroutine
    beq     .attack_player_0
    ldx     #frooty_attack_cost
    jsr     general_player_0_attack_start
    lda     #frooty_color_1
    sta     p1_playfield_2_color
    lda     #frooty_color_2
    sta     p1_playfield_1_color
    mwa     $2002,p0_attack_data + frooty_address
    jsr     get_player_0_random     ; Random value for speeds.
    tax
    lda     #>p0_attack_data        ; Branch is always true.
    bne     frooty_init_attack_block
.attack_player_0
    ldx     #frooty_attack_cost
    jsr     general_player_1_attack_start
    lda     #frooty_color_1
    sta     p0_playfield_2_color
    lda     #frooty_color_2
    sta     p0_playfield_1_color
    mwa     $3142,p1_attack_data + frooty_address
    jsr     get_player_1_random     ; Random value for speeds.
    tax
    lda     #>p1_attack_data
    ; Fall through.

; Initialize the attack data whose
; high byte is in the accumulator.
; Expects a random number in x.
; Attack address should be filled in
; by the caller since that's player
; specific.
frooty_init_attack_block subroutine
    sta     $ff
    lda     #0
    sta     $fe
    ldy     #frooty_tick_count
    lda     #120
    sta     ($fe),y
    tya                             ; Phase 0.
    iny
    sta     ($fe),y
    iny
    lda     ($fe),y
    sta     $fc                     ; Get the address so the
    iny                             ; attack can be drawn.
    lda     ($fe),y
    sta     $fd
    iny
    lda     #2
    sta     ($fe),y                 ; Xpos.
    iny
    lda     #0
    sta     ($fe),y                 ; Ypos.
    iny
    lda     #0
    sta     ($fe),y                 ; X subpixel.
    iny
    sta     ($fe),y                 ; Y subpixel.
    iny
    txa                             ; The lower 4 bits of the random
    and     #$0f                    ; number + 8 are the initial x speed.
    clc
    adc     #8
    sta     ($fe),y                 ; X speed.
    iny
    txa                             ; The upper 4 bits of the random
    lsr                             ; number + 4 are the initial y speed.
    lsr
    lsr
    lsr
    clc
    adc     #4
    sta     ($fe),y                 ; Y speed.
    ldx     #0
    ldy     #1
.loop
    dey
    lda     frooty_lollipop,x
    sta     ($fc),y
    iny
    inx
    lda     frooty_lollipop,x
    sta     ($fc),y
    cpx     #31
    beq     .done
    inx
    lda     $fc
    clc
    adc     #40
    sta     $fc
    bcc     .loop
    inc     $fd
    bcs     .loop
.done
    jmp     start_attack_return

frooty_process_attack subroutine
    beq     frooty_attack_player_0
    ; Fall through.

frooty_attack_player_1 subroutine
    lda     #2
    sta     p1_cloud_timer          ; Suppress cloud spawns.
    lda     pf_high
    beq     .no_damage
    jsr     damage_player_1
.no_damage
    lda     #>p0_attack_data
    jsr     frooty_perform_attack
    bne     .continue
    jmp     end_player_0_attack
.continue
    cpy     #0
    beq     .no_sound
    mwa     frooty_bounce_sound,$fe
    jsr     play_sound_3
.no_sound
    jmp     p0_process_return

frooty_attack_player_0 subroutine
    lda     #2
    sta     p0_cloud_timer          ; Suppress cloud spawns.
    lda     pf_low
    beq     .no_damage
    jsr     damage_player_0
.no_damage
    lda     #>p1_attack_data
    jsr     frooty_perform_attack
    bne     .continue
    jmp     end_player_1_attack
.continue
    cpy     #0
    beq     .no_sound
    mwa     frooty_bounce_sound,$fe
    jsr     play_sound_4
.no_sound
    jmp     p1_process_return

; Process the Frooty attack.  Expects the high
; byte of the attack block in the accumulator.
; Returns 0 in the accumulator if the attack is complete,
; non-zero otherwise.  Returns non-zero in y if the bounce sound
; needs to be played.
frooty_perform_attack subroutine
    sta     $ff                     ; Attack block high byte.
    lda     #0                      ; Attack block low byte.
    sta     $fe
    sta     $f9                     ; Assume no bounce sound.
    ldy     #frooty_address         ; Set up the attack pointer.
    lda     ($fe),y                 ; Two copies, old and new.
    sta     $fc
    sta     $fa
    iny
    lda     ($fe),y
    sta     $fd
    sta     $fb
    ldx     #16                     ; Erase the old attack.
.erase_outer_loop
    ldy     #1                      ; Write index.
    lda     #0
.erase_inner_loop
    sta     ($fa),y
    dey
    bpl     .erase_inner_loop
    dex
    beq     .erase_done
    jsr     .update_hmove_address
    jmp     .erase_outer_loop
.erase_done
    ldy     #frooty_tick_count
    lda     ($fe),y
    beq     .new_phase
    sec
    sbc     #1
    jmp     .update_tick
.new_phase
    iny                             ; Phase ID.
    lda     ($fe),y                 ; Check the phase.
    cmp     #8                      ; End on phase 8.
    bne     .not_done
    lda     #0                      ; Return value.
    rts
.not_done
    clc
    adc     #1                      ; Increment phase.
    sta     ($fe),y
    ldy     #frooty_xspeed          ; Increase the speeds.
    lda     ($fe),y
    bmi     .negative_x
    adc     #2
    .byte   $2c
.negative_x
    adc     #-2
    sta     ($fe),y
    iny
    clc
    lda     ($fe),y
    bmi     .negative_y
    adc     #1
    .byte   $2c
.negative_y
    adc     #-1
    sta     ($fe),y
    lda     #60                     ; Reset tick.
    ldy     #frooty_tick_count
.update_tick
    sta     ($fe),y
    ldy     #frooty_xspeed          ; Do horizontal movement.
    lda     ($fe),y                 ; Apply the speed to the subpixel.
    ldy     #frooty_xsub
    clc
    adc     ($fe),y
    sta     ($fe),y
    ldy     #frooty_yspeed          ; Do vertical movement.
    lda     ($fe),y                 ; Apply the speed to the subpixel.
    ldy     #frooty_ysub
    clc
    adc     ($fe),y
    sta     ($fe),y
    ldy     #frooty_xsub            ; Check for horizontal movement.
    lda     ($fe),y
    bmi     .move_left              ; If negative, move a pixel left.
    cmp     #frooty_max_xsubpixel   ; If over the subpixel cap,
    bpl     .move_right             ; move a pixel right.
    sta     ($fe),y                 ; Just update the subpixel.
    jmp     .horizontal_done
.move_left
    clc
    adc     #frooty_max_xsubpixel   ; Adjust the subpixel.
    sta     ($fe),y
    ldy     #frooty_xpos            ; Update the x position.
    lda     ($fe),y
    sec
    sbc     #1
    sta     ($fe),y
    cmp     #frooty_left_bound      ; Reverse direction if needed.
    bne     .no_left_reverse
    jsr     .reverse_xspeed
    ldy     #frooty_xsub            ; Set the subpixel to 0.
    lda     #0
    sta     ($fe),y
.no_left_reverse
    lda     $fc                     ; Move a byte left.
    bne     .no_left_borrow
    dec     $fd
.no_left_borrow
    dec     $fc
    jmp     .horizontal_done
.move_right
    sec
    sbc     #frooty_max_xsubpixel   ; Adjust the subpixel.
    sta     ($fe),y
    ldy     #frooty_xpos            ; Update the x position.
    lda     ($fe),y
    clc
    adc     #1
    sta     ($fe),y
    cmp     #frooty_right_bound     ; Reverse direction if needed.
    bne     .no_right_reverse
    ldy     #frooty_xsub            ; Set the subpixel to max - 1.
    lda     #frooty_max_xsubpixel - 1
    sta     ($fe),y
    jsr     .reverse_xspeed
.no_right_reverse
    inc     $fc                     ; Move a byte right.
    bne     .horizontal_done
    inc     $fd
.horizontal_done
    ldy     #frooty_ysub            ; Check for vertical move.
    lda     ($fe),y
    bmi     .move_up                ; If negative, move a pixel up.
    cmp     #frooty_max_ysubpixel   ; If over the subpixel cap,
    bpl     .move_down              ; move a pixel down.
    sta     ($fe),y                 ; Just update the subpixel.
    jmp     .vertical_done
.move_up
    clc
    adc     #frooty_max_ysubpixel   ; Adjust the subpixel.
    sta     ($fe),y
    ldy     #frooty_ypos            ; Update the y position.
    lda     ($fe),y
    sec
    sbc     #1
    sta     ($fe),y
    cmp     #frooty_top_bound       ; Reverse direction if needed.
    bne     .no_up_reverse
    ldy     #frooty_ysub            ; Set the subpixel to 0.
    lda     #0
    sta     ($fe),y
    jsr     .reverse_yspeed
.no_up_reverse
    lda     $fc                     ; Move the address up a row.
    sec
    sbc     #40
    sta     $fc
    bcs     .horizontal_done        ; Re-check, might need another
    dec     $fd                     ; move.
    bne     .horizontal_done        ; Always true.
.move_down
    sec
    sbc     #frooty_max_ysubpixel   ; Adjust the subpixel.
    sta     ($fe),y
    ldy     #frooty_ypos            ; Update the y position.
    lda     ($fe),y
    clc
    adc     #1
    sta     ($fe),y
    cmp     #frooty_bottom_bound    ; Reverse direction if needed.
    bne     .no_down_reverse
    ldy     #frooty_ysub            ; Set the subpixel to max - 1.
    lda     #frooty_max_ysubpixel - 1
    sta     ($fe),y
    jsr     .reverse_yspeed
.no_down_reverse
    lda     $fc                     ; Update the base address.
    clc
    adc     #40
    sta     $fc
    bcc     .horizontal_done        ; Re-check, might need another
    inc     $fd                     ; move.
    bcs     .horizontal_done        ; Always true.
.vertical_done
    ldy     #frooty_address         ; Update the address in the
    lda     $fc                     ; data block.
    sta     ($fe),y
    iny
    lda     $fd
    sta     ($fe),y
    ldx     #0                      ; Image read index.
    lda     $fc                     ; Set up write address.
    sta     $fa
    lda     $fd
    sta     $fb
.write_outer_loop
    ldy     #0                      ; Write index.
.write_inner_loop
    lda     frooty_lollipop,x
    sta     ($fa),y
    iny
    inx
    cpy     #2
    bne     .write_inner_loop
    cpx     #32
    beq     .write_done
    jsr     .update_hmove_address
    jmp     .write_outer_loop
.write_done
    ldy     $f9                     ; Flag the sound.
    lda     #1                      ; Return code.
    rts
.update_hmove_address
    lda     $fa
    clc
    adc     #40
    sta     $fa
    bcc     .update_hmove_address_done
    inc     $fb
.update_hmove_address_done
    rts
.reverse_xspeed
    ldy     #frooty_xspeed
    .byte   $2c
.reverse_yspeed
    ldy     #frooty_yspeed
    lda     ($fe),y
    eor     #$ff
    clc
    adc     #1
    sta     ($fe),y
    inc     $f9                     ; Flag the sound.
    rts

frooty_cpu subroutine
    inc     cpu_state
    lda     p0_attack_data + frooty_xpos
    cmp     [#frooty_right_bound - #frooty_left_bound] / #2
    bcc     .object_on_left
    lda     p1_real_hpos            ; Move left to avoid it.
    sbc     #20
    sta     cpu_xdest
    lda     cpu_left_speed
    sta     cpu_hspeed
    jmp     .check_vertical
.object_on_left
    lda     p1_real_hpos            ; Move right to avoid it.
    adc     #20
    sta     cpu_xdest
    lda     cpu_right_speed
    sta     cpu_hspeed
.check_vertical
    lda     p0_attack_data + frooty_ypos
    cmp     [#frooty_bottom_bound - #frooty_top_bound] / #2
    bcc     .object_on_top
    lda     p1_vpos                 ; Move up to avoid it.
    sbc     #20
    sta     cpu_ydest
    lda     cpu_up_speed
    sta     cpu_vspeed
    rts
.object_on_top
    lda     p1_vpos                 ; Move down to avoid it.
    adc     #20
    sta     cpu_ydest
    lda     cpu_down_speed
    sta     cpu_vspeed
    rts

; Sprite images.
frooty_p1_sprite
    .byte       %00000011
    .byte       %00000001
    .byte       %00000011
    .byte       %00000010
    .byte       %00000000
    .byte       %00000000
    .byte       %01100000
    .byte       %11000000
    .byte       %10000000
    .byte       %10000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00100000
    .byte       %00000100
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000

frooty_p2_sprite 
    .byte       %00000000
    .byte       %00001000
    .byte       %00011100
    .byte       %00111100
    .byte       %00011110
    .byte       %00011100
    .byte       %00001000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000

frooty_p3_sprite 
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00100010
    .byte       %00010100
    .byte       %00011100
    .byte       %00001000
    .byte       %00011100
    .byte       %00011100
    .byte       %00111100
    .byte       %01011110
    .byte       %01010001
    .byte       %01100100
    .byte       %00110100
    .byte       %00010100
    .byte       %00000100
    .byte       %00000110
    .byte       %00000010

; Letter data for the select screen.
; See pink_b.s for details.
frooty_letter_data
    .word       big_letter_f,$3000,$29cc
    .byte       big_letter_f_width
    .word       big_letter_r,$3002,$29ce
    .byte       big_letter_r_width
    .word       big_letter_o,$3004,$29d0
    .byte       big_letter_o_width
    .word       big_letter_o,$3006,$29d2
    .byte       big_letter_o_width
    .word       big_letter_t,$3008,$29d4
    .byte       big_letter_t_width
    .word       big_letter_y,$300a,$29d6
    .byte       big_letter_y_width

frooty_game_row_offset      equ 20
frooty_game_portrait_offset equ 59 - frooty_game_row_offset

frooty_matrix_data
    .byte 116,124,116,124   ; Player positions.
    .byte 0,0,0,0           ; Missile positions.
    .byte $7c,$7c,$fc,$fc   ; Colors

frooty_matrix_image equ frooty_matrix_image_real_start - 16
frooty_matrix_image_real_start
    .byte %00000000         ; Player 0.
    .byte %00001100
    .byte %00011011
    .byte %00111111
    .byte %00011110
    .byte %00110100
    .byte %01110010
    .byte %01110000
    .byte %01000000
    .byte %01000000
    .byte %01000000
    .byte %01100000
    .byte %01110000
    .byte %00010100
    .byte %00010100
    .byte %00000000

    .byte %00100000         ; Player 1.
    .byte %01000000
    .byte %11110110
    .byte %01110110
    .byte %10101010
    .byte %10011010
    .byte %10010001
    .byte %10000001
    .byte %00000000
    .byte %00000010
    .byte %00000010
    .byte %00000010
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000

    .byte %00000000         ; Player 2.
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000001
    .byte %00001001
    .byte %00001000
    .byte %00000101
    .byte %00010111
    .byte %00011111
    .byte %00000111
    .byte %00000111
    .byte %00000011
    .byte %00000000
    .byte %00000000

    .byte %00000000         ; Player 3.
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %01000100
    .byte %01100100
    .byte %01110100
    .byte %11110100
    .byte %11111100
    .byte %11111100
    .byte %11101100
    .byte %11111100
    .byte %11101000
    .byte %11110000
    .byte %01100000

frooty_matrix_background
    .byte %00000000,%10100010,%10001010,%00000000
    .byte %00000010,%10101000,%10100010,%00000000
    .byte %00000010,%10101010,%10101010,%10101000
    .byte %00001010,%10101010,%10101010,%10101010
    .byte %00001010,%10101010,%10101010,%10101010
    .byte %00001010,%10101010,%10101010,%10101010
    .byte %00101010,%10101010,%10101010,%10101010
    .byte %00101010,%10101010,%10101010,%10101010
    .byte %00101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10100000
    .byte %00101010,%10101010,%10101010,%10101000
    .byte %00101010,%10101010,%10101010,%10101000
    .byte %00101010,%10101010,%10101010,%10101000
    .byte %00001010,%10101010,%10101010,%10101010
    .byte %00000010,%00100010,%10101010,%10101000
    .byte %00000010,%00000000,%10101010,%10000000

frooty_lollipop
    .byte %00000011,%11000000
    .byte %00111111,%11111100
    .byte %00111010,%10101000
    .byte %11101111,%11111111
    .byte %11101111,%10101010
    .byte %11101110,%11111100
    .byte %11101110,%11110000
    .byte %11101110,%11110000
    .byte %11101110,%11110000
    .byte %11101110,%11110000
    .byte %11101110,%11111100
    .byte %11101111,%10101010
    .byte %11101111,%11111111
    .byte %00111010,%10101000
    .byte %00111111,%11111100
frooty_lollipop_end
    .byte %00000011,%11000000

frooty_bounce_sound
    .byte $e3
    .byte $10,$2
    .byte $a,$2
    .byte $10,$2
    .byte $0

frooty_portrait
    include "frooty_portrait.s"

frooty_portrait_color_1 equ $e ; 239, 239, 239
frooty_portrait_color_2 equ $78 ; 132, 82, 255
frooty_portrait_color_3 equ $0 ; 0, 0, 0
frooty_portrait_color_4 equ $f0 ; 8, 0, 0
frooty_portrait_color_5 equ $38 ; 214, 82, 66
frooty_portrait_color_6 equ $e8 ; 107, 165, 0
frooty_portrait_color_7 equ $7e ; 247, 198, 255
frooty_portrait_color_9 equ $76 ; 107, 57, 255

frooty_portrait_table
    .byte 3 + 128
    .byte frooty_portrait_color_1
    .byte 3
    .byte frooty_portrait_color_4
    .byte frooty_portrait_color_1
    .byte frooty_portrait_color_4
    .byte 6 + 128
    .byte frooty_portrait_color_1
    .byte 3
    .byte frooty_portrait_color_3
    .byte frooty_portrait_color_3
    .byte frooty_portrait_color_4
    .byte 3 + 128
    .byte frooty_portrait_color_3
    .byte 4
    .byte frooty_portrait_color_1
    .byte frooty_portrait_color_4
    .byte frooty_portrait_color_1
    .byte frooty_portrait_color_3
    .byte 9 + 128
    .byte frooty_portrait_color_1
    .byte 1
    .byte frooty_portrait_color_3
    .byte 4 + 128
    .byte frooty_portrait_color_1
    .byte 3 + 128
    .byte frooty_portrait_color_3
    .byte 1
    .byte frooty_portrait_color_4
    .byte 4 + 128
    .byte frooty_portrait_color_1
    .byte 4 + 128
    .byte frooty_portrait_color_3
    .byte 6 + 128
    .byte frooty_portrait_color_2
    .byte 12
    .byte frooty_portrait_color_7
    .byte frooty_portrait_color_4
    .byte frooty_portrait_color_4
    .byte frooty_portrait_color_3
    .byte frooty_portrait_color_2
    .byte frooty_portrait_color_2
    .byte frooty_portrait_color_6
    .byte frooty_portrait_color_4
    .byte frooty_portrait_color_4
    .byte frooty_portrait_color_1
    .byte frooty_portrait_color_6
    .byte frooty_portrait_color_1
    .byte 6 + 128
    .byte frooty_portrait_color_4
    .byte 2
    .byte frooty_portrait_color_1
    .byte frooty_portrait_color_4
    .byte 4 + 128
    .byte frooty_portrait_color_1
    .byte 7
    .byte frooty_portrait_color_4
    .byte frooty_portrait_color_1
    .byte frooty_portrait_color_4
    .byte frooty_portrait_color_1
    .byte frooty_portrait_color_3
    .byte frooty_portrait_color_3
    .byte frooty_portrait_color_4
    .byte 6 + 128
    .byte frooty_portrait_color_3
    .byte 2
    .byte frooty_portrait_color_1
    .byte frooty_portrait_color_3
    .byte 3 + 128
    .byte frooty_portrait_color_4
    .byte 4 + 128
    .byte frooty_portrait_color_1
    .byte 1
    .byte frooty_portrait_color_4
    .byte 3 + 128
    .byte frooty_portrait_color_3
    .byte 4 + 128
    .byte frooty_portrait_color_1
    .byte 5 + 128
    .byte frooty_portrait_color_4
    .byte 8
    .byte frooty_portrait_color_7
    .byte frooty_portrait_color_2
    .byte frooty_portrait_color_2
    .byte frooty_portrait_color_7
    .byte frooty_portrait_color_2
    .byte frooty_portrait_color_3
    .byte frooty_portrait_color_3
    .byte frooty_portrait_color_4
    .byte 3 + 128
    .byte frooty_portrait_color_6
    .byte 1
    .byte frooty_portrait_color_4
    .byte 5 + 128
    .byte frooty_portrait_color_6
    .byte 4
    .byte frooty_portrait_color_9
    .byte frooty_portrait_color_3
    .byte frooty_portrait_color_4
    .byte frooty_portrait_color_1
    .byte 34 + 128
    .byte frooty_portrait_color_5
    .byte 2
    .byte frooty_portrait_color_1
    .byte frooty_portrait_color_1
    .byte 3 + 128
    .byte frooty_portrait_color_7
    .byte 7
    .byte frooty_portrait_color_4
    .byte frooty_portrait_color_4
    .byte frooty_portrait_color_7
    .byte frooty_portrait_color_2
    .byte frooty_portrait_color_7
    .byte frooty_portrait_color_4
    .byte frooty_portrait_color_4

