; This file is part of Magical Fairy Force.

; Magical Fairy Force is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Magical Fairy Force is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Magical Fairy Force.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2020 Average Software.

; Various interrupt routines.

; Keypad data.
key_debounce        equ $750
last_key_pressed    equ $751

; Player 1 portrait coloring interrupt.
game_p1_portrait_interrupt subroutine
    org     $ac00
    pha
    tya
    pha
    txa
    pha

    ldy     p1_portrait_color      ; Get the index into the color table.
    ldx     #9                     ; 10 loops.
.loop
    lda     p1_portrait_table_0,y
    sta     COLPF0
    lda     p1_portrait_table_1,y
    sta     COLPF1
    lda     p1_portrait_table_2,y
    sta     COLPF2
    sta     WSYNC
    dey
    dex
    bpl     .loop
    lda     #frame_color
    sta     COLPF0
    lda     p1_playfield_1_color    ; Now set player/missile positions
    sta     COLPF1                  ; and colors, plus playfield colors
    lda     p1_playfield_2_color
    sta     COLPF2
    lda     #wing_color             ; for the player 1 area.
    sta     COLPM0
    lda     p1_background_color
    sta     COLBK
    lda     #1
    sta     SIZEP0
    lda     #0
    sta     SIZEP2
    lda     p1_wing_hpos
    sta     HPOSP0
    lda     p1_hpos
    sta     HPOSP1
    sta     HPOSP2
    sta     HPOSP3
    lda     #shot_color
    sta     COLPF3
    lda     p1_color_1
    sta     COLPM1
    lda     p1_color_2
    sta     COLPM2
    lda     p1_color_3
    sta     COLPM3
    lda     p1m0_hpos
    sta     HPOSM0
    lda     p1m1_hpos
    sta     HPOSM1
    lda     p1m2_hpos
    sta     HPOSM2
    lda     p1m3_hpos
    sta     HPOSM3
    lda     p1_playfield_0_color    ; Switch the last playfield color
    sta     COLPF0                  ; after the colored frame.
    lda     #$8d
    sta     VDSLST

    pla
    tax
    pla
    tay
    pla
    rti

; Start the upper star zone.  Switch the background to
; black for the center of the screen and set the star color.
; Also, gather the collision data for player 1.
    org     $ac8d
    pha
    txa
    pha
    
    lda     #0
    ldx     #star_color
    sta     WSYNC
    sta     COLBK
    stx     COLPF1
    lda     P1PF
    ora     P2PF
    ora     P3PF
    sta     pf_high
    lda     #$b9
    sta     VDSLST

    pla
    tax
    pla
    rti

; Handle the fuel gauge coloring.
    org     $acb9
    pha
    
    lda     #star_color
    sta     WSYNC
    sta     COLPF0
    lda     M0PL
    ora     M1PL
    sta     mpl_high
    lda     #$f5
    sta     VDSLST
    lda     p1_fuel_color
    sta     COLPF1
    sta     WSYNC
    sta     WSYNC
    sta     WSYNC
    sta     WSYNC
    lda     p0_fuel_color
    sta     COLPF1
    lda     #star_color
    sta     WSYNC
    sta     WSYNC
    sta     WSYNC
    sta     COLPF1

    pla
    rti

; Player 0 position and color interrupt.
    org     $acf5
    pha

    lda     p0_wing_hpos
    sta     HPOSP0
    lda     p0_hpos
    sta     HPOSP1
    sta     HPOSP2
    sta     HPOSP3
    lda     p0_color_1
    sta     COLPM1
    lda     p0_color_2
    sta     COLPM2
    lda     p0_color_3
    sta     COLPM3
    lda     p0m0_hpos
    sta     HPOSM0
    lda     p0m1_hpos
    sta     HPOSM1
    lda     p0m2_hpos
    sta     HPOSM2
    lda     p0m3_hpos
    sta     HPOSM3
    lda     #$ad
    sta     VDSLST + 1
    lda     #$3f
    sta     VDSLST
    lda     p0_playfield_0_color        ; Playfield 1 is the star, and
    sta     COLPF0                      ; needs to wait until later.
    lda     p0_playfield_2_color
    sta     COLPF2

    pla
    rti

; Switch to the player 0 background color and
; switch the last playfield color.  Also, reset
; the collision registers.
    org     $ad3f
    pha
    txa
    pha

    lda     p0_background_color
    ldx     p0_playfield_1_color
    sta     WSYNC
    sta     COLBK
    stx     COLPF1
    sta     HITCLR
    lda     #$5b
    sta     VDSLST

    pla
    tax
    pla
    rti

; Position and color interrupts for player 0's power
; gauge and health indicators.
    org     $ad5b
    pha
    txa
    pha
    tya
    pha

    lda     #184
    sta     HPOSP0
    sta     HPOSP2
    lda     #frame_color
    sta     COLPF0
    lda     #192
    sta     HPOSP1
    sta     HPOSP3
    lda     p0_gauge_color
    sta     COLPM0
    sta     COLPM1
    lda     #0
    sta     COLPM2
    sta     COLPM3
    sta     SIZEP0
    lda     #1
    sta     SIZEP2
    lda     #health_color
    sta     COLPF3
    lda     #p0_health_hpos
    sta     HPOSM3
    lda     #p0_health_hpos + 4
    sta     HPOSM2
    lda     #p0_health_hpos + 8
    sta     HPOSM1
    lda     #p0_health_hpos + 12
    sta     HPOSM0
    lda     #background_color
    sta     COLBK
game_p0_portrait_interrupt subroutine
    ldy     p0_portrait_color
    ldx     #9                      ; 10 loops.
.loop
    lda     p0_portrait_table_0,y
    pha
    lda     p0_portrait_table_1,y
    pha
    lda     p0_portrait_table_2,y
    sta     WSYNC
    sta     COLPF2
    pla
    sta     COLPF1
    pla
    sta     COLPF0
    dey
    dex
    bpl     .loop
    lda     #frame_color
    sta     WSYNC
    sta     COLPF0
    
    pla
    tay
    pla
    tax
    pla
    rti

; Character select screen interrupts.
; Color the upper character portrait.
character_select_color_portrait_1 subroutine
    org     $ae00
    pha
    tya
    pha

    ldy     #58                     ; Only 59 rows, the first has
.loop                               ; already been drawn.
    sta     WSYNC
    lda     p1_portrait_table_0,y
    sta     COLPF0
    lda     p1_portrait_table_1,y
    sta     COLPF1
    lda     p1_portrait_table_2,y
    sta     COLPF2
    dey                             ; This interrupt is too slow to
    bpl     .loop                   ; set the colors on the first row,
    lda     #$26                    ; since the portrait is on the far
    sta     VDSLST                  ; right, we can set the colors after
                                    ; the sync.
    pla
    tay
    pla
    rti

; Change to the big letter colors.
    org     $ae26
    pha
    txa
    pha

    lda     #big_letter_color_1
    ldx     #big_letter_color_2
    sta     WSYNC
    sta     COLPF2
    stx     COLPF1
    lda     #$3f
    sta     VDSLST

    pla
    tax
    pla
    rti

; Change to the matrix colors.
    org     $ae3f
    pha

    lda     #neutral_cell_color
    sta     WSYNC
    sta     COLPF0
    lda     #0
    sta     COLPF1
    lda     active_player_color
    sta     COLPF2
    lda     #$59
    sta     VDSLST

    pla
    rti

; Switch to the lower matrix portrait player/missile positions.
    org     $ae59
    pha

    lda     p0_matrix_pos_low
    sta     HPOSP0
    lda     p1_matrix_pos_low
    sta     HPOSP1
    lda     p2_matrix_pos_low
    sta     HPOSP2
    lda     p3_matrix_pos_low
    sta     HPOSP3
    lda     m0_matrix_pos_low
    sta     HPOSM0
    lda     m1_matrix_pos_low
    sta     HPOSM1
    lda     m2_matrix_pos_low
    sta     HPOSM2
    lda     m3_matrix_pos_low
    sta     HPOSM3
    lda     #$89
    sta     VDSLST

    pla
    rti

; Change back to the big letter colors.
    org     $ae89
    pha
    txa
    pha

    lda     #big_letter_color_1
    ldx     #big_letter_color_2
    sta     WSYNC
    sta     COLPF2
    stx     COLPF1
    lda     #$a2
    sta     VDSLST

    pla
    tax
    pla
    rti

; Color the player 0 portrait.
character_select_color_portrait_0 subroutine
    org     $aea2
    pha
    tya
    pha
    txa
    pha
    
    ldy     #59
.loop
    lda     p0_portrait_table_0,y
    pha
    lda     p0_portrait_table_1,y
    tax
    lda     p0_portrait_table_2,y
    sta     WSYNC
    sta     COLPF2
    stx     COLPF1
    pla
    sta     COLPF0
    dey
    bpl     .loop

    pla
    tax
    pla
    tay
    pla
    rti

; Keypad interrupt.
    org     $af00
keypad_interrupt subroutine
    ldy     key_debounce
    bne     .wait_on_debounce
    cmp     #$ff
    beq     .store_key
    mvy     #30,key_debounce
.store_key
    sta     last_key_pressed
.wait_on_debounce
    jmp     ENDINT

