; This file is part of Magical Fairy Force.

; Magical Fairy Force is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Magical Fairy Force is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Magical Fairy Force.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2020 Average Software.

; Kitty character data and logic routines

; Kitty's attack flow:

; Cat paws descend on the extreme edges of the
; playfield, spraying single pixel litter bits
; across the screen.  One paw only targets even
; rows, the other odd to prevent any weird collision
; problems.

; Paws will descend roughly half a playfield height
; apart, 40 rows.  This will allow for offscreen 
; portions of the paws to clear correctly.

; Kitty paw data.
kitty_paw_active        equ 0
kitty_paw_address       equ 1
kitty_paw_updater       equ 3
kitty_paw_next_row      equ 5
kitty_paw_size          equ 7

; Kitty shot data.
kitty_shot_offset       equ 0
kitty_shot_address      equ 1
kitty_shot_size         equ 3

; Attack constants.
kitty_attack_cost       equ -max_fairy_power
kitty_spawn_length      equ 39
kitty_cooldown_length   equ 20
kitty_shot_count        equ 20
kitty_total_paws        equ 6
kitty_color_0           equ $0f
kitty_color_1           equ 0

; Attack block data offsets.
kitty_spawn_ticks       equ 0
kitty_paw_count         equ 1
kitty_shot_cooldown     equ 2
kitty_next_paw          equ 3
kitty_paw_1             equ 4
kitty_paw_2             equ kitty_paw_1 + kitty_paw_size
kitty_left_shots        equ kitty_paw_2 + kitty_paw_size
kitty_right_shots       equ kitty_left_shots + kitty_shot_count * kitty_shot_size

kitty_start_attack subroutine
    beq     .attack_player_0
    lda     #kitty_color_0
    sta     p1_playfield_0_color
    lda     #kitty_color_1
    sta     p1_playfield_1_color
    ldx     #kitty_attack_cost
    jsr     general_player_0_attack_start
    lda     #>p0_attack_data        ; Branch is always true.
    bne     kitty_init_attack_block
.attack_player_0
    lda     #kitty_color_0
    sta     p0_playfield_0_color
    lda     #kitty_color_1
    sta     p0_playfield_1_color
    ldx     #kitty_attack_cost
    jsr     general_player_1_attack_start
    lda     #>p1_attack_data
    ; Fall through.

; Initialize the Kitty attack block whose
; high byte is in the accumulator.
kitty_init_attack_block subroutine
    sta     $ff
    lda     #0
    sta     $fe
    ldy     #kitty_spawn_ticks
    lda     #1                      ; Spawn on first tick.
    sta     ($fe),y
    iny
    lda     #kitty_total_paws       ; Spawn count.
    sta     ($fe),y
    iny
    lda     #0                      ; No cooldown.
    sta     ($fe),y
    iny
    lda     #kitty_paw_1
    sta     ($fe),y
    iny
    lda     #-1                     ; Inactive paws.
    sta     ($fe),y
    ldy     #kitty_paw_2
    sta     ($fe),y
    ldy     #kitty_right_shots + kitty_shot_count * kitty_shot_size - 1
    lda     #0                      ; Zero out the shot data.
.shot_loop
    sta     ($fe),y
    cpy     #kitty_left_shots
    beq     .shot_loop_done
    dey
    bne     .shot_loop              ; Always true.
.shot_loop_done
    jmp     start_attack_return

kitty_process_attack subroutine
    bne     kitty_attack_player_1
    jmp     kitty_attack_player_0

kitty_attack_player_1 subroutine
    lda     #2
    sta     p1_cloud_timer          ; Suppress cloud spawns.
    lda     pf_high
    beq     .no_damage
    jsr     damage_player_1
.no_damage
    dec     p0_attack_data + kitty_spawn_ticks
    bne     .no_spawn
    lda     #kitty_spawn_length     ; Reset the timer.
    sta     p0_attack_data + kitty_spawn_ticks
    lda     p0_attack_data + kitty_paw_count
    bpl     .continue_attack        ; On -1, disable the second paw.
    cmp     #-3                     ; Wait three cycles for shots to clear.
    beq     .end_attack
    sta     p0_attack_data + kitty_paw_2 + kitty_paw_active
    bne     .update_paw_count       ; Always true.
.end_attack
    lda     #>p0_attack_data
    sta     $ff
    jsr     kitty_erase_paws
    jmp     end_player_0_attack
.continue_attack
    bne     .perform_spawn          ; On 0, spawns are done.
    lda     #-1                     ; Deactivate the first paw.
    sta     p0_attack_data + kitty_paw_1 + kitty_paw_active
    bmi     .update_paw_count       ; Always true.
.perform_spawn
    ldy     p0_attack_data + kitty_next_paw
    sty     $fe
    lda     #>p0_attack_data
    sta     $ff
    lda     #0
    sta     p0_attack_data,y
    iny
    cpy     #kitty_paw_2
    bpl     .init_paw_2
    lda     #kitty_paw_2
    sta     p0_attack_data + kitty_next_paw
    lda     #$c8
    bne     .store_low_byte         ; Always true.
.init_paw_2
    lda     #kitty_paw_1
    sta     p0_attack_data + kitty_next_paw
    lda     #$ee
.store_low_byte
    sta     p0_attack_data,y
    iny
    lda     #$2a
    sta     p0_attack_data,y
    iny
    lda     <#kitty_update_address_subtract
    sta     p0_attack_data,y
    iny
    lda     >#kitty_update_address_subtract
    sta     p0_attack_data,y
    iny
    lda     <#kitty_update_address_add
    sta     p0_attack_data,y
    iny
    lda     >#kitty_update_address_add
    sta     p0_attack_data,y
    jsr     kitty_place_paw
.update_paw_count
    dec     p0_attack_data + kitty_paw_count
.no_spawn
    mva     #>p0_attack_data,$ff
    jsr     kitty_process_shots
    jsr     get_player_0_random
    jsr     kitty_process_paws
    beq     .no_sound
    mwa     kitty_litter_sound,$fe
    jsr     play_sound_3
.no_sound
    jmp     p0_process_return

kitty_attack_player_0 subroutine
    lda     #2
    sta     p0_cloud_timer          ; Suppress cloud spawns.
    lda     pf_low
    beq     .no_damage
    jsr     damage_player_0
.no_damage
    dec     p1_attack_data + kitty_spawn_ticks
    bne     .no_spawn
    lda     #kitty_spawn_length     ; Reset the timer.
    sta     p1_attack_data + kitty_spawn_ticks
    lda     p1_attack_data + kitty_paw_count
    bpl     .continue_attack        ; On -1, disable the second paw.
    cmp     #-3                     ; Wait three cycles for shots to clear.
    beq     .end_attack
    sta     p1_attack_data + kitty_paw_2 + kitty_paw_active
    bne     .update_paw_count       ; Always true.
.end_attack
    lda     #>p1_attack_data
    sta     $ff
    jsr     kitty_erase_paws
    jmp     end_player_1_attack
.continue_attack
    bne     .perform_spawn          ; On 0, spawns are done.
    lda     #-1                     ; Deactivate the first paw.
    sta     p1_attack_data + kitty_paw_1 + kitty_paw_active
    bmi     .update_paw_count       ; Always true.
.perform_spawn
    ldy     p1_attack_data + kitty_next_paw
    sty     $fe
    lda     #>p1_attack_data
    sta     $ff
    lda     #0
    sta     p1_attack_data,y
    iny
    cpy     #kitty_paw_2
    bpl     .init_paw_2
    lda     #kitty_paw_2
    sta     p1_attack_data + kitty_next_paw
    lda     #$18
    bne     .store_low_byte         ; Always true.
.init_paw_2
    lda     #kitty_paw_1
    sta     p1_attack_data + kitty_next_paw
    lda     #$dd
.store_low_byte
    sta     p1_attack_data,y
    iny
    lda     #$31
    sta     p1_attack_data,y
    iny
    lda     <#kitty_update_address_add
    sta     p1_attack_data,y
    iny
    lda     >#kitty_update_address_add
    sta     p1_attack_data,y
    iny
    lda     <#kitty_update_address_subtract
    sta     p1_attack_data,y
    iny
    lda     >#kitty_update_address_subtract
    sta     p1_attack_data,y
    jsr     kitty_place_paw
.update_paw_count
    dec     p1_attack_data + kitty_paw_count
.no_spawn
    mva     #>p1_attack_data,$ff
    jsr     kitty_process_shots
    jsr     get_player_1_random
    jsr     kitty_process_paws
    beq     .no_sound
    mwa     kitty_litter_sound,$fe
    jsr     play_sound_4
.no_sound
    jmp     p1_process_return

; Address updaters for the paw drawing routine.
kitty_update_address_subtract subroutine
    bcc     .one_row                ; The carry bit is set
    lda     $fb                     ; before this call.  In
    sta     $fd                     ; this mode, two rows are
    lda     $fa                     ; being updated for movement.
    sta     $fc
    sec
    sbc     #40
    sta     $fa
    bcs     .done
    dec     $fb
    rts
.one_row
    lda     $fc                     
    sec
    sbc     #40
    sta     $fc
    bcs     .done
    dec     $fd
.done
    rts

kitty_update_address_add subroutine
    bcc     .one_row                ; See above.
    lda     $fb
    sta     $fd
    lda     $fa
    sta     $fc
    clc
    adc     #40
    sta     $fa
    bcc     .done
    inc     $fb
    rts
.one_row
    lda     $fc
    clc
    adc     #40
    sta     $fc
    bcc     .done
    inc     $fd
.done
    rts

; Process the two kitty paws.  Expects the high byte of the
; player's attack block in $ff.  Pass a random number from
; the player's generator in the accumulator.  Clears the
; zero bit on return if the sound should be played.
kitty_process_paws subroutine
    ldy     #0                      ; Set the data block
    sty     $fe                     ; low byte.
    sty     $f4                     ; Sound flag.
    and     #$03                    ; 25% chance of litter.
    tax
    ldy     #kitty_shot_cooldown
    lda     ($fe),y                 ; Check the cooldown.
    beq     .no_cooldown
    sec                             ; Drop the cooldown.
    sbc     #1
    sta     ($fe),y
    jmp     .no_shot_this_frame
.no_cooldown
    cpx     #0                      ; Use a 2 frame cooldown on
    beq     .shot_this_frame        ; no shot to keep the shots
    lda     #2                      ; interleaved.
    sta     ($fe),y
    bne     .no_shot_this_frame     ; Always true.
.shot_this_frame
    inc     $f4                     ; Set the sound trigger.
    lda     #kitty_cooldown_length
    sta     ($fe),y
    .byte   $2c
.no_shot_this_frame
    ldx     #1                      ; Replace the shot flag.
    txa                             ; Store the shot flag.
    pha
    ldy     #kitty_paw_1
    sty     $fe
.loop
    ldy     #kitty_paw_active
    lda     ($fe),y                 ; Check for activity.
    bmi     .next_paw               ; Inactive.
    pla                             ; Shoot if necessary.
    bne     .no_shot
    lda     #kitty_paw_2            ; Check the paw identity.
    cmp     $fe
    bne     .left_paw_shoot
    ldy     #kitty_right_shots      ; Find a position for the shot.
    jsr     kitty_find_open_shot
    sty     $f0                     ; Save the shot index.
    ldy     #kitty_paw_2 + #kitty_paw_address
    lda     ($fe),y                 ; Get the shot start address.
    sec                             ; Offset is calculated from prior row.
    sbc     #40
    sta     $fc
    iny
    lda     ($fe),y
    sbc     #0
    sta     $fd
    ldy     $f0                     ; Set the data in the shot block.
    lda     #39
    sta     ($fe),y                 ; Offset.
    iny
    lda     $fc
    sta     ($fe),y
    iny
    lda     $fd
    sta     ($fe),y
    lda     #kitty_paw_2            ; Replace the paw address.
    sta     $fe
    ldy     #39                     ; Shot offset.
    lda     #$01                    ; Rightmost pixel set.
    bne     .shot_done              ; Always true.
.left_paw_shoot
    ldy     #kitty_left_shots       ; Find a position for the shot.
    jsr     kitty_find_open_shot
    sty     $f0                     ; Save the shot index.
    ldy     #kitty_paw_1 + #kitty_paw_address
    lda     ($fe),y                 ; Get the shot start address.
    sta     $fc
    iny
    lda     ($fe),y
    sta     $fd
    ldy     $f0                     ; Set the data in the shot block.
    lda     #2
    sta     ($fe),y                 ; Offset.
    iny
    lda     $fc
    sta     ($fe),y                 ; Address.
    iny
    lda     $fd
    sta     ($fe),y
    lda     #kitty_paw_1            ; Replace the paw address.
    sta     $fe
    ldy     #2                      ; Shot offset.
    lda     #$80                    ; Leftmost pixel set.
.shot_done
    sta     ($fc),y
    lda     #0                      ; Restore the shot flag.
.no_shot
    pha                             ; Save the flag again.
    jsr     kitty_move_paw
.next_paw
    lda     #kitty_paw_2            ; See which paw was processed.
    cmp     $fe
    beq     .done
    sta     $fe                     ; Do paw 2 now.
    jmp     .loop
.done
    pla                             ; Clear the shot flag.
    ldy     $f4                     ; Get the sound trigger.
    rts
    
; Migrate the kitty paws one row.  Expects the
; paws pointer in $fe-$ff.
kitty_move_paw subroutine
    jsr     kitty_setup_paw_pointers
    clc                             ; Operation flag.
    jsi     ($f0)                   ; Set the base write address.
    ldy     #kitty_paw_address      ; Update the address in the
    lda     $fc                     ; source paw data.
    sta     ($fe),y
    iny
    lda     $fd
    sta     ($fe),y
    ldx     #9                      ; Copy 9 rows.
    ldy     #0                      ; Read/write index.
.loop
    lda     ($fa),y                 ; Read the source byte.
    sta     ($fc),y                 ; Write it to the next row.
    iny                             ; Two bytes per row.
    lda     ($fa),y                 ; Read the second byte.
    sta     ($fc),y                 ; Write the second byte.
    dex                             ; Check the row count.
    beq     .done
    sec                             ; Update both rows.
    jsi     ($f2)                   ; Move to the next row.
    dey                             ; Read index to 0.
    beq     .loop                   ; Always true.
.done
    rts

; Do the initial placement of a kitty paw on the playfied.
; Expects the paw's pointer in $fe-$ff.
kitty_place_paw subroutine
    jsr     kitty_setup_paw_pointers
    ldx     #15                     ; Image read index.
    ldy     #1                      ; Write index.
.loop
    lda     kitty_paw,x             ; Get a byte.
    sta     ($fc),y                 ; Write it.
    dex                             ; Next read index.
    dey                             ; Next write index.
    lda     kitty_paw,x             ; Get a byte.
    sta     ($fc),y                 ; Write it.
    dex                             ; Done after 16 bytes.
    bmi     .done
    clc                             ; Operation flag.
    jsi     ($f2)                   ; Update the write address.
    iny                             ; Restore the write index.
    bne     .loop                   ; Always true.
.done
    rts

; Erase the paws for the player data block in $ff.
kitty_erase_paws subroutine
    lda     #kitty_paw_1
    sta     $fe
.paw_loop
    jsr     kitty_setup_paw_pointers
    ldx     #8                      ; Erase 8 rows.
    ldy     #0                      ; Write index.
.erase_loop
    tya                             ; Get 0.
    sta     ($fc),y
    iny
    sta     ($fc),y
    dex                             ; Check the row count.
    beq     .done_erase_loop
    clc                             ; Operation flag.
    jsi     ($f2)
    dey                             ; Restore the write address.
    beq     .erase_loop             ; Always true.
.done_erase_loop
    lda     $fe
    cmp     #kitty_paw_1
    bne     .done
    lda     #kitty_paw_2
    sta     $fe
    bne     .paw_loop               ; Always true.
.done
    rts

; Common code for setting up paw addresses.
kitty_setup_paw_pointers subroutine
    ldy     #kitty_paw_address      ; Get the base address.
    lda     ($fe),y
    sta     $fc                     ; Store two copies, the move function
    sta     $fa                     ; needs both.
    iny
    lda     ($fe),y
    sta     $fd
    sta     $fb
    iny                             ; Get the updater address.
    lda     ($fe),y
    sta     $f0
    iny
    lda     ($fe),y
    sta     $f1
    iny                             ; Get the row advance address.
    lda     ($fe),y
    sta     $f2
    iny
    lda     ($fe),y
    sta     $f3
    rts

; Find the first inactive shot slot
; in a shot list.  Expects the high byte
; of the player's attack block pointer in
; $ff and the offset to the list start in y.
kitty_find_open_shot subroutine
    lda     #0                      ; Set the low block byte.
    sta     $fe
.loop
    lda     ($fe),y
    beq     .done
    iny                             ; Go to the next shot.
    iny
    iny
    bne     .loop                   ; Always true.
.done
    rts

; Process the shots from the paws.
; Expects the high byte of the 
; player's data block in $ff.
kitty_process_shots subroutine
    lda     $ff                     ; Get the data high byte.
    sta     $fd                     ; Set up parallel shot lists.
    lda     #kitty_left_shots
    sta     $fe
    lda     #kitty_right_shots
    sta     $fc
    ldy     #0                      ; Shot index.
    sty     $f9                     ; Save it.
.loop
    lda     ($fe),y                 ; Check the left shot.
    beq     .left_shot_done         ; Inactive shot on 0 offset.
    tax                             ; Save the offset.
    iny                             ; Get the shot location.
    lda     ($fe),y
    sta     $fa
    iny
    lda     ($fe),y
    sta     $fb
    txa                             ; Retrieve the offset.
    tay
    lda     ($fa),y                 ; Get the byte.
    lsr                             ; Shift two pixels over.
    lsr
    lsr
    lsr
    sta     ($fa),y
    bne     .left_shot_updated
    cpy     #37                     ; See if the shot has completed.
    bne     .left_shot_continue
    ldy     $f9
    lda     #0                      ; Deactivate the shot.
    sta     ($fe),y
    beq     .left_shot_done         ; Always true.
.left_shot_continue
    iny                             ; Move the shot to the next byte.
    lda     #$40
    sta     ($fa),y
    tya
    ldy     $f9                     ; Update the offset in the block.
    sta     ($fe),y
.left_shot_updated
    ldy     $f9                     ; Get the shot index back.
.left_shot_done
    lda     ($fc),y                 ; Check the right shot.
    beq     .right_shot_done        ; Inactive shot on 0 offset.
    tax                             ; Save the offset.
    iny                             ; Get the shot location.
    lda     ($fc),y
    sta     $fa
    iny
    lda     ($fc),y
    sta     $fb
    txa                             ; Retrieve the offset.
    tay
    lda     ($fa),y                 ; Get the byte.
    asl                             ; Shift two pixels over.
    asl
    asl
    asl
    sta     ($fa),y
    bne     .right_shot_updated
    cpy     #4                      ; See if the shot has completed.
    bne     .right_shot_continue
    ldy     $f9
    lda     #0                      ; Deactivate the shot.
    sta     ($fc),y
    beq     .right_shot_done        ; Always true.
.right_shot_continue
    dey                             ; Move the shot to the prior byte.
    lda     #$01
    sta     ($fa),y
    tya
    ldy     $f9                     ; Update the offset in the block.
    sta     ($fc),y
.right_shot_updated
    ldy     $f9                     ; Get the shot index back.
.right_shot_done
    cpy     [#kitty_shot_count - 1] * kitty_shot_size
    beq     .done                   ; Check for completion.
    iny                             ; Update the shot index.
    iny
    iny
    sty     $f9
    bne     .loop                   ; Always true.
.done
    rts

kitty_cpu subroutine
    inc     cpu_state
    lda     #hpos_center            ; Head for the center.
    sta     cpu_xdest
    cmp     p1_real_hpos
    bcs     .move_right
    lda     cpu_left_speed
    .byte   $2c
.move_right
    lda     cpu_right_speed
    sta     cpu_hspeed
    ldy     #kitty_left_shots + #kitty_shot_offset
.shot_loop
    lda     p0_attack_data,y
    beq     .next_shot              ; Inactive, skip it.
    cmp     #18
    bcs     .next_shot              ; Too far right, don't care.
    cmp     #5
    bcc     .next_shot              ; Too far away, don't care.
    lda     p0_attack_data + 2,y    ; Check the address.
    cmp     #$25
    bcs     .lower_half             ; Move based on the vertical half.
    lda     #p1_bottom_move_bound
    sta     cpu_ydest
    ldy     cpu_down_speed
    iny
    sty     cpu_vspeed
    rts
.lower_half
    lda     #p1_top_bound
    sta     cpu_ydest
    ldy     cpu_up_speed
    dey
    sty     cpu_vspeed
    rts
.next_shot
    cpy     #kitty_right_shots - #kitty_shot_size
    beq     .done
    iny
    iny
    iny
    bne     .shot_loop              ; Always true.
.done
    lda     p1_vpos                 ; No shot found, hold position.
    sta     cpu_ydest
    lda     #0
    sta     cpu_vspeed
    rts

; Sprite images.
kitty_p1_sprite
    .byte       %10000001
    .byte       %11000001
    .byte       %01010101
    .byte       %01011101
    .byte       %01011101
    .byte       %01111110
    .byte       %00100010
    .byte       %00111100
    .byte       %00011100
    .byte       %00011100
    .byte       %00011100
    .byte       %00011100
    .byte       %00011100
    .byte       %00011100
    .byte       %00111110
    .byte       %00101010
    .byte       %00101010
    .byte       %01101011
    .byte       %01001000
    .byte       %00001000

kitty_p2_sprite
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00011100
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000

kitty_p3_sprite
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000

; Letter data for the select screen.
; See pink_b.s for details.
kitty_letter_data
    .word       big_letter_k,$3000,$29ce
    .byte       big_letter_k_width
    .word       big_letter_i,$3002,$29d0
    .byte       big_letter_i_width
    .word       big_letter_t,$3004,$29d2
    .byte       big_letter_t_width
    .word       big_letter_t,$3006,$29d4
    .byte       big_letter_t_width
    .word       big_letter_y,$3008,$29d6
    .byte       big_letter_y_width

kitty_game_row_offset       equ 17
kitty_game_portrait_offset  equ 59 - kitty_game_row_offset

kitty_matrix_data
    .byte 156,156,164,164   ; Player positions.
    .byte 0,0,0,0           ; Missile positions.
    .byte $46,$1e,$46,$1e   ; Colors.

kitty_matrix_image equ kitty_matrix_image_real_start - 16
kitty_matrix_image_real_start
    .byte %00110000         ; Player 0.
    .byte %00100000
    .byte %00100000
    .byte %00100000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00100000
    .byte %00100000
    .byte %00000000
    .byte %00010000
    .byte %00011000
    .byte %00001000

    .byte %00000000         ; Player 1.
    .byte %00000100
    .byte %00001110
    .byte %00001110
    .byte %00001010
    .byte %00001010
    .byte %00000111
    .byte %00000100
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000

    .byte %00000010         ; Player 2.
    .byte %00000010
    .byte %00000010
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %01000000
    .byte %00000000
    .byte %00000000
    .byte %01000000
    .byte %01110000
    .byte %11100000
    .byte %10000000
    .byte %00000000

    .byte %00000000         ; Player 3.
    .byte %00011000
    .byte %00011000
    .byte %00011100
    .byte %00011100
    .byte %00101100
    .byte %00101100
    .byte %00011100
    .byte %00001100
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000

kitty_matrix_background
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101000
    .byte %10101010,%10101010,%10101010,%10101000
    .byte %10101010,%10101010,%10101010,%10101000
    .byte %10101010,%10101010,%10101010,%10101000
    .byte %10101010,%10101010,%10101010,%10101000
    .byte %10101010,%10101010,%10101010,%10101000
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101000
    .byte %10101010,%10101010,%10101010,%10101000
    .byte %10101010,%10101010,%10101010,%10100000
    .byte %10101010,%10101010,%10101010,%10100000
    .byte %10101010,%10101010,%10101010,%10000000
    .byte %10101010,%10101010,%10101010,%10000000
    .byte %10101010,%10101010,%10101010,%00000000

kitty_paw
    .byte %00001010,%10100000
    .byte %10101010,%10101010
    .byte %10101010,%10101010
    .byte %10101010,%10101010
    .byte %10101010,%10101010
    .byte %10011001,%10011001
    .byte %00010001,%00010000
    .byte %00010001,%00010000

kitty_litter_sound
    .byte $0a
    .byte $02,$4
    .byte $0

kitty_portrait
    include "kitty_portrait.s"

kitty_portrait_color_1 equ $0 ; 0, 0, 0
kitty_portrait_color_2 equ $fe ; 255, 255, 74
kitty_portrait_color_3 equ $36 ; 189, 57, 41
kitty_portrait_color_4 equ $3c ; 255, 165, 148
kitty_portrait_color_5 equ $38 ; 214, 82, 66
kitty_portrait_color_6 equ $e ; 239, 239, 239
kitty_portrait_color_14 equ $2 ; 16, 16, 16
kitty_portrait_color_17 equ $32 ; 107, 0, 0
kitty_portrait_color_19 equ $22 ; 90, 8, 0

kitty_portrait_table
    .byte 60 + 128
    .byte kitty_portrait_color_1
    .byte 22 + 128
    .byte kitty_portrait_color_3
    .byte 5 + 128
    .byte kitty_portrait_color_5
    .byte 30 + 128
    .byte kitty_portrait_color_2
    .byte 6
    .byte kitty_portrait_color_3
    .byte kitty_portrait_color_4
    .byte kitty_portrait_color_4
    .byte kitty_portrait_color_19
    .byte kitty_portrait_color_1
    .byte kitty_portrait_color_17
    .byte 3 + 128
    .byte kitty_portrait_color_1
    .byte 1
    .byte kitty_portrait_color_17
    .byte 4 + 128
    .byte kitty_portrait_color_1
    .byte 1
    .byte kitty_portrait_color_14
    .byte 3 + 128
    .byte kitty_portrait_color_1
    .byte 7 + 128
    .byte kitty_portrait_color_6
    .byte 5 + 128
    .byte kitty_portrait_color_1
    .byte 3 + 128
    .byte kitty_portrait_color_5
    .byte 27 + 128
    .byte kitty_portrait_color_3
    .byte 3
    .byte kitty_portrait_color_2
    .byte kitty_portrait_color_3
    .byte kitty_portrait_color_1

