; This file is part of Magical Fairy Force.

; Magical Fairy Force is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Magical Fairy Force is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Magical Fairy Force.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2020 Average Software.

; Pink Barrette character data and logic routines

; Pink Barrette's attack flow:

; Large fists emerge from the background and
; attempt to strike the player.  They will
; appear in three steps, only the final
; step will register damage.

; Three fists will be in play at a time.  Their
; locations will be randomly determined.
; After sixteen fists have appeared, the attack
; will conclude.

; Fists will spawn every 15 frames.  They will
; also advance their phases on the same 15 frame
; intervals.

; Attack structure offsets.
pink_b_attack_phase     equ 0
pink_b_attack_address   equ 1
pink_b_attack_seed      equ 3
pink_b_attack_size      equ 4

; Attack constants.
pink_b_attack_cost      equ -max_fairy_power * 2 / 3
pink_b_spawn_length     equ 15
pink_b_total_attacks    equ 32
pink_b_active_attacks   equ 4       ; Three active at once, one waiting to spawn.
pink_b_fist_pink        equ $48

; Attack block offsets.
pink_b_spawn_ticks      equ 0
pink_b_attack_count     equ 1
pink_b_next_spawn       equ 2
pink_b_attack_data      equ 3

pink_b_start_attack subroutine
    beq     .attack_player_0
    ldx     #pink_b_attack_cost
    jsr     general_player_0_attack_start
    mva     #pink_b_fist_pink,p1_playfield_0_color
    mva     #0,p1_playfield_1_color
    sta     p1_playfield_2_color
    lda     #>p0_attack_data        ; Branch is always true.
    bne     pink_b_init_attack_block
.attack_player_0
    ldx     #pink_b_attack_cost
    jsr     general_player_1_attack_start
    mva     #pink_b_fist_pink,p0_playfield_0_color
    mva     #0,p0_playfield_1_color
    sta     p0_playfield_2_color
    lda     #>p1_attack_data
    ; Fall through.

; Initalize the attack block whose high
; byte is in $ff.
pink_b_init_attack_block subroutine
    sta     $ff
    lda     #0
    sta     $fe
    ldy     #pink_b_spawn_ticks
    lda     #1                      ; Spawn on first frame.
    sta     ($fe),y
    iny                             ; Attack count.
    lda     #pink_b_total_attacks
    sta     ($fe),y
    iny                             ; Next spawn index.
    lda     #0
    sta     ($fe),y
    ldx     #pink_b_active_attacks * #pink_b_attack_size
    lda     #-1                     ; -1 all the attack data.
.attack_loop
    iny
    sta     ($fe),y
    dex
    bne     .attack_loop
    jmp     start_attack_return

pink_b_process_attack subroutine
    bne     pink_b_attack_player_1
    jmp     pink_b_attack_player_0

pink_b_attack_player_1 subroutine
    lda     #2
    sta     p1_cloud_timer          ; Suppress cloud spawns.
    lda     pf_high                 ; Only check PF2.
    and     #$04
    beq     .no_damage
    jsr     damage_player_1
.no_damage
    dec     p0_attack_data + pink_b_spawn_ticks
    beq     .not_done
    jmp     .done
.not_done
    lda     #pink_b_spawn_length
    sta     p0_attack_data + pink_b_spawn_ticks
    mvx     #5,$ff                  ; Attack data high byte.
    lda     p0_attack_data + pink_b_attack_count
    beq     .process_attacks        ; Done spawning.
    dec     p0_attack_data + pink_b_attack_count
.seed_loop
    jsr     get_player_0_random
    jsr     pink_b_validate_attack_seed
    bne     .seed_loop
    lda     #pink_b_attack_data     ; Find the attack to spawn.
    ldy     p0_attack_data + pink_b_next_spawn
    clc
.spawn_loop
    dey
    bmi     .spawn_loop_done
    adc     #pink_b_attack_size
    bpl     .spawn_loop             ; Always true.
.spawn_loop_done
    sta     $fe                     ; $ff is still correct.
    ldy     #pink_b_attack_address  ; Set the base attack address.
    lda     #$a1
    sta     ($fe),y
    iny
    lda     #$20
    sta     ($fe),y
    lda     $fd                     ; Get the spawn seed.
    jsr     pink_b_randomize_fist
    ldy     p0_attack_data + pink_b_next_spawn
    cpy     #pink_b_active_attacks - #1
    bne     .no_spawn_overflow
    ldy     #-1                     ; Reset the spawn index.
.no_spawn_overflow
    iny
    sty     p0_attack_data + pink_b_next_spawn
    mwa     pink_b_punch_sound,$fe
    jsr     play_sound_3
.process_attacks
    mwa     [p0_attack_data + pink_b_attack_data],$fe
    mwa     pink_b_player_0_update_source_address,$f0
    mwa     pink_b_player_0_fist_offsets,$f2
    mva     #0,$f6                  ; Assume attack is over.
.attack_process_loop
    jsr     pink_b_process_fist
    bmi     .attack_inactive
    inc     $f6                     ; Attack will continue.
.attack_inactive
    lda     $fe
    cmp     #pink_b_attack_data + #pink_b_attack_size * [#pink_b_active_attacks - #1]
    beq     .attack_process_done
    clc
    adc     #pink_b_attack_size
    sta     $fe
    bne     .attack_process_loop    ; Always true.
.attack_process_done
    lda     $f6                     ; See if the attack is over.
    bne     .done                   ; Not over.
    jmp     end_player_0_attack
.done
    jmp     p0_process_return

pink_b_attack_player_0 subroutine
    lda     #2
    sta     p0_cloud_timer          ; Suppress cloud spawns.
    lda     pf_low                  ; Only check PF2.
    and     #$04
    beq     .no_damage
    jsr     damage_player_0
.no_damage
    dec     p1_attack_data + pink_b_spawn_ticks
    beq     .not_done
    jmp     .done
.not_done
    lda     #pink_b_spawn_length
    sta     p1_attack_data + pink_b_spawn_ticks
    mvx     #6,$ff                  ; Attack data high byte.
    lda     p1_attack_data + pink_b_attack_count
    beq     .process_attacks        ; Done spawning.
    dec     p1_attack_data + pink_b_attack_count
.seed_loop
    jsr     get_player_1_random
    jsr     pink_b_validate_attack_seed
    bne     .seed_loop
    lda     #pink_b_attack_data     ; Find the attack to spawn.
    ldy     p1_attack_data + pink_b_next_spawn
    clc
.spawn_loop
    dey
    bmi     .spawn_loop_done
    adc     #pink_b_attack_size
    bpl     .spawn_loop             ; Always true.
.spawn_loop_done
    sta     $fe                     ; $ff is still correct.
    ldy     #pink_b_attack_address  ; Set the base attack address.
    lda     #$09
    sta     ($fe),y
    iny
    lda     #$32
    sta     ($fe),y
    lda     $fd                     ; Get the spawn seed.
    jsr     pink_b_randomize_fist
    ldy     p1_attack_data + pink_b_next_spawn
    cpy     #pink_b_active_attacks - #1
    bne     .no_spawn_overflow
    ldy     #-1                     ; Reset the spawn index.
.no_spawn_overflow
    iny
    sty     p1_attack_data + pink_b_next_spawn
    mwa     pink_b_punch_sound,$fe
    jsr     play_sound_4
.process_attacks
    mwa     [p1_attack_data + pink_b_attack_data],$fe
    mwa     pink_b_player_1_update_source_address,$f0
    mwa     pink_b_player_1_fist_offsets,$f2
    mva     #0,$f6                  ; Assume attack is over.
.attack_process_loop
    jsr     pink_b_process_fist
    bmi     .attack_inactive
    inc     $f6                     ; Attack will continue.
.attack_inactive
    lda     $fe
    cmp     #pink_b_attack_data + #pink_b_attack_size * [#pink_b_active_attacks - #1]
    beq     .attack_process_done
    clc
    adc     #pink_b_attack_size
    sta     $fe
    bne     .attack_process_loop    ; Always true.
.attack_process_done
    lda     $f6                     ; See if the attack is over.
    bne     .done                   ; Not over.
    jmp     end_player_1_attack
.done
    jmp     p1_process_return

; Process the attack in $fe-$ff.  Expects
; a pointer to a player update routine in $f0-$f1 and
; a pointer to the image table in $f2-$f3.  Negative bit
; will be set if the attack was inactive.
pink_b_process_fist subroutine
    ldy     #pink_b_attack_phase
    lda     ($fe),y
    bpl     .attack_active
    rts                             ; Attack inactive, return.
.attack_active
    tax                             ; Save the phase.
    tay
    lda     ($f2),y                 ; Get the image offset.
    sta     $f5                     ; Image offset.
    ldy     #pink_b_attack_address  ; Move the target address to
    lda     ($fe),y                 ; $fc-$fd.
    sta     $fc
    iny
    lda     ($fe),y
    sta     $fd
    txa                             ; Check the phase.
    beq     .phase_0
    cmp     #1
    beq     .phase_1
    cmp     #2
    beq     .phase_2
    ldy     #pink_b_attack_phase    ; Disable the fist.
    lda     #-1
    sta     ($fe),y
    ldx     #16                     ; Clear 16 rows.
.clear_outer_loop
    ldy     #3                      ; Clear 4 bytes.
    lda     #0
.clear_inner_loop
    sta     ($fc),y
    dey
    bpl     .clear_inner_loop
    dex
    beq     .clear_done
    lda     $fc                     ; Update the address.
    clc
    adc     #40
    sta     $fc
    bcc     .clear_outer_loop
    inc     $fd
    bcs     .clear_outer_loop       ; Always true.
.clear_done
    rts
.phase_2
    ldy     #4                      ; Image width.
    ldx     #16                     ; Image height.
    mwa     pink_b_fist_2,$fa
    jmp     .start_loop
.phase_0
    ldy     #2                      ; Image width.
    ldx     #8                      ; Image height.
    mwa     pink_b_fist_0,$fa
    clc                             ; Subtract 81 instead of 80.
    jmp     .update_address
.phase_1
    ldy     #4                      ; Image width.
    ldx     #12                     ; Image height.
    mwa     pink_b_fist_1,$fa
    sec                             ; Only subtract 80.
.update_address
    sty     $f4                     ; Save the width.
    ldy     #pink_b_attack_address
    lda     ($fe),y
    sbc     #80                     ; Phase 0 clears the carry bit,
    sta     ($fe),y                 ; so this will subtract 81.
    iny
    lda     ($fe),y
    sbc     #0
    sta     ($fe),y
    .byte   $2c
.start_loop
    sty     $f4                     ; Save the width.
    lda     $f5                     ; Apply the image offset.
    clc
    adc     $fa
    sta     $fa
    bcc     pink_b_draw_fist_row_loop
    inc     $fb
pink_b_draw_fist_row_loop
    ldy     $f4                     ; Get the width.
.column_loop
    dey
    bmi     .row_done
    lda     ($fa),y                 ; Read an image byte.
    sta     ($fc),y                 ; Write it.
    jmp     .column_loop
.row_done
    dex
    beq     .done
    jmp     ($f0)                   ; Update the source address.
pink_b_draw_fist_update_target_address
    lda     $fc                     ; Update the target address.
    clc
    adc     #40
    sta     $fc
    bcc     pink_b_draw_fist_row_loop
    inc     $fd
    bcs     pink_b_draw_fist_row_loop
.done
    ldy     #pink_b_attack_phase    ; Update the phase.
    lda     ($fe),y
    clc
    adc     #1
    sta     ($fe),y
    tya                             ; Clear the negative bit.
    rts

; Row update routines for the fist draw operation.
pink_b_player_0_update_source_address subroutine
    lda     $fa
    sec
    sbc     $f4
    sta     $fa
    bcs     .no_borrow
    dec     $fb
.no_borrow
    jmp     pink_b_draw_fist_update_target_address

pink_b_player_1_update_source_address subroutine
    lda     $fa
    clc
    adc     $f4
    sta     $fa
    bcc     .no_carry
    inc     $fb
.no_carry
    jmp     pink_b_draw_fist_update_target_address

; Validate the random seed for a new fist.  Expects
; the random seed to be checked in the accumulator.
; Will mask off the high bit of the seed, since it
; isn't used.  Pass the high byte of the player's
; data block in $ff.  Sets the zero bit on success.
; The final random seed will be in $fd.
pink_b_validate_attack_seed subroutine
    ldy     #pink_b_attack_data     ; Set up the attack pointer.
    sty     $fe
    and     #$1f                    ; Mask off the top 3 bits.
    sta     $fd                     ; Save the seed for comparison.
    ldx     #pink_b_active_attacks  ; Loop counter.
.loop
    ldy     #pink_b_attack_phase
    lda     ($fe),y                 ; Check the phase.
    bmi     .next_attack            ; See if it's active.
    ldy     #pink_b_attack_seed     ; Check the seed.
    lda     ($fe),y
    cmp     $fd
    beq     .check_failed
.next_attack
    dex
    beq     .check_passed
    lda     $fe                     ; Go to the next attack.
    clc
    adc     #pink_b_attack_size
    sta     $fe
    bne     .loop                   ; Always true.
.check_failed
    lda     #1                      ; Failure flag.
.check_passed                       ; Zero is set if branching here.
    rts

; Set up the random address for a new fist attack.
; Expects a random number in the accumulator with the
; top 3 bits masked off, plus the attack pointer in $fe-$ff.
; The attack address should be initialized with the base address
; for the player's playfield area.
pink_b_randomize_fist subroutine
    ldy     #pink_b_attack_seed     ; Save the seed.
    sta     ($fe),y
    pha                             ; Save the random number.
    and     #$03                    ; Mask to 0-3.
    tax                             ; Loop counter.
    ldy     #pink_b_attack_address
.row_loop
    dex                             ; Check the counter.
    bmi     .row_done
    lda     ($fe),y                 ; Add $280 to the base address.
    clc
    adc     #$80
    sta     ($fe),y
    iny
    lda     ($fe),y
    adc     #2
    sta     ($fe),y
    dey
    jmp     .row_loop
.row_done
    pla                             ; Get the second random number.
    lsr                             ; Shift off two bits.
    lsr
    tax                             ; Loop counter.
.column_loop
    dex                             ; Check the counter.
    bmi     .column_done
    lda     ($fe),y                 ; Add 5 to the base address.
    clc
    adc     #5
    sta     ($fe),y
    iny
    lda     ($fe),y
    adc     #0
    sta     ($fe),y
    dey
    jmp     .column_loop
.column_done
    ldy     #pink_b_attack_phase    ; Initialize the attack phase.
    lda     #0
    sta     ($fe),y
    rts

pink_b_cpu subroutine
    lda     pf_high                 ; Check for playfield collisions.
    bne     .dodge_fist
    lda     #0                      ; Hold in place.
    sta     p1_h_input
    sta     p1_v_input
    rts
.dodge_fist
    inc     cpu_state
    lda     p1_real_hpos            ; Move 30 toward the center.
    cmp     #hpos_center
    bcc     .dodge_right
    adc     #29                     ; Carry is set.
    sta     cpu_xdest
    lda     cpu_right_speed
    sta     cpu_hspeed
    rts
.dodge_right
    sbc     #29                     ; Carry is clear.
    sta     cpu_xdest
    lda     cpu_left_speed
    sta     cpu_hspeed
    rts

; Sprite images.
pink_b_p1_sprite
    .byte       %01000010
    .byte       %10100101
    .byte       %01000010
    .byte       %10110100
    .byte       %01111110
    .byte       %01111100
    .byte       %01111100
    .byte       %00011000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000

pink_b_p2_sprite
    .byte       %00000000
    .byte       %00000000
    .byte       %00001000
    .byte       %00001000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00011000
    .byte       %00011000
    .byte       %11011001
    .byte       %11011011
    .byte       %00111101
    .byte       %01111110
    .byte       %00000000
    .byte       %00000000
    .byte       %10100001
    .byte       %11100111
    .byte       %11000110

pink_b_p3_sprite
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000010
    .byte       %10000011
    .byte       %00100110
    .byte       %00011000
    .byte       %00100110
    .byte       %01100011
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00101000
    .byte       %01101100
    .byte       %01000110
    .byte       %00000000
    .byte       %00000000

; Letter data for the select screen.
; Format is: address of big letter
;            target address for player 0
;            target address for player 1
;            byte width of big letter
pink_b_letter_data
    .word       big_letter_p,$3000,$29cc
    .byte       big_letter_p_width
    .word       big_letter_i,$3002,$29ce
    .byte       big_letter_i_width
    .word       big_letter_n,$3004,$29d0
    .byte       big_letter_n_width
    .word       big_letter_k,$3006,$29d2
    .byte       big_letter_k_width
    .word       big_letter_b,$3009,$29d5
    .byte       big_letter_b_width
    .word       big_letter_period,$300b,$29d7
    .byte       big_letter_period_width

pink_b_game_row_offset      equ 18
pink_b_game_portrait_offset equ 59 - pink_b_game_row_offset

pink_b_matrix_data
    .byte 96,104,97,105     ; Player positions.
    .byte 0,0,0,0           ; Missile positions.
    .byte $86,$86,$f6,$f6   ; Colors.

pink_b_matrix_image equ pink_b_matrix_image_real_start - 16
pink_b_matrix_image_real_start
    .byte %00000000         ; Player 0.
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00001111
    .byte %01111111
    .byte %11111111
    .byte %11111110
    .byte %11111100
    .byte %01110000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000

    .byte %00000000         ; Player 1.
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %11111000
    .byte %11111111
    .byte %00111111
    .byte %00001111
    .byte %00001111
    .byte %00000011
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000

    .byte %00000000         ; Player 2.
    .byte %00000000
    .byte %00000000
    .byte %00111111
    .byte %11111111
    .byte %01100000
    .byte %00000000
    .byte %00000001
    .byte %00000011
    .byte %00000111
    .byte %00011111
    .byte %11111111
    .byte %01111000
    .byte %00111100
    .byte %00001110
    .byte %11111111

    .byte %00000000         ; Player 3.
    .byte %00000000
    .byte %00000000
    .byte %01100000
    .byte %11110000
    .byte %00001000
    .byte %10000000
    .byte %11100000
    .byte %11110000
    .byte %11111000
    .byte %01111110
    .byte %11111110
    .byte %00111100
    .byte %00111000
    .byte %00110000
    .byte %11111100

pink_b_matrix_background
    .byte %10000000,%00000000,%00000000,%00000000
    .byte %10100010,%10000000,%00000000,%00000010
    .byte %10101010,%10101010,%10101010,%00000010
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %00101010,%10101010,%10101010,%10101010
    .byte %00101010,%10101010,%10101010,%10101000
    .byte %00101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %00101010,%10101010,%10101010,%10101010
    .byte %00101010,%10101010,%10101010,%10101010
    .byte %00001010,%10101010,%10101010,%10101000
    .byte %00000010,%10101010,%10101010,%10100000
    .byte %00000000,%10101010,%10101010,%10000000
    .byte %00101010,%10101010,%10101010,%10101000

pink_b_fist_0
    .byte %00001010,%10101010
    .byte %10101001,%01100110
    .byte %10011001,%01100110
    .byte %10011001,%10100110
    .byte %10011001,%10100110
    .byte %10101010,%10101010
    .byte %00000000,%10010110
pink_b_fist_0_reverse
    .byte %00000000,%10101010

pink_b_fist_1
    .byte %00000000,%00000000,%00101000,%00000000
    .byte %00000000,%00001010,%10010110,%10100000
    .byte %00000010,%10100101,%10010110,%01100000
    .byte %00001001,%01100101,%10010110,%01100000
    .byte %00001001,%01100101,%10010110,%01100000
    .byte %00001001,%01100101,%10010110,%01100000
    .byte %00001001,%01100101,%10010110,%01100000
    .byte %00001001,%01100101,%10010110,%01100000
    .byte %00000010,%10101010,%10101010,%10100000
    .byte %00000000,%00000000,%10010101,%01100000
    .byte %00000000,%00000000,%10010101,%01100000
pink_b_fist_1_reverse
    .byte %00000000,%00000000,%00101010,%10100000

pink_b_fist_2
    .byte %00000000,%00000000,%00111111,%00000000
    .byte %00000000,%00111111,%11010101,%11111111
    .byte %00111111,%11010101,%11010101,%11010111
    .byte %11010101,%11010101,%11010101,%11010111
    .byte %11010101,%11010101,%11010101,%11010111
    .byte %11010101,%11010101,%11010101,%11010111
    .byte %11010101,%11010101,%11010101,%11010111
    .byte %11010101,%11010101,%11010101,%11010111
    .byte %11010101,%11010101,%11010101,%11010111
    .byte %11010101,%11010101,%11010101,%11010111
    .byte %11110101,%11010101,%11010101,%11010111
    .byte %00001111,%11111111,%11111111,%11111111
    .byte %00000000,%00000000,%11010101,%01010111
    .byte %00000000,%00000000,%11010101,%01010111
    .byte %00000000,%00000000,%11010101,%01010111
pink_b_fist_2_reverse
    .byte %00000000,%00000000,%00111111,%11111111

pink_b_player_0_fist_offsets
    .byte pink_b_fist_0_reverse - pink_b_fist_0
    .byte pink_b_fist_1_reverse - pink_b_fist_1
    .byte pink_b_fist_2_reverse - pink_b_fist_2

pink_b_player_1_fist_offsets
    .byte $0,$0,$0

pink_b_punch_sound
    .byte $06
    .byte $10,$4
    .byte $b,$4
    .byte $0

pink_b_portrait
    include "pink_b_portrait.s"

pink_b_portrait_color_1 equ $0 ; 0, 0, 0
pink_b_portrait_color_2 equ $28 ; 189, 107, 0
pink_b_portrait_color_3 equ $38 ; 214, 82, 66
pink_b_portrait_color_4 equ $e ; 239, 239, 239
pink_b_portrait_color_6 equ $3c ; 255, 165, 148
pink_b_portrait_color_10 equ $a0 ; 0, 16, 33
pink_b_portrait_color_14 equ $3e ; 255, 198, 181

pink_b_portrait_table
    .byte 3
    .byte pink_b_portrait_color_3
    .byte pink_b_portrait_color_1
    .byte pink_b_portrait_color_2
    .byte 5 + 128
    .byte pink_b_portrait_color_1
    .byte 1
    .byte pink_b_portrait_color_4
    .byte 5 + 128
    .byte pink_b_portrait_color_1
    .byte 4
    .byte pink_b_portrait_color_4
    .byte pink_b_portrait_color_4
    .byte pink_b_portrait_color_1
    .byte pink_b_portrait_color_1
    .byte 5 + 128
    .byte pink_b_portrait_color_2
    .byte 22 + 128
    .byte pink_b_portrait_color_1
    .byte 1
    .byte pink_b_portrait_color_2
    .byte 15 + 128
    .byte pink_b_portrait_color_1
    .byte 2
    .byte pink_b_portrait_color_2
    .byte pink_b_portrait_color_1
    .byte 4 + 128
    .byte pink_b_portrait_color_2
    .byte 3
    .byte pink_b_portrait_color_4
    .byte pink_b_portrait_color_1
    .byte pink_b_portrait_color_4
    .byte 4 + 128
    .byte pink_b_portrait_color_2
    .byte 1
    .byte pink_b_portrait_color_1
    .byte 3 + 128
    .byte pink_b_portrait_color_2
    .byte 5 + 128
    .byte pink_b_portrait_color_1
    .byte 8 + 128
    .byte pink_b_portrait_color_2
    .byte 12 + 128
    .byte pink_b_portrait_color_3
    .byte 3
    .byte pink_b_portrait_color_2
    .byte pink_b_portrait_color_2
    .byte pink_b_portrait_color_1
    .byte 4 + 128
    .byte pink_b_portrait_color_2
    .byte 3 + 128
    .byte pink_b_portrait_color_3
    .byte 1
    .byte pink_b_portrait_color_2
    .byte 5 + 128
    .byte pink_b_portrait_color_6
    .byte 2
    .byte pink_b_portrait_color_1
    .byte pink_b_portrait_color_2
    .byte 5 + 128
    .byte pink_b_portrait_color_3
    .byte 1
    .byte pink_b_portrait_color_14
    .byte 3 + 128
    .byte pink_b_portrait_color_2
    .byte 6
    .byte pink_b_portrait_color_3
    .byte pink_b_portrait_color_14
    .byte pink_b_portrait_color_4
    .byte pink_b_portrait_color_4
    .byte pink_b_portrait_color_2
    .byte pink_b_portrait_color_1
    .byte 3 + 128
    .byte pink_b_portrait_color_4
    .byte 12 + 128
    .byte pink_b_portrait_color_3
    .byte 6
    .byte pink_b_portrait_color_2
    .byte pink_b_portrait_color_2
    .byte pink_b_portrait_color_1
    .byte pink_b_portrait_color_10
    .byte pink_b_portrait_color_1
    .byte pink_b_portrait_color_1
    .byte 4 + 128
    .byte pink_b_portrait_color_6
    .byte 2
    .byte pink_b_portrait_color_2
    .byte pink_b_portrait_color_2
    .byte 7 + 128
    .byte pink_b_portrait_color_3
    .byte 3 + 128
    .byte pink_b_portrait_color_2
    .byte 3
    .byte pink_b_portrait_color_6
    .byte pink_b_portrait_color_2
    .byte pink_b_portrait_color_2
    .byte 4 + 128
    .byte pink_b_portrait_color_1

