; This file is part of Magical Fairy Force.

; Magical Fairy Force is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Magical Fairy Force is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Magical Fairy Force.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2020 Average Software.

; Pork-Chop character data and logic routines

; Pork-Chop's attack flow:
; 10 alligators will cross the screen, five from the left
; and five from the right.  The alligators are staggered
; between the sides, so they will always be on different
; vertical levels.

; Two alligators will attack at a time, one from the left
; and one from the right.  The slot for each gator will be
; determined randomly.  The movement will be chunky, on a
; byte-by-byte basis using playfield sprites.

; Attack constants.
porkchop_attack_cost            equ -max_fairy_power * 2 / 3
porkchop_gator_move_time        equ 2

; Gator block offsets.
porkchop_gator_address          equ 0
porkchop_gator_sprite_index     equ 2
porkchop_gator_vpos             equ 3
porkchop_gator_block_size       equ 4

; Attack block data offsets.
porkchop_gator_ticks            equ 0
porkchop_gator_timer            equ 1
porkchop_wave_count             equ 2
porkchop_gator_loop_count       equ 3
porkchop_left_gator             equ 4
porkchop_right_gator            equ porkchop_left_gator + porkchop_gator_block_size

; Gator color constants.
porkchop_gator_color_0          equ $c6
porkchop_gator_color_1          equ $c8
porkchop_gator_color_2          equ $ca

porkchop_start_attack subroutine
    beq     .attack_player_0
    ldx     #porkchop_attack_cost
    jsr     general_player_0_attack_start
    lda     #5                      ; Set the wave count.
    sta     p0_attack_data + porkchop_wave_count
    lda     #1                      ; Cause a trigger on the next frame.
    sta     p0_attack_data + porkchop_gator_timer
    lda     #0                      ; Set the tick count.
    sta     p0_attack_data + porkchop_gator_ticks
    lda     #porkchop_gator_color_0
    sta     p1_playfield_0_color
    lda     #porkchop_gator_color_1
    sta     p1_playfield_1_color
    lda     #porkchop_gator_color_2
    sta     p1_playfield_2_color
    jmp     start_attack_return
.attack_player_0
    ldx     #porkchop_attack_cost
    jsr     general_player_1_attack_start
    lda     #5                      ; Set the wave count.
    sta     p1_attack_data + porkchop_wave_count
    lda     #1                      ; Cause a trigger on the next frame.
    sta     p1_attack_data + porkchop_gator_timer
    lda     #0                      ; Set the tick count.
    sta     p1_attack_data + porkchop_gator_ticks
    lda     #porkchop_gator_color_0
    sta     p0_playfield_0_color
    lda     #porkchop_gator_color_1
    sta     p0_playfield_1_color
    lda     #porkchop_gator_color_2
    sta     p0_playfield_2_color
    jmp     start_attack_return

porkchop_process_attack subroutine
    bne     porkchop_attack_player_1
    jmp     porkchop_attack_player_0

porkchop_attack_player_1 subroutine
    ldy     #2                      ; Suppress cloud spawns.
    sty     p1_cloud_timer
    dec     p0_attack_data + porkchop_gator_timer
    beq     .not_done               ; When timer is 0, move the gators.
    jmp     .check_hits
.not_done
    lda     #porkchop_gator_move_time
    sta     p0_attack_data + porkchop_gator_timer
    ldx     p0_attack_data + porkchop_gator_ticks
    inx
    cpx     #1                      ; On 1, spawn new gators.
    bne     .no_spawn
    jsr     porkchop_setup_player_0_attack
    jmp     .draw_gators
.no_spawn
    cpx     #48                     ; Wave isn't over.
    bne     .draw_gators
    lda     p0_attack_data + porkchop_left_gator + porkchop_gator_address
    sta     $fe
    lda     p0_attack_data + porkchop_left_gator + porkchop_gator_address + 1
    sta     $ff
    jsr     porkchop_alligator_erase
    lda     p0_attack_data + porkchop_right_gator + porkchop_gator_address
    sta     $fe
    lda     p0_attack_data + porkchop_right_gator + porkchop_gator_address + 1
    sta     $ff
    jsr     porkchop_alligator_erase
    dec     p0_attack_data + porkchop_wave_count
    beq     .end_attack
    lda     #0                      ; Reset tick count.
    sta     p0_attack_data + porkchop_gator_ticks
    lda     #1                      ; Reset the timer.
    sta     p0_attack_data + porkchop_gator_timer
    jmp     p0_process_return
.end_attack
    jmp     end_player_0_attack
.draw_gators
    stx     p0_attack_data + porkchop_gator_ticks
    cpx     #1                      ; On tick 1, no adjustment needed.
    beq     .draw_ready
    cpx     #41                     ; Below 41, normal advance.
    bpl     .exiting_gator
    inc     p0_attack_data + porkchop_left_gator + porkchop_gator_address
    bne     .left_gator_no_carry
    inc     p0_attack_data + porkchop_left_gator + porkchop_gator_address + 1
.left_gator_no_carry
    lda     p0_attack_data + porkchop_right_gator + porkchop_gator_address
    bne     .right_gator_no_borrow
    dec     p0_attack_data + porkchop_right_gator + porkchop_gator_address + 1
.right_gator_no_borrow
    dec     p0_attack_data + porkchop_right_gator + porkchop_gator_address
    cpx     #9                      ; Below 8, increase the length.
    bpl     .draw_ready
    inc     p0_attack_data + porkchop_gator_loop_count
    jmp     .draw_ready
.exiting_gator
    dec     p0_attack_data + porkchop_gator_loop_count
    dec     p0_attack_data + porkchop_left_gator + porkchop_gator_sprite_index
    inc     p0_attack_data + porkchop_right_gator + porkchop_gator_sprite_index
.draw_ready
    ldy     p0_attack_data + porkchop_gator_loop_count
    sty     $f9                     ; Draw loop count.
    ldy     p0_attack_data + porkchop_left_gator + porkchop_gator_sprite_index
    sty     $f8
    lda     p0_attack_data + porkchop_left_gator + porkchop_gator_address
    sta     $fe
    lda     p0_attack_data + porkchop_left_gator + porkchop_gator_address + 1
    sta     $ff
    mwa     porkchop_alligator_right,$fc
.left_draw_loop
    ldy     $f8
    jsr     porkchop_copy_alligator_reverse
    lda     $fe                     ; Update the address.
    bne     .left_draw_no_borrow
    dec     $ff
.left_draw_no_borrow
    dec     $fe
    dec     $f9
    beq     .left_erase
    dec     $f8                     ; Update the sprite index.
    jmp     .left_draw_loop
.left_erase
    ldx     p0_attack_data + porkchop_gator_ticks
    cpx     #8                      ; After 8 ticks, start erasing.
    bmi     .right_draw_setup
    jsr     porkchop_alligator_erase
.right_draw_setup
    ldy     p0_attack_data + porkchop_gator_loop_count
    sty     $f9                     ; Draw loop count.
    ldy     p0_attack_data + porkchop_right_gator + porkchop_gator_sprite_index
    sty     $f8
    lda     p0_attack_data + porkchop_right_gator + porkchop_gator_address
    sta     $fe
    lda     p0_attack_data + porkchop_right_gator + porkchop_gator_address + 1
    sta     $ff
    mwa     porkchop_alligator_left,$fc
.right_draw_loop
    ldy     $f8
    jsr     porkchop_copy_alligator_reverse
    inc     $fe                     ; Update the address.
    bne     .right_draw_no_carry
    inc     $ff
.right_draw_no_carry
    dec     $f9
    beq     .right_erase
    inc     $f8                     ; Update the sprite index.
    jmp     .right_draw_loop
.right_erase
    ldx     p0_attack_data + porkchop_gator_ticks
    cpx     #8                      ; After 8 ticks, start erasing.
    bmi     .check_hits
    jsr     porkchop_alligator_erase
.check_hits
    lda     pf_high                 ; Check for hits.
    beq     .done
    jsr     damage_player_1
.done
    jmp     p0_process_return

porkchop_attack_player_0 subroutine
    ldy     #2                      ; Suppress cloud spawns.
    sty     p0_cloud_timer
    dec     p1_attack_data + porkchop_gator_timer
    beq     .not_done               ; When timer is 0, move the gators.
    jmp     .check_hits
.not_done
    lda     #porkchop_gator_move_time
    sta     p1_attack_data + porkchop_gator_timer
    ldx     p1_attack_data + porkchop_gator_ticks
    inx
    cpx     #1                      ; On 1, spawn new gators.
    bne     .no_spawn
    jsr     porkchop_setup_player_1_attack
    jmp     .draw_gators
.no_spawn
    cpx     #48                     ; Wave isn't over.
    bne     .draw_gators
    lda     p1_attack_data + porkchop_left_gator + porkchop_gator_address
    sta     $fe
    lda     p1_attack_data + porkchop_left_gator + porkchop_gator_address + 1
    sta     $ff
    jsr     porkchop_alligator_erase
    lda     p1_attack_data + porkchop_right_gator + porkchop_gator_address
    sta     $fe
    lda     p1_attack_data + porkchop_right_gator + porkchop_gator_address + 1
    sta     $ff
    jsr     porkchop_alligator_erase
    dec     p1_attack_data + porkchop_wave_count
    beq     .end_attack
    lda     #0                      ; Reset tick count.
    sta     p1_attack_data + porkchop_gator_ticks
    lda     #1
    sta     p1_attack_data + porkchop_gator_timer
    jmp     p1_process_return
.end_attack
    jmp     end_player_1_attack
.draw_gators
    stx     p1_attack_data + porkchop_gator_ticks
    cpx     #1                      ; On tick 1, no adjustment needed.
    beq     .draw_ready
    cpx     #41                     ; Below 41, normal advance.
    bpl     .exiting_gator
    inc     p1_attack_data + porkchop_left_gator + porkchop_gator_address
    bne     .left_gator_no_carry
    inc     p1_attack_data + porkchop_left_gator + porkchop_gator_address + 1
.left_gator_no_carry
    lda     p1_attack_data + porkchop_right_gator + porkchop_gator_address
    bne     .right_gator_no_borrow
    dec     p1_attack_data + porkchop_right_gator + porkchop_gator_address + 1
.right_gator_no_borrow
    dec     p1_attack_data + porkchop_right_gator + porkchop_gator_address
    cpx     #9                      ; Below 8, increase the length.
    bpl     .draw_ready
    inc     p1_attack_data + porkchop_gator_loop_count
    jmp     .draw_ready
.exiting_gator
    dec     p1_attack_data + porkchop_gator_loop_count
    dec     p1_attack_data + porkchop_left_gator + porkchop_gator_sprite_index
    inc     p1_attack_data + porkchop_right_gator + porkchop_gator_sprite_index
.draw_ready
    ldy     p1_attack_data + porkchop_gator_loop_count
    sty     $f9                     ; Draw loop count.
    ldy     p1_attack_data + porkchop_left_gator + porkchop_gator_sprite_index
    sty     $f8
    lda     p1_attack_data + porkchop_left_gator + porkchop_gator_address
    sta     $fe
    lda     p1_attack_data + porkchop_left_gator + porkchop_gator_address + 1
    sta     $ff
    mwa     porkchop_alligator_right,$fc
.left_draw_loop
    ldy     $f8
    jsr     porkchop_copy_alligator
    lda     $fe                     ; Update the address.
    bne     .left_draw_no_borrow
    dec     $ff
.left_draw_no_borrow
    dec     $fe
    dec     $f9
    beq     .left_erase
    dec     $f8                     ; Update the sprite index.
    jmp     .left_draw_loop
.left_erase
    ldx     p1_attack_data + porkchop_gator_ticks
    cpx     #8                      ; After 8 ticks, start erasing.
    bmi     .right_draw_setup
    jsr     porkchop_alligator_erase
.right_draw_setup
    ldy     p1_attack_data + porkchop_gator_loop_count
    sty     $f9                     ; Draw loop count
    ldy     p1_attack_data + porkchop_right_gator + porkchop_gator_sprite_index
    sty     $f8
    lda     p1_attack_data + porkchop_right_gator + porkchop_gator_address
    sta     $fe
    lda     p1_attack_data + porkchop_right_gator + porkchop_gator_address + 1
    sta     $ff
    mwa     porkchop_alligator_left,$fc
.right_draw_loop
    ldy     $f8
    jsr     porkchop_copy_alligator
    inc     $fe                     ; Update the address.
    bne     .right_draw_no_carry
    inc     $ff
.right_draw_no_carry
    dec     $f9
    beq     .right_erase
    inc     $f8                     ; Update the sprite index.
    jmp     .right_draw_loop
.right_erase
    ldx     p1_attack_data + porkchop_gator_ticks
    cpx     #8                      ; After 8 ticks. start erasing.
    bmi     .check_hits
    jsr     porkchop_alligator_erase
.check_hits
    lda     pf_low
    beq     .done
    jsr     damage_player_0
.done
    jmp     p1_process_return

; Set up the player 0 alligators, the ones attacking
; player 1.  Chooses the locations randomly and assigns
; the positions and starting addresses.
porkchop_setup_player_0_attack subroutine
    jsr     get_player_0_random     ; Grab a left position.
    jsr     porkchop_trim_random
    lda     porkchop_left_gator_vpos,y
    sta     p0_attack_data + porkchop_left_gator + porkchop_gator_vpos
    lda     porkchop_p0_left_gator_base_low,y
    sta     p0_attack_data + porkchop_left_gator + porkchop_gator_address
    lda     porkchop_p0_left_gator_base_high,y
    sta     p0_attack_data + porkchop_left_gator + porkchop_gator_address + 1
    lda     #7
    sta     p0_attack_data + porkchop_left_gator + porkchop_gator_sprite_index
    jsr     get_player_0_random     ; Grab a right position.
    jsr     porkchop_trim_random
    lda     porkchop_right_gator_vpos,y
    sta     p0_attack_data + porkchop_right_gator + porkchop_gator_vpos
    lda     porkchop_p0_right_gator_base_low,y
    sta     p0_attack_data + porkchop_right_gator + porkchop_gator_address
    lda     porkchop_p0_right_gator_base_high,y
    sta     p0_attack_data + porkchop_right_gator + porkchop_gator_address + 1
    lda     #0
    sta     p0_attack_data + porkchop_right_gator + porkchop_gator_sprite_index
    lda     #porkchop_gator_move_time
    sta     p0_attack_data + porkchop_gator_timer
    lda     #1
    sta     p0_attack_data + porkchop_gator_loop_count
    mwa     porkchop_gator_sound,$fe
    jsr     play_sound_3
    rts

; The same, but for player 1.
porkchop_setup_player_1_attack subroutine
    jsr     get_player_1_random     ; Grab a left position.
    jsr     porkchop_trim_random
    lda     porkchop_p1_left_gator_base_low,y
    sta     p1_attack_data + porkchop_left_gator + porkchop_gator_address
    lda     porkchop_p1_left_gator_base_high,y
    sta     p1_attack_data + porkchop_left_gator + porkchop_gator_address + 1
    lda     #7
    sta     p1_attack_data + porkchop_left_gator + porkchop_gator_sprite_index
    jsr     get_player_1_random     ; Grab a right position.
    jsr     porkchop_trim_random
    lda     porkchop_p1_right_gator_base_low,y
    sta     p1_attack_data + porkchop_right_gator + porkchop_gator_address
    lda     porkchop_p1_right_gator_base_high,y
    sta     p1_attack_data + porkchop_right_gator + porkchop_gator_address + 1
    lda     #0
    sta     p1_attack_data + porkchop_right_gator + porkchop_gator_sprite_index
    lda     #porkchop_gator_move_time
    sta     p1_attack_data + porkchop_gator_timer
    lda     #1
    sta     p1_attack_data + porkchop_gator_loop_count
    mwa     porkchop_gator_sound,$fe
    jsr     play_sound_4
    rts

; Trim a random number down to the alligator
; range.  This will have some bias, but I
; don't really care.  The number enters in
; the accumulator and leaves in y.
porkchop_trim_random subroutine
    and     #$07                    ; Mask to three bits.
    cmp     #5                      ; Greater than four?
    bmi     .done                   ; If not, it's fine.
    lsr                             ; Shift it down.
.done
    tay
    rts

; Copy an alligator block to the address in $fe-$ff.
; Expects the address of the image in $fc-$fd and the
; offset in y.
porkchop_copy_alligator subroutine
    ldx     #6                      ; 6 bytes to copy.
    sty     $fb                     ; Read offset.
    lda     #0
    sta     $fa                     ; Write offset.
    clc
.loop
    lda     ($fc),y                 ; Read a byte.
    ldy     $fa                     ; Get the write offset.
    sta     ($fe),y
    dex
    beq     .done
    lda     $fb                     ; Update the read offset.
    adc     #8
    sta     $fb
    tay
    lda     $fa                     ; Update the write offset.
    adc     #40
    sta     $fa
    jmp     .loop
.done
    rts

; Same as the above, but reversed for the upper display.
porkchop_copy_alligator_reverse subroutine
    ldx     #6                      ; 6 bytes to copy.
    tya                             ; Add 40 to the offset to get to
    clc                             ; the bottom of the image.
    adc     #40
    sta     $fb                     ; Read offset.
    tay
    lda     #0
    sta     $fa                     ; Write offset.
.loop
    lda     ($fc),y                 ; Read a byte.
    ldy     $fa                     ; Get the write offset.
    sta     ($fe),y
    dex
    beq     .done
    lda     $fb                     ; Update the read offset.
    sec
    sbc     #8
    sta     $fb
    tay
    lda     $fa                     ; Update the write offset.
    clc
    adc     #40
    sta     $fa
    jmp     .loop
.done
    rts

; Erase an alligator section.  Pass the base address
; in $fe-$ff.
porkchop_alligator_erase subroutine
    ldx     #6                      ; 6 bytes to erase.
    ldy     #0                      ; Write offset.
    clc
.loop
    lda     #0
    sta     ($fe),y                 ; Clear the byte.
    dex
    beq     .done
    tya                             ; Update the target.
    adc     #40
    tay
    jmp     .loop
.done
    rts

porkchop_cpu subroutine
    inc     cpu_state               ; Head for the middle of the screen.
    lda     #hpos_center
    sta     cpu_xdest
    cmp     p1_real_hpos
    bcs     .move_right
    lda     cpu_left_speed
    .byte   $2c
.move_right
    lda     cpu_right_speed
    sta     cpu_hspeed
    lda     p0_attack_data + porkchop_left_gator + porkchop_gator_vpos
    sta     $fe                     ; Going to need these a lot.
    lda     p0_attack_data + porkchop_right_gator + porkchop_gator_vpos
    sta     $ff
    sec
    sbc     $fe                     ; Figure out which is on top.
    bcs     .right_on_bottom
    ldx     #0                      ; Flag for left on bottom.
    eor     #$ff                    ; Make the distance positive.
    adc     #1                      ; Carry is clear.
    .byte   $2c
.right_on_bottom
    ldx     #1                      ; Flag for right on bottom.
    cmp     #26                     ; A separation of 26 can be split.
    bmi     .cant_split
    cpx     #0                      ; Add 6 to the upper gator's vpos.
    beq     .split_right_top
    lda     $fe                     ; Get the left gator's vpos.
    .byte   $2c
.split_right_top
    lda     $ff                     ; Get the right gator's vpos.
    clc
    adc     #6
    sta     cpu_ydest
    bne     .find_vspeed            ; Always true.
.cant_split
    lda     $fe                     ; Get the upper vpos.
    ldy     $ff
    jsr     get_min_value
    cmp     #p1_top_bound + 1       ; Can we go high?
    bcs     .go_high
    lda     $fe                     ; Get the lower vpos.
    ldy     $ff
    jsr     get_max_value
    cmp     #p1_bottom_move_bound - #6
    bcs     .find_vspeed            ; Nothing to do, hope for the best.
    lda     #p1_bottom_move_bound   ; Go low.
    .byte   $2c
.go_high
    lda     #p1_top_bound           ; Go high.
    sta     cpu_ydest
.find_vspeed
    lda     p1_vpos
    cmp     cpu_ydest
    bcs     .move_up
    lda     cpu_down_speed
    .byte   $2c
.move_up
    lda     cpu_up_speed
    sta     cpu_vspeed
    rts

; Sprite images.
porkchop_p1_sprite
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %01111110
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %11111111
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00100100
    .byte       %01100110
    .byte       %01100110
    .byte       %01000010

porkchop_p2_sprite
    .byte       %01000010
    .byte       %11000011
    .byte       %00011000
    .byte       %00111100
    .byte       %00000000
    .byte       %01111110
    .byte       %01111110
    .byte       %01111110
    .byte       %00111100
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000

porkchop_p3_sprite
    .byte       %00000000
    .byte       %00000000
    .byte       %11000011
    .byte       %11000011
    .byte       %10000001
    .byte       %10000001
    .byte       %10000001
    .byte       %10000001
    .byte       %11000011
    .byte       %01111110
    .byte       %11111111
    .byte       %11111111
    .byte       %00000000
    .byte       %11111111
    .byte       %01111110
    .byte       %00100100
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000

; Letter data for the select screen.
; See pink_b.s for details.
porkchop_letter_data
    .word       big_letter_p,$3000,$29c7
    .byte       big_letter_p_width
    .word       big_letter_o,$3002,$29c9
    .byte       big_letter_o_width
    .word       big_letter_r,$3004,$29cb
    .byte       big_letter_r_width
    .word       big_letter_k,$3006,$29cd
    .byte       big_letter_k_width
    .word       big_letter_hyphen,$3008,$29cf
    .byte       big_letter_hyphen_width
    .word       big_letter_c,$3009,$29d0
    .byte       big_letter_c_width
    .word       big_letter_h,$300b,$29d2
    .byte       big_letter_h_width
    .word       big_letter_o,$300d,$29d4
    .byte       big_letter_o_width
    .word       big_letter_p,$300f,$29d6
    .byte       big_letter_p_width

porkchop_game_row_offset        equ 12
porkchop_game_portrait_offset   equ 59 - porkchop_game_row_offset

porkchop_matrix_data
    .byte 101,97,97,101     ; Player positions.
    .byte 109,108,105,109   ; Missile positions.
    .byte $84,$06,$f8,$fc   ; Colors.

porkchop_matrix_image
    .byte %00000000         ; Missiles.
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %10000000
    .byte %00000011
    .byte %00001101
    .byte %00100011
    .byte %00110011
    .byte %11000000
    .byte %11000000
    .byte %11000000
    .byte %11000000
    .byte %10000000
    .byte %00000000
    .byte %01000000

    .byte %00000000         ; Player 0.
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %11111000
    .byte %11111111
    .byte %10010010
    .byte %11100011
    .byte %10000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000

    .byte %00000000         ; Player 1.
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00010000
    .byte %00110000
    .byte %11011100
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %10100000

    .byte %00000000         ; Player 2.
    .byte %00010000
    .byte %00110000
    .byte %01110000
    .byte %01100000
    .byte %00000000
    .byte %00000000
    .byte %11100000
    .byte %01110001
    .byte %11110001
    .byte %01110000
    .byte %00111000
    .byte %00111100
    .byte %00011111
    .byte %00001111
    .byte %00010111

    .byte %00000000         ; Player 3.
    .byte %11111100
    .byte %11111110
    .byte %01111111
    .byte %00000000
    .byte %00000000
    .byte %00101100
    .byte %00000110
    .byte %00000010
    .byte %11100001
    .byte %11111111
    .byte %01100001
    .byte %00000001
    .byte %00000011
    .byte %00001111
    .byte %00001110

porkchop_matrix_background
    .byte %00000000,%00101010,%10100000,%00000000
    .byte %00000010,%10101010,%10101010,%00000000
    .byte %00001010,%10101010,%10101010,%10000000
    .byte %00001010,%10101010,%10101010,%10100000
    .byte %00101010,%10101010,%10101010,%10100000
    .byte %00101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %00001010,%10101010,%10101010,%10101010
    .byte %00001010,%10101010,%10101010,%10101000
    .byte %00001010,%10101010,%10101010,%10101000
    .byte %00001010,%10101010,%10101010,%10100000
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101010

porkchop_alligator_left
    .byte %00010101,%11000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00110100
    .byte %10001001,%01011101,%11011001,%11000001,%11010111,%01011101,%11000000,%00001111
    .byte %00000000,%10001001,%01010101,%01010101,%01010101,%01010101,%01011101,%11000001
    .byte %00000000,%00100000,%01010101,%01010101,%01010101,%01010101,%01010101,%01010111
    .byte %00100010,%01010101,%01010000,%00010101,%01010101,%01010101,%01000000,%01011100
    .byte %00010101,%01010000,%00000001,%01010100,%00000000,%00000001,%01010100,%00000000

porkchop_alligator_right
    .byte %00000000,%00000000,%00000001,%00010001,%00010001,%00010000,%00000000,%01010101
    .byte %00000000,%00000000,%01000101,%01010101,%01010101,%01010101,%11010101,%01010010
    .byte %00000000,%00000001,%01010101,%01010101,%01010101,%01010101,%01010101,%00100000
    .byte %00000001,%00010101,%01010101,%01010101,%01010101,%01010101,%01010010,%00000000
    .byte %00000101,%01010101,%00000000,%01000000,%00000001,%01000001,%01010101,%01100010
    .byte %01010101,%00000000,%00000000,%01010000,%00000000,%01010000,%00000000,%01010101

porkchop_p0_left_gator_base_low
    .byte $0,$e0,$c0,$a0,$80

porkchop_p0_left_gator_base_high
    .byte $20,$21,$23,$25,$27

porkchop_left_gator_vpos
    .byte $2e,$3a,$46,$52,$5e

porkchop_p0_right_gator_base_low
    .byte $17,$f7,$d7,$b7,$97

porkchop_p0_right_gator_base_high
    .byte $21,$22,$24,$26,$28

porkchop_right_gator_vpos
    .byte $34,$40,$4c,$58,$64

porkchop_p1_left_gator_base_low
    .byte $f0,$10,$30,$50,$70

porkchop_p1_left_gator_base_high
    .byte $3a,$39,$37,$35,$33

porkchop_p1_right_gator_base_low
    .byte $27,$47,$67,$87,$a7

porkchop_p1_right_gator_base_high
    .byte $3a,$38,$36,$34,$32

porkchop_gator_sound
    .byte $8f
    .byte $50,$0a
    .byte $54,$0a
    .byte $58,$0a
    .byte $0

porkchop_portrait
    include "porkchop_portrait.s"

porkchop_portrait_color_1 equ $3e ; 255, 198, 181
porkchop_portrait_color_2 equ $0 ; 0, 0, 0
porkchop_portrait_color_3 equ $2a ; 231, 148, 41
porkchop_portrait_color_5 equ $c ; 198, 198, 198
porkchop_portrait_color_9 equ $10 ; 16, 0, 0
porkchop_portrait_color_21 equ $94 ; 0, 66, 173
porkchop_portrait_color_22 equ $34 ; 148, 16, 0

porkchop_portrait_table
    .byte 5
    .byte porkchop_portrait_color_3
    .byte porkchop_portrait_color_3
    .byte porkchop_portrait_color_2
    .byte porkchop_portrait_color_2
    .byte porkchop_portrait_color_1
    .byte 3 + 128
    .byte porkchop_portrait_color_2
    .byte 10 + 128
    .byte porkchop_portrait_color_1
    .byte 4 + 128
    .byte porkchop_portrait_color_2
    .byte 16 + 128
    .byte porkchop_portrait_color_1
    .byte 15 + 128
    .byte porkchop_portrait_color_2
    .byte 1
    .byte porkchop_portrait_color_9
    .byte 6 + 128
    .byte porkchop_portrait_color_1
    .byte 5
    .byte porkchop_portrait_color_2
    .byte porkchop_portrait_color_2
    .byte porkchop_portrait_color_3
    .byte porkchop_portrait_color_1
    .byte porkchop_portrait_color_2
    .byte 3 + 128
    .byte porkchop_portrait_color_1
    .byte 10 + 128
    .byte porkchop_portrait_color_2
    .byte 4 + 128
    .byte porkchop_portrait_color_1
    .byte 3
    .byte porkchop_portrait_color_2
    .byte porkchop_portrait_color_2
    .byte porkchop_portrait_color_9
    .byte 3 + 128
    .byte porkchop_portrait_color_2
    .byte 1
    .byte porkchop_portrait_color_9
    .byte 9 + 128
    .byte porkchop_portrait_color_2
    .byte 3 + 128
    .byte porkchop_portrait_color_1
    .byte 3
    .byte porkchop_portrait_color_21
    .byte porkchop_portrait_color_21
    .byte porkchop_portrait_color_1
    .byte 3 + 128
    .byte porkchop_portrait_color_3
    .byte 2
    .byte porkchop_portrait_color_1
    .byte porkchop_portrait_color_3
    .byte 5 + 128
    .byte porkchop_portrait_color_1
    .byte 6 + 128
    .byte porkchop_portrait_color_2
    .byte 3 + 128
    .byte porkchop_portrait_color_1
    .byte 7 + 128
    .byte porkchop_portrait_color_3
    .byte 4 + 128
    .byte porkchop_portrait_color_1
    .byte 2
    .byte porkchop_portrait_color_5
    .byte porkchop_portrait_color_5
    .byte 4 + 128
    .byte porkchop_portrait_color_22
    .byte 4 + 128
    .byte porkchop_portrait_color_5
    .byte 14 + 128
    .byte porkchop_portrait_color_3
    .byte 3 + 128
    .byte porkchop_portrait_color_21
    .byte 3
    .byte porkchop_portrait_color_1
    .byte porkchop_portrait_color_1
    .byte porkchop_portrait_color_21
    .byte 3 + 128
    .byte porkchop_portrait_color_1
    .byte 2
    .byte porkchop_portrait_color_3
    .byte porkchop_portrait_color_1
    .byte 11 + 128
    .byte porkchop_portrait_color_3

