; This file is part of Magical Fairy Force.

; Magical Fairy Force is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Magical Fairy Force is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Magical Fairy Force.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2020 Average Software.

; Queen Sugar character data and logic routines

; Queen Sugar's attack flow:

; Thin projectiles will launch from the bottom of the screen.
; These will detonate into large hearts, similar to fireworks.
; The detonation point will be randomized.  The explosions and
; the projectiles themselves will damage the other player.

; Projectile phase constants.
q_sugar_shot_inactive   equ -1
q_sugar_shot_enter      equ 0
q_sugar_shot_rise       equ 1
q_sugar_shot_detonate   equ 2

; Projectile phase lengths.
q_sugar_enter_length    equ 6
q_sugar_detonate_length equ 30

; Projectile structure offsets.
q_sugar_shot_phase      equ 0
q_sugar_shot_ticks      equ 1
q_sugar_shot_address    equ 2
q_sugar_shot_size       equ 4

; Other attack constants.
q_sugar_attack_cost     equ -max_fairy_power * 5 / 6
q_sugar_total_shots     equ 24
q_sugar_heart_height    equ 8
q_sugar_spawn_interval  equ 10
q_sugar_heart_color     equ $4a
q_sugar_text_color      equ 0

; Attack block data offsets.
q_sugar_spawn_ticks     equ 0
q_sugar_next_shot       equ 1
q_sugar_shots_left      equ 2
q_sugar_shot_data       equ 3

q_sugar_start_attack subroutine
    beq     .attack_player_0
    ldx     #q_sugar_attack_cost
    jsr     general_player_0_attack_start
    lda     #q_sugar_heart_color
    sta     p1_playfield_2_color
    lda     #q_sugar_text_color
    sta     p1_playfield_1_color
    lda     #$0f
    sta     p1_playfield_0_color
    mwa     q_sugar_start_sound,$fe
    jsr     play_sound_3
    lda     #>p0_attack_data        ; Branch is always true.
    bne     q_sugar_init_attack_block
.attack_player_0
    ldx     #q_sugar_attack_cost
    jsr     general_player_1_attack_start
    lda     #q_sugar_heart_color
    sta     p0_playfield_2_color
    lda     #q_sugar_text_color
    sta     p0_playfield_1_color
    lda     #$0f
    sta     p0_playfield_0_color
    mwa     q_sugar_start_sound,$fe
    jsr     play_sound_4
    lda     #>p1_attack_data
    ; Fall through.

; Initialize the player attack block
; whose high byte is in the accumulator.
q_sugar_init_attack_block subroutine
    sta     $ff
    lda     #0
    sta     $fe
    ldy     #q_sugar_spawn_ticks
    lda     #1                      ; Spawn on first frame.
    sta     ($fe),y
    ldy     #q_sugar_next_shot
    lsr                             ; Shift to 0.
    sta     ($fe),y
    ldy     #q_sugar_shots_left
    lda     #q_sugar_total_shots
    sta     ($fe),y
    ldx     #q_sugar_total_shots
    lda     #q_sugar_shot_data
    sta     $fc
    lda     $ff
    sta     $fd
    ldy     #q_sugar_shot_phase
.shot_init_loop
    lda     #q_sugar_shot_inactive
    sta     ($fc),y
    dex
    beq     .shot_init_done
    clc
    lda     $fc
    adc     #q_sugar_shot_size
    sta     $fc
    bne     .shot_init_loop         ; Always true.
.shot_init_done
    jmp     start_attack_return

q_sugar_process_attack subroutine
    bne     q_sugar_attack_player_1
    jmp     q_sugar_attack_player_0

q_sugar_attack_player_1 subroutine
    lda     #2
    sta     p1_cloud_timer          ; Suppress cloud spawns.
    lda     pf_high                 ; Check for damage.
    beq     .no_damage
    jsr     damage_player_1
.no_damage
    lda     #0                      ; Sound effect trigger.
    sta     $f1
    dec     p0_attack_data + q_sugar_spawn_ticks
    bne     .no_spawn
    lda     p0_attack_data + q_sugar_next_shot
    cmp     #q_sugar_total_shots * q_sugar_shot_size
    beq     .no_spawn               ; All shots have fired.
    tax                             ; Save the shot index.
    clc
    adc     #<[p0_attack_data + q_sugar_shot_data]
    sta     $fe
    lda     #>[p0_attack_data + q_sugar_shot_data]
    adc     #0
    sta     $ff
    txa                             ; Get the shot index back.
    clc
    adc     #q_sugar_shot_size      ; Update the index.
    sta     p0_attack_data + q_sugar_next_shot
    jsr     q_sugar_init_shot
    jsr     get_player_0_random
    jsr     q_sugar_trim_random
    ldy     #q_sugar_shot_address
    sta     ($fe),y                 ; Address is $2000 + offset.
    lda     #$20
    iny
    sta     ($fe),y
    lda     #q_sugar_spawn_interval
    sta     p0_attack_data + q_sugar_spawn_ticks
.no_spawn
    mwa     [p0_attack_data + q_sugar_shot_data],$fe
    ldx     #q_sugar_total_shots
    stx     $f0                     ; Shot count.
.shot_loop
    jsr     q_sugar_process_p0_shot
    bne     .shot_not_done
    dec     p0_attack_data + q_sugar_shots_left
    bne     .shot_not_done
    jmp     end_player_0_attack
.shot_not_done
    tya                             ; Save the sound trigger.
    ora     $f1
    sta     $f1
    dec     $f0
    beq     .shot_loop_done
    lda     $fe
    clc
    adc     #q_sugar_shot_size
    sta     $fe
    bne     .shot_loop              ; Always true.
.shot_loop_done
    ldy     $f1                     ; Check for sound.
    beq     .sound_trigger_clear
    mwa     q_sugar_explode_sound,$fe
    jsr     play_sound_3
.sound_trigger_clear
    jmp     p0_process_return

q_sugar_attack_player_0 subroutine
    lda     #2
    sta     p0_cloud_timer          ; Suppress cloud spawns.
    lda     pf_low                  ; Check for damage.
    beq     .no_damage
    jsr     damage_player_0
.no_damage
    lda     #0                      ; Sound effect trigger.
    sta     $f1
    dec     p1_attack_data + q_sugar_spawn_ticks
    bne     .no_spawn
    lda     p1_attack_data + q_sugar_next_shot
    cmp     #q_sugar_total_shots * q_sugar_shot_size
    beq     .no_spawn               ; All shots have fired.
    tax                             ; Save the shot index.
    clc
    adc     #<[p1_attack_data + q_sugar_shot_data]
    sta     $fe
    lda     #>[p1_attack_data + q_sugar_shot_data]
    adc     #0
    sta     $ff
    txa                             ; Get the shot index back.
    clc
    adc     #q_sugar_shot_size      ; Update the index.
    sta     p1_attack_data + q_sugar_next_shot
    jsr     q_sugar_init_shot
    jsr     get_player_1_random
    jsr     q_sugar_trim_random
    ldy     #q_sugar_shot_address
    clc
    adc     #$e0
    sta     ($fe),y                 ; Address is $3be0 + offset.
    lda     #$3b
    adc     #0
    iny
    sta     ($fe),y
    lda     #q_sugar_spawn_interval
    sta     p1_attack_data + q_sugar_spawn_ticks
.no_spawn
    mwa     [p1_attack_data + q_sugar_shot_data],$fe
    ldx     #q_sugar_total_shots
    stx     $f0                     ; Shot count.
.shot_loop
    jsr     q_sugar_process_p1_shot
    bne     .shot_not_done
    dec     p1_attack_data + q_sugar_shots_left
    bne     .shot_not_done
    jmp     end_player_1_attack
.shot_not_done
    tya                             ; Save the sound trigger.
    ora     $f1
    sta     $f1
    dec     $f0
    beq     .shot_loop_done
    lda     $fe
    clc
    adc     #q_sugar_shot_size
    sta     $fe
    bne     .shot_loop              ; Always true.
.shot_loop_done
    ldy     $f1                     ; Check for sound.
    beq     .sound_trigger_clear
    mwa     q_sugar_explode_sound,$fe
    jsr     play_sound_4
.sound_trigger_clear
    jmp     p1_process_return

; Common initialization code for shots.
; Expects the shot address in $fe-$ff.
q_sugar_init_shot subroutine
    ldy     #q_sugar_shot_phase
    lda     #q_sugar_shot_enter
    sta     ($fe),y
    ldy     #q_sugar_shot_ticks
    lda     #q_sugar_enter_length
    sta     ($fe),y
    rts

; Trim a random number to a valid spawn offset.
; Expects the player's random number in the accumulator.
q_sugar_trim_random subroutine
    and     #63                     ; Get it close.
    cmp     #37                     ; We want 0 - 36
    bmi     .range_ok
    lsr                             ; Shift into range.
.range_ok
    clc
    adc     #1                      ; Add 1 to get 1 - 37.  The edges are bad.
    rts
    
; Process the player 0 projectile in $fe-$ff.  Returns 0 in
; accumulator if the shot has finished, non-zero otherwise.
; Returns non-zero in y to signal the explosion sound.
q_sugar_process_p0_shot subroutine
    ldy     #0                      ; Assume no sound.
    sty     $f9
    ldy     #q_sugar_shot_phase
    lda     ($fe),y
    bpl     .shot_active
.done
    ldy     $f9                     ; Get the sound trigger.
    lda     #1                      ; Return value.
    rts                             ; Inactive shot, do nothing.
.shot_active
    beq     .shot_enter
    cmp     #q_sugar_shot_rise
    beq     .shot_rise
    jsr     q_sugar_adjust_ticks
    bne     .shot_detonate_active
    jsr     q_sugar_copy_shot_address
    ldx     #q_sugar_heart_height
.clear_shot_outer_loop
    ldy     #3                      ; Write index.
.clear_shot_inner_loop
    lda     #0
    sta     ($fc),y
    dey
    bpl     .clear_shot_inner_loop
    dex
    beq     .clear_shot_done
    jsr     q_sugar_backup_shot_address
    jmp     .clear_shot_outer_loop
.clear_shot_done
    ldy     #q_sugar_shot_phase
    lda     #q_sugar_shot_inactive
    sta     ($fe),y
    ldy     #0                      ; No sound.
    tya                             ; Return value.
    rts
.shot_detonate_active
    jsr     q_sugar_copy_shot_address
    inc     $f9                     ; Signal the sound effect.
    ldx     #0                      ; Read index.
.shot_outer_loop
    ldy     #0                      ; Write index
.shot_inner_loop
    lda     q_sugar_heart,x
    ora     ($fc),y                 ; Blend with whatever's there.
    sta     ($fc),y
    inx
    iny
    cpy     #4
    bne     .shot_inner_loop
    cpx     #q_sugar_heart_height * 4
    beq     .done
    jsr     q_sugar_backup_shot_address
    jmp     .shot_outer_loop
.shot_enter
    jsr     q_sugar_adjust_ticks
    bne     .shot_enter_active
    jsr     get_player_0_random
    and     #31                     ; Cut down to 0-31, then add 30.
    clc
    adc     #30
    jmp     q_sugar_advance_phase
.shot_enter_active
    jmp     q_sugar_extend_shot_down
.shot_rise
    jsr     q_sugar_adjust_ticks
    bne     .shot_rise_active
    jmp     q_sugar_prepare_detonation
.shot_rise_active
    jsr     q_sugar_extend_shot_down
    lda     $fc
    sec
    sbc     #240
    sta     $fc
    bcs     .no_rise_borrow
    dec     $fd
.no_rise_borrow
    lda     #0
    tay
    sta     ($fc),y
    ldy     $f9                     ; Sound effect flag.
    lda     #1                      ; Return value.
    rts

; The same, but for player 1.
q_sugar_process_p1_shot subroutine
    ldy     #0                      ; Assume no sound.
    sty     $f9
    ldy     #q_sugar_shot_phase
    lda     ($fe),y
    bpl     .shot_active
.done
    ldy     $f9                     ; Get the sound trigger.
    lda     #1                      ; Return value.
    rts                             ; Inactive shot, do nothing.
.shot_active
    beq     .shot_enter
    cmp     #q_sugar_shot_rise
    beq     .shot_rise
    jsr     q_sugar_adjust_ticks
    bne     .shot_detonate_active
    jsr     q_sugar_copy_shot_address
    ldx     #q_sugar_heart_height
.clear_shot_outer_loop
    ldy     #3                      ; Write index.
.clear_shot_inner_loop
    lda     #0
    sta     ($fc),y
    dey
    bpl     .clear_shot_inner_loop
    dex
    beq     .clear_shot_done
    jsr     q_sugar_advance_shot_address
    jmp     .clear_shot_outer_loop
.clear_shot_done
    ldy     #q_sugar_shot_phase
    lda     #q_sugar_shot_inactive
    sta     ($fe),y
    ldy     #0                      ; No sound.
    tya                             ; Return value.
    rts
.shot_detonate_active
    jsr     q_sugar_copy_shot_address
    inc     $f9                     ; Signal the sound effect.
    ldx     #0                      ; Read index.
.shot_outer_loop
    ldy     #0                      ; Write index
.shot_inner_loop
    lda     q_sugar_heart,x
    ora     ($fc),y                 ; Blend with whatever's there.
    sta     ($fc),y
    inx
    iny
    cpy     #4
    bne     .shot_inner_loop
    cpx     #q_sugar_heart_height * 4
    beq     .done
    jsr     q_sugar_advance_shot_address
    jmp     .shot_outer_loop
.shot_enter
    jsr     q_sugar_adjust_ticks
    bne     .shot_enter_active
    jsr     get_player_1_random
    and     #31                     ; Cut down to 0-31, then add 30.
    clc
    adc     #30
    jmp     q_sugar_advance_phase
.shot_enter_active
    jmp     q_sugar_extend_shot_up
.shot_rise
    jsr     q_sugar_adjust_ticks
    bne     .shot_rise_active
    jmp     q_sugar_prepare_detonation
.shot_rise_active
    jsr     q_sugar_extend_shot_up
    ldy     #240
    lda     #0
    sta     ($fc),y
    ldy     $f9                     ; Sound effect flag.
    lda     #1                      ; Return value.
    rts

; Add a shot segment to the top of the shot in $fe-$ff.
; Also updates the address for the next segment.
q_sugar_extend_shot_up subroutine
    ldy     #q_sugar_shot_address
    lda     ($fe),y                 ; Put the target address in $fc-$fd.
    sta     $fc
    sec
    sbc     #40                     ; Update the address at the same time.
    sta     ($fe),y
    iny
    lda     ($fe),y
    sta     $fd
    sbc     #0
    jmp     q_sugar_add_pixel

; Just like above, but moving down.
q_sugar_extend_shot_down subroutine
    ldy     #q_sugar_shot_address
    lda     ($fe),y                 ; Put the target address in $fc-$fd.
    sta     $fc
    clc
    adc     #40                     ; Update the address at the same time.
    sta     ($fe),y
    iny
    lda     ($fe),y
    sta     $fd
    adc     #0
    ; Fall through to add pixel.

q_sugar_add_pixel subroutine
    sta     ($fe),y
    ldy     #0
    lda     #1
    sta     ($fc),y                 ; Add a new pixel.
    rts

; Decrement the shot ticks for the shot in $fe-$ff.
q_sugar_adjust_ticks subroutine
    ldy     #q_sugar_shot_ticks
    lda     ($fe),y
    sec
    sbc     #1
    sta     ($fe),y
    rts

; Ready a shot in $fe-$ff for final detonation.
q_sugar_prepare_detonation subroutine
    ldy     #q_sugar_shot_address   ; Decrement the address to set
    lda     ($fe),y                 ; the heart address.
    sec
    sbc     #1
    sta     ($fe),y
    iny
    lda     ($fe),y
    sbc     #0
    sta     ($fe),y
    lda     #q_sugar_detonate_length
    ; Fall through to advance phase.

; Advance the phase for the shot in $fe-$ff.
; Expects the new tick count in the accumulator.
q_sugar_advance_phase subroutine
    ldy     #q_sugar_shot_ticks
    sta     ($fe),y
    ldy     #q_sugar_shot_phase
    lda     ($fe),y
    clc
    adc     #1
    sta     ($fe),y
    rts

; Copies the address of the shot in $fe-$ff to
; $fc-$fd.
q_sugar_copy_shot_address subroutine
    ldy     #q_sugar_shot_address
    lda     ($fe),y
    sta     $fc
    iny
    lda     ($fe),y
    sta     $fd
    rts

; Bump the address in $fc-$fd to the next row.
q_sugar_advance_shot_address subroutine
    lda     $fc
    clc
    adc     #40
    sta     $fc
    bcc     .no_carry
    inc     $fd
.no_carry
    rts

; Drop the address in $fc-$fd to the previous row.
q_sugar_backup_shot_address subroutine
    lda     $fc
    sec
    sbc     #40
    sta     $fc
    bcs     .no_borrow
    dec     $fd
.no_borrow
    rts

q_sugar_cpu subroutine
    inc     cpu_state
    lda     #p1_top_bound + #20     ; Head for the top-ish.
    sta     cpu_ydest
    cmp     p1_vpos
    bcs     .move_down
    lda     cpu_up_speed
    .byte   $2c
.move_down
    lda     cpu_down_speed
    sta     cpu_vspeed
    lda     p1_real_hpos            ; Convert to byte offset.
    sec
    sbc     #left_bound
    lsr
    lsr
    tay                             ; Check the incoming bytes.
    lda     $2000,y
    bne     .dodge_right
    lda     $2001,y
    bne     .dodge_random
    lda     $2002,y
    bne     .dodge_left
    lda     p1_real_hpos            ; Hold still!
    sta     cpu_xdest
    lda     #0
    sta     cpu_hspeed
    rts
.dodge_right
    lda     p1_real_hpos
    clc
    adc     #10
    sta     cpu_xdest
    ldy     cpu_right_speed
    iny
    sty     cpu_hspeed
    rts
.dodge_random
    lda     RANDOM
    bmi     .dodge_right
.dodge_left
    lda     p1_real_hpos
    sec
    sbc     #10
    sta     cpu_xdest
    ldy     cpu_left_speed
    dey
    sty     cpu_hspeed
    rts

; Sprite images.
q_sugar_p1_sprite
    .byte       %10000001
    .byte       %10000001
    .byte       %10000001
    .byte       %10000001
    .byte       %11000001
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %01000010
    .byte       %01100110
    .byte       %11111111
    .byte       %11111111
    .byte       %01111110
    .byte       %00111100
    .byte       %00011000

q_sugar_p2_sprite
    .byte       %00011000
    .byte       %00111100
    .byte       %00111100
    .byte       %00111100
    .byte       %00111110
    .byte       %01111110
    .byte       %11111111
    .byte       %01111110
    .byte       %00010010
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000

q_sugar_p3_sprite
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00001000
    .byte       %00111100
    .byte       %01111110
    .byte       %01111110
    .byte       %01111110
    .byte       %00111100
    .byte       %00011000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000

; Letter data for the select screen.
; See pink_b.s for details.
q_sugar_letter_data
    .word       big_letter_q,$3000,$29cb
    .byte       big_letter_q_width
    .word       big_letter_period,$3002,$29cd
    .byte       big_letter_period_width
    .word       big_letter_s,$3003,$29ce
    .byte       big_letter_s_width
    .word       big_letter_u,$3005,$29d0
    .byte       big_letter_u_width
    .word       big_letter_g,$3007,$29d2
    .byte       big_letter_g_width
    .word       big_letter_a,$3009,$29d4
    .byte       big_letter_a_width
    .word       big_letter_r,$300b,$29d6
    .byte       big_letter_r_width

q_sugar_game_row_offset       equ 9
q_sugar_game_portrait_offset  equ 59 - q_sugar_game_row_offset

q_sugar_matrix_data
    .byte 136,144,136,144   ; Player positions.
    .byte 0,0,0,0           ; Missile positions.
    .byte $48,$48,$fa,$fa   ; Colors.

q_sugar_matrix_image equ q_sugar_matrix_image_real_start - 16
q_sugar_matrix_image_real_start
    .byte %00000000         ; Player 0.
    .byte %00011000
    .byte %00010000
    .byte %00010000
    .byte %00010000
    .byte %00110000
    .byte %00110000
    .byte %00110000
    .byte %00110000
    .byte %00010000
    .byte %00010000
    .byte %00010000
    .byte %00010000
    .byte %00011000
    .byte %00011010
    .byte %00010100

    .byte %00011000         ; Player 1.
    .byte %00011000
    .byte %00000000
    .byte %00011000
    .byte %00011000
    .byte %00011000
    .byte %00001000
    .byte %00001000
    .byte %00001100
    .byte %00000100
    .byte %00010000
    .byte %00010000
    .byte %00100000
    .byte %00000000
    .byte %00010000
    .byte %00010000

    .byte %00000111         ; Player 2.
    .byte %00000101
    .byte %00000011
    .byte %00000101
    .byte %00000011
    .byte %00001111
    .byte %00001111
    .byte %00001110
    .byte %00000111
    .byte %00000111
    .byte %00000111
    .byte %00000011
    .byte %00000010
    .byte %00000001
    .byte %00000001
    .byte %00000000

    .byte %11100000         ; Player 3.
    .byte %11100000
    .byte %10100000
    .byte %10100000
    .byte %10000000
    .byte %11100000
    .byte %11100000
    .byte %11100000
    .byte %11100000
    .byte %11100000
    .byte %11100000
    .byte %11000000
    .byte %11000000
    .byte %10000000
    .byte %00000000
    .byte %00000000

q_sugar_matrix_background
    .byte %00000010,%10101010,%10101010,%10100000
    .byte %00000010,%10101010,%10101010,%10000000
    .byte %00000010,%10101010,%10101010,%10000000
    .byte %00000010,%10101010,%10101010,%10000000
    .byte %00001010,%10101010,%10101010,%10000000
    .byte %00001010,%10101010,%10101010,%10100000
    .byte %00001010,%10101010,%10101010,%10100000
    .byte %00001010,%10101010,%10101010,%10100000
    .byte %00001010,%10101010,%10101010,%10100000
    .byte %00001010,%10101010,%10101010,%10100000
    .byte %00001010,%10101010,%10101010,%10100000
    .byte %00000010,%10101010,%10101010,%10100000
    .byte %00000010,%10101010,%10101010,%10100000
    .byte %00000010,%10101010,%10101010,%10000000
    .byte %00000010,%10101010,%10101010,%10000000
    .byte %00000010,%10101010,%10101010,%10000000

q_sugar_heart
    .byte %00000011,%11110000,%00111111,%00000000
    .byte %00111111,%11111111,%11111111,%11110000
    .byte %11111010,%11101111,%10111010,%11111100
    .byte %11111110,%11101011,%10111011,%11111100
    .byte %00111111,%11111111,%11111111,%11110000
    .byte %00000011,%11111111,%11111111,%00000000
    .byte %00000000,%00111111,%11110000,%00000000
    .byte %00000000,%00000011,%00000000,%00000000

q_sugar_start_sound
    .byte $ca
    .byte $20,$2
    .byte $1a,$2
    .byte $14,$2
    .byte $0

q_sugar_explode_sound
    .byte $0f
    .byte $20,$2
    .byte $fe,$04
    .byte $20,$4
    .byte $fe,$02
    .byte $30,$6
    .byte $fe,$01
    .byte $40,$6
    .byte $0

q_sugar_portrait
    include "q_sugar_portrait.s"

q_sugar_portrait_color_1 equ $0 ; 0, 0, 0
q_sugar_portrait_color_2 equ $3e ; 255, 198, 181
q_sugar_portrait_color_3 equ $4a ; 255, 107, 181
q_sugar_portrait_color_8 equ $f0 ; 8, 0, 0

q_sugar_portrait_table
    .byte 5 + 128
    .byte q_sugar_portrait_color_1
    .byte 6 + 128
    .byte q_sugar_portrait_color_3
    .byte 4
    .byte q_sugar_portrait_color_1
    .byte q_sugar_portrait_color_3
    .byte q_sugar_portrait_color_1
    .byte q_sugar_portrait_color_1
    .byte 3 + 128
    .byte q_sugar_portrait_color_2
    .byte 1
    .byte q_sugar_portrait_color_1
    .byte 22 + 128
    .byte q_sugar_portrait_color_2
    .byte 13 + 128
    .byte q_sugar_portrait_color_1
    .byte 3
    .byte q_sugar_portrait_color_2
    .byte q_sugar_portrait_color_2
    .byte q_sugar_portrait_color_1
    .byte 3 + 128
    .byte q_sugar_portrait_color_2
    .byte 5 + 128
    .byte q_sugar_portrait_color_3
    .byte 6 + 128
    .byte q_sugar_portrait_color_1
    .byte 4
    .byte q_sugar_portrait_color_3
    .byte q_sugar_portrait_color_1
    .byte q_sugar_portrait_color_3
    .byte q_sugar_portrait_color_2
    .byte 3 + 128
    .byte q_sugar_portrait_color_1
    .byte 1
    .byte q_sugar_portrait_color_2
    .byte 4 + 128
    .byte q_sugar_portrait_color_1
    .byte 17 + 128
    .byte q_sugar_portrait_color_3
    .byte 1
    .byte q_sugar_portrait_color_1
    .byte 13 + 128
    .byte q_sugar_portrait_color_2
    .byte 8
    .byte q_sugar_portrait_color_1
    .byte q_sugar_portrait_color_1
    .byte q_sugar_portrait_color_2
    .byte q_sugar_portrait_color_1
    .byte q_sugar_portrait_color_1
    .byte q_sugar_portrait_color_3
    .byte q_sugar_portrait_color_1
    .byte q_sugar_portrait_color_8
    .byte 3 + 128
    .byte q_sugar_portrait_color_1
    .byte 9 + 128
    .byte q_sugar_portrait_color_2
    .byte 9 + 128
    .byte q_sugar_portrait_color_3
    .byte 17 + 128
    .byte q_sugar_portrait_color_1
    .byte 19 + 128
    .byte q_sugar_portrait_color_3
    .byte 1
    .byte q_sugar_portrait_color_1

