; This file is part of Magical Fairy Force.

; Magical Fairy Force is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Magical Fairy Force is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Magical Fairy Force.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2020 Average Software.

; Miscellaneous startup routines

; One-time initialization upon power up.
init_system subroutine
    jsr     clear_ram               ; Clear RAM and hardware.
    jsr     clear_hardware
    jsr     init_common_vectors     ; Set system vectors.
    jmp     init_common_registers   ; Set system registers, tail call.

; Blank the page of RAM in $ff.
clear_page subroutine
    mva     #0,$fe                  ; Set the low byte to 0.
    tay                             ; Set the index to 0.
.loop
    sta     ($fe),y
    iny
    bne     .loop                   ; Jump back if the page isn't done.
    rts

; Clear page 0 while avoiding
; reserved memory locations.
safe_clear_page_0 subroutine
    ldy     #$19                    ; First non-reserved byte.
    lda     #0
.loop
    sta     0,y
    iny
    bne     .loop
    rts

; Clear all memory, excluding the stack page.
clear_ram subroutine
    ldx     #0                      ; Clear page 0.
    stx     $ff
    jsr     clear_page
    ldx     #2                      ; Start on page 2.
.loop
    stx     $ff
    jsr     clear_page
    inx
    cpx     #$40                    ; Quit upon reaching $4000.
    bne     .loop
    rts

; Blanks the memory pages for GTIA, ANTIC, and POKEY.
clear_hardware subroutine
    mva     #$c0,$ff                ; Clear GTIA.
    jsr     clear_page
    mva     #$d4,$ff                ; Clear ANTIC.
    jsr     clear_page
    mva     #$e8,$ff                ; Clear POKEY.
    jmp     clear_page              ; Tail call.

; Initialize the system vectors that are common to all modes.
init_common_vectors subroutine
    mwa     IRQINT,VIMIRQ           ; Set the main IRQ vector.
    mwa     VBIINT,VVBLKI           ; Set the VBI vector.
    mwa     KBDINT,VKYBDI           ; Set the keyboard IRQ vector.
    mwa     keypad_interrupt,VKYBDF ; Set the deferred keyboard vector.
    mwa     ENDINT,VVBLKD           ; Set the deferred VBI vector.
    rts

; Initialize the various registers that are common to all modes.
init_common_registers subroutine
    mva     #$40,sIRQEN             ; Enable keyboard interrupts.
    sta     IRQEN             
    mva     #$3e,sDMACTL            ; Enable DMA.
    sta     DMACTL            
    mva     #$f8,CHBASE             ; Set the character base.
    mva     #$10,PMBASE             ; Set the player/missile address.
    mva     #$03,GRACTL             ; Enable player/missiles.
    mva     #4,CONSOL               ; Enable POT scanning.
    mva     #2,SKCTL                ; Enable keypad scanning.
    rts

