; This file is part of Magical Fairy Force.

; Magical Fairy Force is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Magical Fairy Force is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Magical Fairy Force.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2020 Average Software.

; Title screen logic

start_title_screen subroutine
    lda     #<null_display_list
    sta     sDLISTL
    sta     DLISTL
    lda     #>null_display_list
    sta     sDLISTH
    sta     DLISTH
    mwa     ENDINT,VVBLKD           ; Shut down the vblank interrupt.
    jsr     create_text_display_list
    lda     #$07                    ; Insert one big text line.
    sta     $808
    lda     #0
    sta     sDLISTL                 ; Set up the display list.
    sta     DLISTL
    lda     #$8
    sta     sDLISTH
    sta     DLISTH
    lda     #$4e
    sta     sCOLOR1
    sta     COLPF1
    lda     #$0f
    sta     sCOLOR0
    sta     COLPF0
    lda     #0
    sta     sCOLBK                  ; Black background.
    sta     COLBK
    sta     sCOLOR2
    sta     COLPF2
    mwa     title_text,$fc
    mwa     $2078,$fe
    jsr     copy_text_full
    mwa     copyright_text,$fc
    mwa     $235c,$fe
    jsr     copy_text_full
    mwa     game_select_text,$fc
    mwa     $21cc,$fe
    jsr     copy_text_full
    mwa     $21f4,$fe
    jsr     copy_text_full
    inc     $21fa
    inc     $2200
    lda     #115                    ; 's' character.
    sta     $2208
    mwa     view_license_text,$fc
    mwa     $2334,$fe
    jsr     copy_text_full
    mwa     gpl_text,$fc
    mwa     $3000,$fe               ; Copy the license text.
    ldx     #22                     ; 22 lines.
.license_loop
    jsr     copy_text_full
    dex
    beq     .license_done
    iny                             ; Y has the index of the terminator.
    tya
    clc
    adc     $fc
    sta     $fc
    bcc     .no_carry
    inc     $fd
.no_carry
    jsr     next_playfield_row
    jmp     .license_loop
.license_done
    lda     #0
    sta     sDLISTL
    lda     #$8
    sta     sDLISTH
    mva     #4,CONSOL               ; Activate player 1 controller.
    mwa     title_vblank,VVBLKD
    mva     #$ff,last_key_pressed
    jsr     hide_all_sprites        ; Move players and missiles off screen.
    cli
    mva     #$c0,NMIEN
    ; Fall through to the title loop.

title_screen_loop subroutine
    jsr     wait_for_vb
    lda     last_key_pressed
    bmi     title_screen_loop
    cmp     #$01
    beq     .start_game
    cmp     #$02
    beq     .start_game
    cmp     #$0b
    bne     title_screen_loop
    lda     #$ff
    sta     last_key_pressed
    lda     #$30                    ; Alter the display list to show
    sta     $805                    ; the license text.
    lda     #$02
    sta     $808
    jmp     license_loop
.start_game
    sta     game_mode
    cmp     #2
    beq     .two_player
    ldy     #0                      ; Set up the one player schedule.
    sty     schedule_progress
    sty     $e0                     ; Start building a character pool.
    iny
    sty     $e1
    iny
    sty     $e2
    iny
    sty     fight_schedule + 6      ; Kitty is always in slot 7.
    iny
    sty     $e3
    iny
    sty     $e4
    iny
    sty     $e5
    sty     $d0                     ; Remaining characters. (6)
    iny
    sty     fight_schedule + 7      ; Zevasha is usually last.
    ldx     #0                      ; Index into the schedule.
.schedule_loop
    lda     RANDOM                  ; Get a random position.
    and     #7                      ; Mask down to a reasonable range.
.range_check
    cmp     $d0                     ; Check against the remaining numbers.
    bcc     .index_ok
    lsr                             ; Shift and check again.
    jmp     .range_check
.index_ok
    sta     $d1                     ; Save the read index.
    tay                             ; Index into the remaining picks.
    lda     $e0,y
    sta     fight_schedule,x        ; Add to the list.
    dec     $d0                     ; See if it's done.
    beq     .two_player
    inx                             ; Next write index.
    ldy     $d0                     ; Move the last number to the
    lda     $e0,y                   ; position that was just used.
    ldy     $d1
    sta     $e0,y
    jmp     .schedule_loop
.two_player
    lda     #0                      ; Normal character select.
    sta     other_screen_active
    jsr     wait_for_vb
    jmp     start_character_select

; Create an all text display list.
create_text_display_list subroutine
    lda     #$70
    sta     $800
    sta     $801
    sta     $802
    lda     #$42
    sta     $803
    lda     #0
    sta     $804
    lda     #$20
    sta     $805
    ldy     #21                     ; Do 22 mode 2 lines.
    lda     #$02
.dlist_loop
    sta     $806,y
    dey
    bpl     .dlist_loop
    lda     #$41                    ; Wait for vblank and jump.
    sta     $81c
    lda     #0
    sta     $81d
    lda     #$8
    sta     $81e
    ldx     #32                     ; Blank the text RAM.
    lda     #$20
    sta     $ff
.clear_ram_loop
    jsr     clear_page
    inc     $ff
    dex
    bne     .clear_ram_loop
    rts

license_loop subroutine
    jsr     wait_for_vb
    lda     last_key_pressed
    cmp     #$0b
    bne     license_loop
    lda     #$ff
    sta     last_key_pressed
    lda     #$20                    ; Restore the display list.
    sta     $805
    lda     #$07
    sta     $808
    jmp     title_screen_loop

; Vblank handler.
title_vblank subroutine
    jsr     process_sounds
    lda     #0
    sta     ATRACT
    ldy     key_debounce
    beq     .no_debounce
    dec     key_debounce
.no_debounce
    jmp     ENDINT

