; This file is part of Magical Fairy Force.

; Magical Fairy Force is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Magical Fairy Force is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Magical Fairy Force.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2020 Average Software.

; Pre-fight versus screen

; The versus screen has two entry points.
; One is for the character select screen,
; which can reuse the display list and the
; portraits are already in place.

; Constants.
versus_sprite_base  equ #120

; Data.
bright_color    equ $50
dark_color      equ $51
time_to_game    equ $52

start_versus_screen subroutine
    lda     #1                      ; We reuse the vblank interrupt
    sta     other_screen_active     ; but need to cut parts out.
    jsr     hide_all_sprites
    jsr     clear_pm_and_matrix
    lda     #$78                    ; Copy the versus sprites.
    sta     $fe
    lda     #$17
    sta     $ff
    ldx     #99                     ; Read index.
.pm_outer_loop
    ldy     #19                     ; Write index.
.pm_inner_loop
    lda     versus_sprites,x
    sta     ($fe),y
    dex
    bmi     .done
    dey
    bpl     .pm_inner_loop
    dec     $ff
    bne     .pm_outer_loop          ; Always true.
.done
    lda     #versus_sprite_base
    sta     HPOSM0
    sta     m0_matrix_pos_low       ; Appease the interrupts.
    sta     HPOSM2
    sta     m2_matrix_pos_low
    lda     #versus_sprite_base + 2
    sta     HPOSM1
    sta     m1_matrix_pos_low
    sta     HPOSM3
    sta     m3_matrix_pos_low
    lda     #versus_sprite_base + 4
    sta     HPOSP0
    sta     p0_matrix_pos_low
    sta     HPOSP2
    sta     p2_matrix_pos_low
    lda     #versus_sprite_base + 12
    sta     HPOSP1
    sta     p1_matrix_pos_low
    sta     HPOSP3
    sta     p3_matrix_pos_low
    lda     #$0f
    sta     bright_color
    lda     #$04
    sta     dark_color
    lda     #180
    sta     time_to_game
    ; Fall through to the screen loop.

versus_screen_loop
    jsr     wait_for_vb
    dec     time_to_game
    bne     .continue
    jmp     start_game
.continue
    ldy     bright_color
    iny
    iny
    sty     bright_color
    sty     sCOLPM0
    sty     sCOLPM1
    ldy     dark_color
    iny
    iny
    sty     dark_color
    sty     sCOLPM2
    sty     sCOLPM3
    jmp     versus_screen_loop

; Clear the P/M graphics and matrix zone.
clear_pm_and_matrix subroutine
    ldx     #6
    lda     #8                      ; Blank pages 8 through c to erase
    sta     $ff                     ; the character matrix.
.matrix_erase_loop
    jsr     clear_page
    inc     $ff
    dex
    bne     .matrix_erase_loop
    ldx     #$13                    ; Clear the player/missiles.
.pm_clear_loop
    stx     $ff
    jsr     clear_page
    inx
    cpx     #$18
    bne     .pm_clear_loop
    rts

