; This file is part of Magical Fairy Force.

; Magical Fairy Force is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; Magical Fairy Force is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with Magical Fairy Force.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2020 Average Software.

; Xorex character data and logic routines

; Xorex's attack flow:
; For 199 frames, ice spikes will spawn from the top of
; the screen.  These will spawn every six frames, and
; they're six pixels high, so there will be only one of
; them potentially spawning at any given time.

; After 199 frames, 100 frames must pass for the screen
; to clear out.  At that point the attack will end.

; All movement will rely on the existing scrolling code
; in the deferred vertical blank routine.

; Attack constants.
xorex_attack_cost           equ -max_fairy_power * 3 / 4
xorex_spawn_length          equ 199
xorex_attack_finish_length  equ 100
xorex_spawn_phase           equ 0
xorex_attack_finish_phase   equ 1

; Attack block data offsets.
xorex_tick_count        equ 0
xorex_attack_phase      equ 1
xorex_spawning_index    equ 2
xorex_spawn_offset      equ 3

xorex_start_attack subroutine
    beq     .attack_player_0
    ldy     #2
    sty     p1_cloud_timer          ; Suppress cloud spawns.
    sty     p0_attack_active
    ldx     #xorex_attack_cost
    jsr     apply_p0_fairy_power
    ldy     #0                      ; Start with phase 0.
    sty     p0_attack_data + xorex_attack_phase
    ldy     #6
    sty     p0_attack_data + xorex_spawn_offset
    ldy     #xorex_spawn_length
    sty     p0_attack_data + xorex_tick_count
    ldy     #$0f
    sty     p1_playfield_0_color
    jsr     erase_all_player_1_clouds
    jmp     start_attack_return
.attack_player_0
    ldy     #2
    sty     p0_cloud_timer          ; Suppress cloud spawns.
    sty     p1_attack_active
    ldx     #xorex_attack_cost
    jsr     apply_p1_fairy_power
    ldy     #0                      ; Start with phase 0.
    sty     p1_attack_data + xorex_attack_phase
    ldy     #6
    sty     p1_attack_data + xorex_spawn_offset
    ldy     #xorex_spawn_length
    sty     p1_attack_data + xorex_tick_count
    ldy     #$0f
    sty     p0_playfield_0_color
    jsr     erase_all_player_0_clouds
    jmp     start_attack_return

xorex_process_attack subroutine
    bne     xorex_attack_player_1
    jmp     xorex_attack_player_0

xorex_attack_player_1 subroutine
    ldy     #2                      ; Suppress cloud spawns.
    sty     p1_cloud_timer
    dey                             ; Scroll every frame.
    sty     p1_scroll_flag
    lda     p1_scroll_base + 1      ; Erase the previous line.  If
    bne     .subtract_erase         ; the address is not $2000, the
    lda     p1_scroll_base + 2      ; address is found by subtracting
    cmp     #$20                    ; 40.  Otherwise the address is
    bne     .subtract_erase         ; $2e88.
    mwa     $2e88,$fe
    jmp     .erase_address_ready
.subtract_erase
    lda     p1_scroll_base + 1
    sec
    sbc     #40
    sta     $fe
    lda     p1_scroll_base + 2
    sbc     #0
    sta     $ff
.erase_address_ready
    lda     #0
    jsr     set_playfield_row
    lda     pf_high                 ; Check for hits.
    beq     .no_hit
    jsr     damage_player_1
.no_hit
    dec     p0_attack_data + xorex_tick_count
    lda     p0_attack_data + xorex_attack_phase
    beq     .spawn_phase
    lda     p0_attack_data + xorex_tick_count
    bne     .done                   ; This should just return.
    jmp     end_player_0_attack
.spawn_phase
    lda     p0_attack_data + xorex_tick_count
    beq     .spawn_phase_done
    lda     p0_attack_data + xorex_spawn_offset
    cmp     #6                      ; Spawn a new spike on offset 6.
    bne     .draw_spikes
    mwa     xorex_ice_sound,$fe
    jsr     play_sound_3
    mwa     p0_attack_data + xorex_spawning_index,$fe
    jsr     get_player_0_random     ; Get a random index.
    and     #$3f                    ; Get close to 39.
    jsr     xorex_find_spawn_position
    sta     p0_attack_data + xorex_spawning_index
    lda     #0
    sta     p0_attack_data + xorex_spawn_offset
.draw_spikes
    lda     p0_attack_data + xorex_spawning_index
    sta     $fd                     ; Save the offset.
    lda     p1_cloud_address        ; Set up the base address
    sta     $fe                     ; for the spawn line.
    lda     p1_cloud_address + 1
    sta     $ff
    ldy     p0_attack_data + xorex_spawn_offset
    lda     xorex_ice_spike,y
    ldy     $fd                     ; Get the offset back.
    sta     ($fe),y                 ; Write the byte.
    inc     p0_attack_data + xorex_spawn_offset
.done
    jmp     p0_process_return
.spawn_phase_done
    inc     p0_attack_data + xorex_attack_phase
    lda     #xorex_attack_finish_length
    sta     p0_attack_data + xorex_tick_count
    jmp     p0_process_return

xorex_attack_player_0 subroutine
    ldy     #2                      ; Suppress cloud spawns.
    sty     p0_cloud_timer
    dey                             ; Scroll every frame.
    sty     p0_scroll_flag
    lda     p0_scroll_erase
    sta     $fe
    lda     p0_scroll_erase + 1
    sta     $ff
    lda     #0
    jsr     set_playfield_row
    lda     pf_low                  ; Check for hits.
    beq     .no_hit
    jsr     damage_player_0
.no_hit
    dec     p1_attack_data + xorex_tick_count
    lda     p1_attack_data + xorex_attack_phase
    beq     .spawn_phase
    lda     p1_attack_data + xorex_tick_count
    bne     .done                   ; This should just return.
    jmp     end_player_1_attack
.spawn_phase
    lda     p1_attack_data + xorex_tick_count
    beq     .spawn_phase_done
    lda     p1_attack_data + xorex_spawn_offset
    cmp     #6                      ; Spawn a new spike on offset 6.
    bne     .draw_spikes
    mwa     xorex_ice_sound,$fe
    jsr     play_sound_4
    mwa     p1_attack_data + xorex_spawning_index,$fe
    jsr     get_player_1_random     ; Get a random index.
    and     #$3f                    ; Get close to 39.
    jsr     xorex_find_spawn_position
    sta     p1_attack_data + xorex_spawning_index
    lda     #0
    sta     p1_attack_data + xorex_spawn_offset
.draw_spikes
    lda     p1_attack_data + xorex_spawning_index
    sta     $fd                     ; Save the offset.
    lda     p0_scroll_base + 1      ; Set up the base address
    sta     $fe                     ; for the spawn line.
    lda     p0_scroll_base + 2
    sta     $ff
    ldy     p1_attack_data + xorex_spawn_offset
    lda     xorex_ice_spike,y
    ldy     $fd                     ; Get the offset back.
    sta     ($fe),y                 ; Write the byte.
    inc     p1_attack_data + xorex_spawn_offset
.done
    jmp     p1_process_return
.spawn_phase_done
    inc     p1_attack_data + xorex_attack_phase
    lda     #xorex_attack_finish_length
    sta     p1_attack_data + xorex_tick_count
    jmp     p1_process_return

; Locate a suitable spawn position for the next spike.
; Expects the player's random number in the accumulator
; and the address of the active indices array in $fe-$ff.
; Result will be in the accumulator.
xorex_find_spawn_position subroutine
    cmp     #40                     ; If the position is 40+,
    bmi     .in_bounds              ; drop it into range,
    sec
    sbc     #40
.in_bounds
    ldy     #0
    cmp     ($fe),y                 ; Compare to the last index.
    bne     .index_ok
    clc
    adc     #1                      ; Increment to the next position.
    cmp     #40                     ; On 40, wrap back to 0.
    bne     .index_ok
    lda     #0
.index_ok
    rts

xorex_cpu subroutine
    lda     p1_scroll_base + 1
    clc                             ; Find a playfield row to check
    adc     #$b0                    ; for hazards.
    sta     $fe
    lda     p1_scroll_base + 2
    adc     #4
    sta     $ff
    cmp     #$2e
    bcc     .no_wrap
    bne     .wrap
    lda     $fe
    cmp     #$b0
    bcc     .no_wrap
.wrap
    lda     $fe
    sec
    sbc     #$b0
    sta     $fe
    lda     $ff
    sbc     #$e
    sta     $ff
.no_wrap
    lda     p1_real_hpos            ; Find the playfield bytes that
    sec                             ; correspond to the player.
    sbc     #left_bound
    lsr                             ; Divide by four.
    lsr
    tay
    inc     cpu_state               ; Assume a dodge.
    lda     #p1_top_bound           ; Head for the top.
    sta     cpu_ydest
    lda     ($fe),y                 ; Look for non-0 bytes.
    bne     .dodge_right
    iny
    lda     ($fe),y
    bne     .dodge_random
    iny
    lda     ($fe),y
    bmi     .dodge_left
    dec     cpu_state
    lda     #0
    sta     p1_h_input
    sta     p1_v_input
    rts                             ; Remain in place, clear.
.dodge_random
    lda     cpu_up_speed
    sta     cpu_vspeed
    lda     RANDOM                  ; Pick a random direction.
    bmi     .dodge_left
.dodge_right
    lda     p1_real_hpos
    clc
    adc     #12
    sta     cpu_xdest
    lda     cpu_right_speed
    sta     cpu_hspeed
    rts
.dodge_left
    lda     p1_real_hpos
    sec
    sbc     #12
    sta     cpu_xdest
    lda     cpu_left_speed
    sta     cpu_hspeed
    rts

; Sprite images.
xorex_p1_sprite
    .byte       %00011000
    .byte       %00011000
    .byte       %00111000
    .byte       %00111000
    .byte       %00111000
    .byte       %00000000
    .byte       %00011000
    .byte       %00111100
    .byte       %00111100
    .byte       %00111100
    .byte       %00011000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00010000
    .byte       %00110110
    .byte       %00100010

xorex_p2_sprite
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000001
    .byte       %00011001
    .byte       %00000000
    .byte       %00000000
    .byte       %10000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00010100
    .byte       %00010100
    .byte       %00000100
    .byte       %00000000
    .byte       %00000000

xorex_p3_sprite
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000001
    .byte       %00000011
    .byte       %00000010
    .byte       %11000000
    .byte       %01000000
    .byte       %00011000
    .byte       %00011100
    .byte       %00111100
    .byte       %00111100
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000
    .byte       %00000000

; Letter data for the select screen.
; See pink_b.s for details.
xorex_letter_data
    .word       big_letter_x,$3000,$29ce
    .byte       big_letter_x_width
    .word       big_letter_o,$3002,$29d0
    .byte       big_letter_o_width
    .word       big_letter_r,$3004,$29d2
    .byte       big_letter_r_width
    .word       big_letter_e,$3006,$29d4
    .byte       big_letter_e_width
    .word       big_letter_x,$3008,$29d6
    .byte       big_letter_x_width

xorex_game_row_offset       equ 8
xorex_game_portrait_offset  equ 59 - xorex_game_row_offset

xorex_matrix_data
    .byte 116,124,0,0       ; Player positions.
    .byte 0,0,0,0           ; Missile positions.
    .byte $fc,$fc,$00,$00   ; Colors.

xorex_matrix_image equ xorex_matrix_image_real_start - 16
xorex_matrix_image_real_start
    .byte %00010111         ; Player 0.
    .byte %00110101
    .byte %01101101
    .byte %01010011
    .byte %01100011
    .byte %01111111
    .byte %01111111
    .byte %01111101
    .byte %01111101
    .byte %00111100
    .byte %00111100
    .byte %00011111
    .byte %00011000
    .byte %00011110
    .byte %00001100
    .byte %00001111

    .byte %11111010         ; Player 1.
    .byte %11011110
    .byte %10000000
    .byte %11001000
    .byte %10100000
    .byte %11000010
    .byte %11111000
    .byte %11111010
    .byte %11111100
    .byte %11111100
    .byte %11111000
    .byte %11111000
    .byte %00110000
    .byte %01110000
    .byte %11100000
    .byte %11000000

xorex_matrix_background
    .byte %00000010,%10101010,%10101010,%10101010
    .byte %00101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %10101010,%10101010,%10101010,%10101010
    .byte %00101010,%10101010,%10101010,%10101010
    .byte %00101010,%10101010,%10101010,%10101000
    .byte %00101010,%10101010,%10101010,%10101000
    .byte %00101010,%10101010,%10101010,%10100000
    .byte %00001010,%10101010,%10101010,%10100000
    .byte %00001010,%10101010,%10101010,%10100000
    .byte %00001010,%10101010,%10101010,%10100000
    .byte %00000010,%10101010,%10101010,%10100000
    .byte %00000010,%10101010,%10101010,%10100000

xorex_ice_spike
    .byte %00010000
    .byte %00010100
    .byte %00010100
    .byte %00010101
    .byte %01010101
    .byte %01010101

xorex_ice_sound
    .byte $e1
    .byte $30,$1
    .byte $28,$1
    .byte $0

xorex_portrait
    include "xorex_portrait.s"

xorex_portrait_color_1 equ $0 ; 0, 0, 0
xorex_portrait_color_2 equ $3e ; 255, 198, 181
xorex_portrait_color_3 equ $3c ; 255, 165, 148
xorex_portrait_color_4 equ $2e ; 255, 222, 115
xorex_portrait_color_5 equ $54 ; 140, 0, 148
xorex_portrait_color_6 equ $e ; 239, 239, 239

xorex_portrait_table
    .byte 1
    .byte xorex_portrait_color_5
    .byte 11 + 128
    .byte xorex_portrait_color_1
    .byte 3
    .byte xorex_portrait_color_3
    .byte xorex_portrait_color_1
    .byte xorex_portrait_color_1
    .byte 3 + 128
    .byte xorex_portrait_color_3
    .byte 4 + 128
    .byte xorex_portrait_color_1
    .byte 8 + 128
    .byte xorex_portrait_color_3
    .byte 13 + 128
    .byte xorex_portrait_color_2
    .byte 4 + 128
    .byte xorex_portrait_color_1
    .byte 3
    .byte xorex_portrait_color_2
    .byte xorex_portrait_color_2
    .byte xorex_portrait_color_1
    .byte 4 + 128
    .byte xorex_portrait_color_2
    .byte 1
    .byte xorex_portrait_color_1
    .byte 5 + 128
    .byte xorex_portrait_color_2
    .byte 5
    .byte xorex_portrait_color_1
    .byte xorex_portrait_color_5
    .byte xorex_portrait_color_5
    .byte xorex_portrait_color_4
    .byte xorex_portrait_color_4
    .byte 7 + 128
    .byte xorex_portrait_color_3
    .byte 3
    .byte xorex_portrait_color_1
    .byte xorex_portrait_color_3
    .byte xorex_portrait_color_3
    .byte 3 + 128
    .byte xorex_portrait_color_1
    .byte 4 + 128
    .byte xorex_portrait_color_3
    .byte 8 + 128
    .byte xorex_portrait_color_1
    .byte 3 + 128
    .byte xorex_portrait_color_3
    .byte 3 + 128
    .byte xorex_portrait_color_1
    .byte 5 + 128
    .byte xorex_portrait_color_3
    .byte 2
    .byte xorex_portrait_color_1
    .byte xorex_portrait_color_1
    .byte 4 + 128
    .byte xorex_portrait_color_2
    .byte 3
    .byte xorex_portrait_color_1
    .byte xorex_portrait_color_1
    .byte xorex_portrait_color_2
    .byte 4 + 128
    .byte xorex_portrait_color_1
    .byte 1
    .byte xorex_portrait_color_2
    .byte 5 + 128
    .byte xorex_portrait_color_1
    .byte 3 + 128
    .byte xorex_portrait_color_4
    .byte 2
    .byte xorex_portrait_color_5
    .byte xorex_portrait_color_5
    .byte 21 + 128
    .byte xorex_portrait_color_4
    .byte 3 + 128
    .byte xorex_portrait_color_1
    .byte 1
    .byte xorex_portrait_color_2
    .byte 3 + 128
    .byte xorex_portrait_color_1
    .byte 3 + 128
    .byte xorex_portrait_color_3
    .byte 5 + 128
    .byte xorex_portrait_color_1
    .byte 1
    .byte xorex_portrait_color_3
    .byte 7 + 128
    .byte xorex_portrait_color_6
    .byte 11 + 128
    .byte xorex_portrait_color_4

